package Category;
#!/usr/bin/perl
use strict;
use warnings;
use Scalar::Util qw(looks_like_number);
use File::Basename;



############################################


sub class {

	my $dest_dir				= shift or die;
	my $array_stat_files		= shift or die;
	my $array_category 			= shift or die;
	my $hash_stat_category		= shift or die;
	my $array_samples			= shift or die;
	my $repeat2rRNA_stat		= shift or die;
	my $repeat2tRNA_stat		= shift or die;
	my $repeat2snRNA_snoRNA_stat= shift or die;
	my $repeat2piRNA_stat		= shift;
	my $repeat_basename			= shift;
	
	# **************************************** #


	my %hash_class = ();
	my %hash_unknown = ();

	for my $stat_file ( @$array_stat_files ){
		
		my %hash_row = ();
		&toMatrix($stat_file,\%hash_row);
		
		my $c = $hash_stat_category->{$stat_file};
		
		for my $row ( keys %hash_row ){
			for my $col ( keys %{$hash_row{$row}} ){
				unless(exists $hash_class{$c}{$col}{$row}){
					$hash_class{$c}{$col}{$row} = $hash_row{$row}{$col};
				} else {
					print STDERR "Error:($c,$row,$col) repeat\n";
					die;
				}
			}
		}		
	}


	# **************************************** #
	
	
	my @items = ();
	push(@items,"mapped_read");
	push(@items,"mapped_tag");

	my $nc = scalar @$array_category - 1;
	for my $sample ( @$array_samples ){
		my %hash_unknown = ();
		for my $c ( 1 .. $nc ){
			$c = $array_category->[$c];
			for my $item ( @items ){
				unless(exists $hash_unknown{$item}){
					$hash_unknown{$item} = $hash_class{$c}{$sample}{$item};
				} else {
					$hash_unknown{$item}+= $hash_class{$c}{$sample}{$item};
				}
			}
		}
			
		for my $item ( @items ){
			my $t = $hash_class{genome}{$sample}{$item};
			$t -= $hash_unknown{$item};
			$hash_class{"unknown"}{$sample}{$item} = $t;
		}
	}
	push(@$array_category,"unknown");	
		
	
	# **************************************** #
	
	
	my $title = "Category";
	for my $sample ( @$array_samples ){
		$title .= "\t$sample";
	}

	my $outfile = "$dest_dir/category-read_before_recall_recue.txt";
	open  CATEGORYOUT, ">$outfile" or die;
	print CATEGORYOUT "$title\n";

	for my $c ( @$array_category ){
		my $str = $c;
		for my $sample ( @$array_samples ){
			my $t = $hash_class{$c}{$sample}{mapped_read};
			$str .= "\t$t";	
		}
		print CATEGORYOUT "$str\n";
	}
	close CATEGORYOUT;	
	
	
	# **************************************** #
			
		
	$outfile = "$dest_dir/category-tag_before_recall_recue.txt";
	open  CATEGORYOUT, ">$outfile" or die;
	print CATEGORYOUT "$title\n";

	for my $c ( @$array_category ){
		my $str = $c;
		for my $sample ( @$array_samples ){
			my $t = $hash_class{$c}{$sample}{mapped_tag};
			$str .= "\t$t";	
		}
		print CATEGORYOUT "$str\n";
	}
	close CATEGORYOUT;		
		
		
	# **************************************** #
	
	
	$title = "Category\t";
	for my $sample ( @$array_samples ){
		$title .= "$sample\tread(%)\ttag\ttag(%)\t";
	}


	$outfile = "$dest_dir/category_before_recall_recue.txt";
	open  CATEGORYOUT, ">$outfile" or die;
	print CATEGORYOUT "$title\n";

	for my $c ( @$array_category ){
		my $str = $c;
		for my $sample ( @$array_samples ){
			for my $item ( @items ){
				my $t = $hash_class{$c}{$sample}{$item};
				my $p = $t/$hash_class{genome}{$sample}{$item};
				$p = &round($p,6);
				$p *= 100;
				$str .= "\t$t\t$p";	
			}
		}
		print CATEGORYOUT "$str\n";
	}
	close CATEGORYOUT;	
	
	
	# **************************************** #
	# rRNA
	# **************************************** #


	my %hash_recall = ();
	&toMatrix($repeat2rRNA_stat,\%hash_recall);


	for my $sample ( @$array_samples ){
		my $mapped_read = $hash_recall{mapped_read}{$sample};
		$hash_class{repeat}{$sample}{mapped_read} -= $mapped_read;
		$hash_class{rRNA}{$sample}{mapped_read} += $mapped_read;
	}


	for my $sample ( @$array_samples ){
		my $mapped_tag = $hash_recall{mapped_tag}{$sample};
		$hash_class{repeat}{$sample}{mapped_tag} -= $mapped_tag;
		$hash_class{rRNA}{$sample}{mapped_tag} += $mapped_tag;
	}



	# **************************************** #
	# tRNA
	# **************************************** #


	%hash_recall = ();
	&toMatrix($repeat2tRNA_stat,\%hash_recall);


	for my $sample ( @$array_samples ){
		my $mapped_read = $hash_recall{mapped_read}{$sample};
		$hash_class{repeat}{$sample}{mapped_read} -= $mapped_read;
		$hash_class{tRNA}{$sample}{mapped_read} += $mapped_read;
	}


	for my $sample ( @$array_samples ){
		my $mapped_tag = $hash_recall{mapped_tag}{$sample};
		$hash_class{repeat}{$sample}{mapped_tag} -= $mapped_tag;
		$hash_class{tRNA}{$sample}{mapped_tag} += $mapped_tag;
	}


	# **************************************** #
	# snRNA_snoRNA
	# **************************************** #


	%hash_recall = ();
	&toMatrix($repeat2snRNA_snoRNA_stat,\%hash_recall);


	for my $sample ( @$array_samples ){
		my $mapped_read = $hash_recall{mapped_read}{$sample};
		$hash_class{repeat}{$sample}{mapped_read} -= $mapped_read;
		$hash_class{"snRNA/snoRNA"}{$sample}{mapped_read} += $mapped_read;
	}


	for my $sample ( @$array_samples ){
		my $mapped_tag = $hash_recall{mapped_tag}{$sample};
		$hash_class{repeat}{$sample}{mapped_tag} -= $mapped_tag;
		$hash_class{"snRNA/snoRNA"}{$sample}{mapped_tag} += $mapped_tag;
	}


	# **************************************** #
	# piRNA
	# **************************************** #


	if ( -e $repeat2piRNA_stat ){
		
		my %hash_rescue = ();
		&toMatrix($repeat2piRNA_stat,\%hash_rescue);

		for my $sample ( @$array_samples ){
			my $s = "$repeat_basename.$sample.mapped";
			my $mapped_read = $hash_rescue{mapped_read}{$s};
			$hash_class{repeat}{$sample}{mapped_read} -= $mapped_read;
			$hash_class{piRNA}{$sample}{mapped_read} += $mapped_read;
		}

		for my $sample ( @$array_samples ){
			my $s = "$repeat_basename.$sample.mapped";
			my $mapped_tag = $hash_rescue{mapped_tag}{$s};
			$hash_class{repeat}{$sample}{mapped_tag} -= $mapped_tag;
			$hash_class{piRNA}{$sample}{mapped_tag} += $mapped_tag;
		}
	}


	# **************************************** #
	
	
	$title = "Category";
	for my $sample ( @$array_samples ){
		$title .= "\t$sample";
	}

	$outfile = "$dest_dir/category-read.txt";
	open  CATEGORYOUT, ">$outfile" or die;
	print CATEGORYOUT "$title\n";

	for my $c ( @$array_category ){
		my $str = $c;
		for my $sample ( @$array_samples ){
			my $t = $hash_class{$c}{$sample}{mapped_read};
			$str .= "\t$t";	
		}
		print CATEGORYOUT "$str\n";
	}
	close CATEGORYOUT;			
				
				
	# **************************************** #				
				
				
	$outfile = "$dest_dir/category-tag.txt";
	open  CATEGORYOUT, ">$outfile" or die;
	print CATEGORYOUT "$title\n";

	for my $c ( @$array_category ){
		my $str = $c;
		for my $sample ( @$array_samples ){
			my $t = $hash_class{$c}{$sample}{mapped_tag};
			$str .= "\t$t";	
		}
		print CATEGORYOUT "$str\n";
	}
	close CATEGORYOUT;			
				
		
	# **************************************** #
	
	
	$title = "Category\t";
	for my $sample ( @$array_samples ){
		$title .= "$sample\tread(%)\ttag\ttag(%)\t";
	}


	$outfile = "$dest_dir/category.txt";
	open  CATEGORYOUT, ">$outfile" or die;
	print CATEGORYOUT "$title\n";

	for my $c ( @$array_category ){
		my $str = $c;
		for my $sample ( @$array_samples ){
			for my $item ( @items ){
				my $t = $hash_class{$c}{$sample}{$item};
				my $p = $t/$hash_class{genome}{$sample}{$item};
				$p = &round($p,6);
				$p *= 100;
				$str .= "\t$t\t$p";	
			}
		}
		print CATEGORYOUT "$str\n";
	}
	close CATEGORYOUT;	

}


############################################


sub is_numeric {
	
    my $v = shift;

    if( looks_like_number( $v ) ){  	
        return 1;
    } else {
        return 0;
    }
    
}


############################################


sub round {
	
	my $usage = "<  numerical value > < the numbers after the decimal point >";
    my $val = shift;
    my $col = shift;
    
    unless(defined $val){
    	die $usage;
    }
    
    unless(defined $col){
    	die $usage;
    }
    
    unless( &is_numeric($val) ){
    	print STDERR "Error:$val not a numeric";
		die;
	}
	
	unless( &is_numeric($col) ){
    	print STDERR "Error:$col not a numeric";
		die;
	}
    
    my $r = 10 ** $col;
    my $a = ($val > 0) ? 0.5 : -0.5;
    
    return int($val * $r + $a) / $r;
    
}


############################################


sub toMatrix {
	
	my $infile 	 = shift or die;
	my $hash_row = shift or die;

	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	
	my $flag = 1;
	my @columns = ();
	my %hash_col_tmp = ();
	
	while( defined( my $line = <IN> )){
		chomp($line);
		next if $line =~/^\s*$/;
		
		my @tmp = split/\t/,$line;
		my $row = shift @tmp;
		
		if ( $flag ) {
			for my $i ( 0 .. $#tmp ){		
				unless( exists $hash_col_tmp{ $i } ){
					$hash_col_tmp{ $i } = $tmp[$i];
				}	
				push(@columns,$tmp[$i]);		
			}
			$flag = 0;
			next;
		} 
		
		for my $i ( 0 .. $#tmp ){
			my $col = $hash_col_tmp{ $i };
			unless( exists $hash_row->{ $row }{ $col } ){	
				$hash_row->{ $row }{ $col } = $tmp[$i];
			} else {
				print STDERR "Error:($row,$col) repeat\n";
				die;
			}
		}
	}
	close IN;
	
	return @columns;
	
}

############################################
1;