package Libsvm;
#!/usr/bin/perl
use strict;
use warnings;
use Cwd;
use File::Basename;

# My Library
use MyMath;
use MyFile;





# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================	

sub ensemble_train {
	
	my $libsvm_path  	= shift or die;
	my $hash_feature 	= shift or die;
	my $array_cv 		= shift or die;
	
	
	# ****************************************************** #
	
	my $cv 			= 4;
	my $kernel		= 2;
	my $grid_n_fold = 5;
	my $round_num 	= 1;
	my $hash_model;
	my $hash_SVM_model;
	
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/cv/i) {$cv=shift @_}
		if ($argument=~/kernel/i) {$kernel=shift @_}
		if ($argument=~/grid_n_fold/i) {$grid_n_fold=shift @_}
		if ($argument=~/round/i) {$round_num=shift @_}
		if ($argument=~/hash_model/i) {$hash_model=shift @_}
		if ($argument=~/hash_SVM_model/i) {$hash_SVM_model=shift @_}
	}
	
	
	
	# ****************************************************** #
	
	for my $m ( sort {$a<=>$b} keys %$hash_feature ){
		
		# ****************************************************** #
		# k-fold cross-validation 
		# ****************************************************** #
		
		my $input_feature = $hash_feature->{$m}{feature};	
		unless(-e $input_feature){
			print STDERR "Error:$input_feature not exists\n";
			die;
		}
		
		my $dir = dirname($input_feature);
		
		if ( @$array_cv ){
			
			#################################################
			for my $cv ( sort{$a <=> $b} @$array_cv ) {
			#################################################
				
				my $cv_dir = "$dir/cv-$cv";
				MyFile::makeDir($cv_dir);
				
				my $outfile_roc = "$cv_dir/roc.txt";
				my $outfile_out = "$cv_dir/out.txt";
				
				Libsvm::cross_validation($libsvm_path,$input_feature,$outfile_roc,
				$outfile_out,$cv_dir,'cv'=>$cv,'kernel'=>$kernel,'round'=>$round_num);
				
				
			#####################	
			} # End k-fold cv
			#####################
			
		}
		
	
		# ****************************************************** #
		# train model 
		# ****************************************************** #
		
		my $train_dir = "$dir/train";
		MyFile::makeDir($train_dir);
		
		my $svm_model 	 = "$train_dir/svm_model.txt";
		my $scaled_model = "$train_dir/scaled_model.txt";
		
		&training($libsvm_path,$input_feature,$svm_model,
		$scaled_model,$train_dir,'cv'=>$grid_n_fold,'kernel'=>$kernel);
		
		
		my $feat_model = "$dir/feature_model.txt";
		if (-e $feat_model){
			if ( defined $hash_model ){
				$hash_model->{$svm_model} = $feat_model;
			}
		}
		
		
		if ( defined $hash_SVM_model ){
			$hash_SVM_model->{$svm_model} = $scaled_model;
		}
			
		
		# ****************************************************** #
		
	}
	
}
		
		


# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================	

sub aggregation {
	
	my $libsvm_path  	= shift or die;
	my $hash_feature 	= shift or die;
	my $array_cv 		= shift or die;
	my $dest_dir		= shift or die;
		
	my $cv 			= 4;
	my $kernel		= 2;
	my $grid_n_fold = 5;
	my $round_num 	= 1;

	while (@_) {
		my $argument = shift @_;
		if ($argument=~/cv/i) {$cv=shift @_}
		if ($argument=~/kernel/i) {$kernel=shift @_}
		if ($argument=~/grid_n_fold/i) {$grid_n_fold=shift @_}
		if ($argument=~/round/i) {$round_num=shift @_}
	}
		
	
	
	print STDERR "\n\n####### aggregate models #######\n\n";
	
	
	my $model_n = 0;
	my %hash_type = ();
	my %hash_score = ();
	
	for my $svm_model ( sort keys %$hash_feature ){
		
		$model_n++;
		
		my $dir = "$dest_dir/model$model_n";
		MyFile::makeDir($dir);
		
		# ==================== Predict ====================
			
		my @tmp = split /\t/,$hash_feature->{$svm_model};
		my $scaled_model = shift @tmp;
		my $infile_data  = shift @tmp;
		
		unless(-e $scaled_model){
			print STDERR "Error:$scaled_model not exists\n";
			die;
		}
		
		unless(-e $infile_data){
			print STDERR "Error:$infile_data not exists\n";
			die;
		}
		
	
		my %hash_tmp = ();
		$hash_tmp{$svm_model} = $scaled_model;
		
		my $outfile_out = "$dir/out.txt";
		my $outfile_roc = "$dir/roc.txt";
		
		&predict($libsvm_path,\%hash_tmp,$infile_data,$outfile_out,$outfile_roc,$dir);
		
		# ==================== Parse ====================
		
		my %hash_row = ();
		MyMath::toMatrix($outfile_out,\%hash_row);
		
		for my $row ( keys %hash_row ){	
			my $positive = $hash_row{$row}{"1"};
			my $negative = $hash_row{$row}{"-1"};			
			my $realType = $hash_row{$row}{inputLable};
			unless(exists $hash_type{$row}){
				$hash_type{$row} = $realType;
			}
			
			$hash_score{$row}{$model_n}{positive} = $positive;
			$hash_score{$row}{$model_n}{negative} = $negative;	
		}
	
	#############################	
	} # End 
	#############################
		
	
		
	my $title = "Name";
	for my $model ( 1 .. $model_n ){
		$title .= "\t$model-positive\t$model-negative";
	}
	
	
	my $input_feature = "$dest_dir/aggregation_feature.txt";
	open  AGGREGATION,">$input_feature" or die;
	print AGGREGATION "$title\ttype\n";
	for my $row ( sort keys %hash_score ){
		my $str = $row;
		for my $model ( sort keys %{$hash_score{$row}} ){
			my $positive = $hash_score{$row}{$model}{positive};
			my $negative = $hash_score{$row}{$model}{negative};
			$str .= "\t$positive\t$negative";
		}
		my $type = "positive";
		if ( $hash_type{$row} == -1 ){
			$type = "negative";
		}
		print AGGREGATION "$str\t$type\n";
	}
	close AGGREGATION;
	
	
	
	# ****************************************************** #
	# k-fold cross-validation 
	# ****************************************************** #
	
	
	if ( @$array_cv ){	
		for my $cv ( sort{$a <=> $b} @$array_cv ) {		
			my $cv_dir = "$dest_dir/cv-$cv";
			MyFile::makeDir($cv_dir);
			
			my $outfile_roc = "$cv_dir/roc.txt";
			my $outfile_out = "$cv_dir/out.txt";
			
			Libsvm::cross_validation($libsvm_path,$input_feature,$outfile_roc,
			$outfile_out,$cv_dir,'cv'=>$cv,'kernel'=>$kernel,'round'=>$round_num);
		} 
	}


	

	# ****************************************************** #
	# train model 
	# ****************************************************** #
	
		
	my $train_dir = "$dest_dir/train";
	MyFile::makeDir($train_dir);
	
	my $svm_model 	 = "$train_dir/svm_model.txt";
	my $scaled_model = "$train_dir/scaled_model.txt";
	
	&training($libsvm_path,$input_feature,$svm_model,
	$scaled_model,$train_dir,'cv'=>$grid_n_fold,'kernel'=>$kernel);
	
	
	return ($svm_model,$scaled_model);
	
}
	




# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================	

sub ensemble_predict {
	
	my $libsvm_path = shift or die;
	my $hash_model	= shift or die;
	my $outfile_out = shift or die;
	my $dest_dir 	= shift or die;
	
	
	
	my $aggre_model;
	my $aggre_scaled_model;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/aggregated_model/i) {$aggre_model=shift @_}
		if ($argument=~/aggre_scaled_model/i) {$aggre_scaled_model=shift @_}
	}
	
	
	my $model_n = 0;
	my %hash_type = ();
	my %hash_score = ();
	
	for my $svm_model ( sort keys %$hash_model ){
		
		$model_n++;
		
		my $dir = "$dest_dir/model$model_n";
		MyFile::makeDir($dir);
		
		# ==================== Predict ====================
			
		my @tmp = split /\t/,$hash_model->{$svm_model};
		my $scaled_model = shift @tmp;
		my $infile_data  = shift @tmp;
		
		unless(-e $scaled_model){
			print STDERR "Error:$scaled_model not exists\n";
			die;
		}
		
		unless(-e $infile_data){
			print STDERR "Error:$infile_data not exists\n";
			die;
		}
		
	
		my %hash_tmp = ();
		$hash_tmp{$svm_model} = $scaled_model;
		
		my $outfile_out = "$dir/out.txt";
		my $outfile_roc = "$dir/roc.txt";
		
		&predict($libsvm_path,\%hash_tmp,$infile_data,$outfile_out,$outfile_roc,$dir);
		
		# ==================== Parse ====================
		
		my %hash_row = ();
		MyMath::toMatrix($outfile_out,\%hash_row);
		
		for my $row ( keys %hash_row ){	
			my $positive = $hash_row{$row}{"1"};
			my $negative = $hash_row{$row}{"-1"};			
			my $realType = $hash_row{$row}{inputLable};
			unless(exists $hash_type{$row}){
				$hash_type{$row} = $realType;
			}
			
			$hash_score{$row}{$model_n}{positive} = $positive;
			$hash_score{$row}{$model_n}{negative} = $negative;	
		}
	
	#############################	
	} # End vote
	#############################
		
		
	if ((defined $aggre_model) && (-e $aggre_model) &&
		(defined $aggre_scaled_model) && (-e $aggre_scaled_model) ){	
		
		my $title = "Name";
		for my $model ( 1 .. $model_n ){
			$title .= "\t$model-positive\t$model-negative";
		}
		
		
		my $infile_data = "$dest_dir/aggregation_feature.txt";
		open  AGGREGATION,">$infile_data" or die;
		print AGGREGATION "$title\ttype\n";
		for my $row ( sort keys %hash_score ){
			my $str = $row;
			for my $model ( sort keys %{$hash_score{$row}} ){
				my $positive = $hash_score{$row}{$model_n}{positive};
				my $negative = $hash_score{$row}{$model_n}{negative};
				$str .= "\t$positive\t$negative";
			}
			my $type = "positive";
			if ( $hash_type{$row} == -1 ){
				$type = "negative";
			}
			print AGGREGATION "$str\t$type\n";
		}
		close AGGREGATION;
		
		
		# ****************************************************** #
		# Predict
		# ****************************************************** #
		
		my %hash_tmp = ();
		$hash_tmp{$aggre_model} = $aggre_scaled_model;
		
		my $outfile_roc = "$dest_dir/roc.txt";
		&predict($libsvm_path,\%hash_tmp,$infile_data,$outfile_out,$outfile_roc,$dest_dir);
		
		
	} else {
	
		open  VOTE,">$outfile_out" or die;
		print VOTE "Name\trealType\tpredictType\tPositive\tNegative\n";
		for my $row ( sort keys %hash_score ){
			my $av_proP = MyMath::round($hash_score{$row}{$model_n}{positive}/$model_n,6);
			my $av_proN = MyMath::round($hash_score{$row}{$model_n}{negative}/$model_n,6);
			my $predictType = 1;
			if ( $av_proN > $av_proP ){
				$predictType = -1;
			}
			my $realType = $hash_type{$row};
			print VOTE "$row\t$realType\t$predictType\t$av_proP\t$av_proN\n";
		}
		close VOTE;
		
	}
}
	
	
	
	


# =============================
# Function: n-fold cross-validation 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================	

sub cross_validation {
	
	my $libsvm_path = shift or die;
	my $infile_data = shift or die;
	my $outfile_roc = shift or die;
	my $outfile_out = shift or die;
	my $dest_dir	= shift or die;
	
	
	
	
	# ****************************************************** #
	
	my $cv 			= 4;
	my $kernel		= 2;
	my $grid_n_fold = 5;
	my $round_num 	= 1;
	
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/cv/i) {$cv=shift @_}
		if ($argument=~/kernel/i) {$kernel=shift @_}
		if ($argument=~/grid_n_fold/i) {$grid_n_fold=shift @_}
		if ($argument=~/round/i) {$round_num=shift @_}
	}
	
	print "\n\n############### $cv cross-validation ###############\n\n";
	
	print "\nLibsvm::cross_validation\n";
	print "cv:$cv\n";
	print "kernel:$kernel\n";
	print "cv for grid search:$grid_n_fold\n";
	print "The number of rounds:$round_num\n\n";
	
	# ****************************************************** #
	
	$libsvm_path =~ s/\/$//g;
	$libsvm_path .= "/";
		
	unless(-e $infile_data){
		print STDERR "Error:$infile_data not exists\n";
		die;
	}
	

	# ****************************************************** #
	# randomly partitioned into k equal size subsamples
	# ****************************************************** #
	

	my $infile_data_tmp = $infile_data.".tmp";
	
	open CVIN,"<$infile_data" or die;
	open CVOUT,">$infile_data_tmp" or die;

	while(defined(my $line=<CVIN>)){
		chomp($line);
		$line =~ s/^\s*//g;
		$line =~ s/\s*$//g;
	
		if ($line =~ /^\s*$/) {
			next;
		}
		
		if ($line =~ /^Name\b/i){
			next;
		}
		print CVOUT "$line\n";
	}
	close CVIN;
	close CVOUT;
		
		
	
	unlink($infile_data);
	sleep(1);
	rename($infile_data_tmp,$infile_data);
	
	

	my (@orderd_rows,@orderd_cols,%hash_row) = ();
	MyMath::toMatrix_no_title($infile_data,\%hash_row,\@orderd_rows,\@orderd_cols);
	
	my $sample_size = scalar @orderd_rows;		
	$cv = $sample_size if $cv =~ /loo/i;	
	my $subsample_size = int($sample_size/$cv);	
	$subsample_size = 1 if $subsample_size < 1;
		
	
	# ****************************************************** #
	# To reduce variability, multiple rounds of cross-validation 
	# are performed using different partitions, 
	# and the validation results are averaged over the rounds.
	# ****************************************************** #
	
	
	my $round_n = 0;
	my %hash_parse = ();
	for my $round ( 1 .. $round_num ){
		
		$round_n++;
				
		my $round_dir = "$dest_dir/round$round";
		MyFile::makeDir($round_dir);
		
		my $log_file = "$round_dir/log.txt";
		open CVLOG,">$log_file" or die;
		
		print "\n######## round:$round ########\n";
		print CVLOG "######## round:$round ########\n";
		print CVLOG "cv:$cv\n";
		print CVLOG "kernel:$kernel\n";
		print CVLOG "cv for grid search:$grid_n_fold\n";
	
		
		# ****************************************************** #
		# shuffle samples
		# ****************************************************** #
		
	
		
		MyMath::shuffle(\@orderd_rows);
		
	
		
		# ****************************************************** #
		# partitize samples
		# ****************************************************** #
		
		my $k = 1;
		my %hash_feature = ();	
		for my $i ( 0 .. $#orderd_rows ){
			$hash_feature{$k}{ $orderd_rows[$i] } = 1;		
			if ( ( ($i+1) % $subsample_size == 0 ) && ( $k < $cv ) ){
				$k++;
			}
		}
		
		print CVLOG "\nSize of subsamples:\n";
		
		my $total_sample = 0;
		for my $k ( sort {$a<=>$b} keys %hash_feature ){
			my $t = scalar keys %{$hash_feature{$k}};
			$total_sample += $t;
			print "$k:$t\n";
			print CVLOG "$k:$t\n";
		}
		
		if ( $total_sample != $sample_size ){
			print STDERR "Error:$sample_size\t$total_sample\n";
			die;
		}
		
		
		# ****************************************************** #
		# a single subsample is retained as the validtion data 
		# for testing the model, and the remaining k - 1 subsamples
		# are used as training data.
		# The cross-validation process is then repeated k times (the folds),
		# with each of the k subsamples used exactly once as the validation data.
		# The k results from the folds then can be averaged 
		# (or otherwise combined) to produce a single estimation.
		# ****************************************************** #
		
			
		my %hash_out = ();
		for my $k ( sort {$a<=>$b} keys %hash_feature ){
			
			print "\n********* $k *********\n";
			
			my $dir = "$round_dir/$k";
			MyFile::makeDir($dir);
			
			
			# ============ Test dataset =============
			
			my $test_data = "$dir/test.txt";
			open  CVOUT,">$test_data" or die;
					
			for my $row ( keys %{$hash_feature{$k}} ){
				my $str = $row;
				for my $col ( @orderd_cols ){
					$str .= "\t$hash_row{$row}{$col}";
				}
				print CVOUT "$str\n";
			}
			
			close CVOUT;
		
		
			# ============ Traning dataset =============
			
			my $training_dataset = "$dir/training.txt";
			open CVOUT,">$training_dataset" or die;	
					
			for my $n ( sort {$a<=>$b} keys %hash_feature ){
				next if $n == $k;
				for my $row ( keys %{$hash_feature{$n}} ){
					my $str = $row;	
					for my $col ( @orderd_cols ){
						$str .= "\t$hash_row{$row}{$col}";
					}
				 	print CVOUT "$str\n";
				}
			}
			
			close CVOUT;
		
			# ================ Traning =================
				
			my $train_dir = "$dir/train";
			MyFile::makeDir($train_dir);
			
			my $svm_model 	 = "$train_dir/svm_model.txt";
			my $scaled_model = "$train_dir/scaled_model.txt";
			
			&training($libsvm_path,$training_dataset,$svm_model,
			$scaled_model,$train_dir,'cv'=>$grid_n_fold,'kernel'=>$kernel);
			
			
			# ================ Test =================
			
			my $test_dir = "$dir/test";
			MyFile::makeDir($test_dir);
			
			my $outfile_out = "$test_dir/$k-out.txt";
			my $outfile_roc = "$test_dir/$k-roc.txt";
			
			my %hash_model = ();
			$hash_model{$svm_model} = $scaled_model;
			
			&predict($libsvm_path,\%hash_model,$test_data,$outfile_out,$outfile_roc,$test_dir);
		
			
			# ==================== out ==================
			
			my %hash_out_tmp = ();
			MyMath::toMatrix($outfile_out,\%hash_out_tmp);
			
			for my $row ( sort keys %hash_out_tmp ){
				for my $col ( keys %{$hash_out_tmp{$row}} ){
					unless(exists $hash_out{$row}{$col}){
						$hash_out{$row}{$col} = $hash_out_tmp{$row}{$col};
					} else {
						print STDERR "Error:($row,$col) repeat\n";
						die;
					}
				}
			}
			
		###################
		} # End k-fold
		###################
				
		my $outfile_out = "$round_dir/out.txt";
		open  CVOUT,">$outfile_out" or die;
		print CVOUT "Name\tinputLable\tpredictLable\tpositive\tnegative\n";
		
		for my $row ( sort keys %hash_out ){
			my $inputLable = $hash_out{$row}{inputLable};
			my $predictLable = $hash_out{$row}{labels};
			my $proP = $hash_out{$row}{1};
			my $proN = $hash_out{$row}{-1};
			print CVOUT "$row\t$inputLable\t$predictLable\t$proP\t$proN\n";
			
			unless(exists $hash_parse{$row}{inputLable}){
				$hash_parse{$row}{inputLable} = $inputLable;
			} 
			unless(exists $hash_parse{$row}{1}){
				$hash_parse{$row}{1} = $proP;
			} else {
				$hash_parse{$row}{1}+= $proP;
			}
			unless(exists $hash_parse{$row}{-1}){
				$hash_parse{$row}{-1} = $proN;
			} else {
				$hash_parse{$row}{-1}+= $proN;
			}
		}
		
		close CVOUT;
		close CVLOG;
		
		
		#############################
		last if $subsample_size == 1;
		#############################
		
	
	#####################	
	} # End rounds
	#####################
	
	
	# ****************************************************** #
	# the validation results are averaged over the rounds.
	# ****************************************************** #
	
	my %hash_predict = ();
	open  CVOUT,">$outfile_out" or die;
	print CVOUT "Name\tinputLable\tpredictLable\t1\t-1\n";
	
	for my $row ( sort keys %hash_parse ){		
		my $inputLable = $hash_parse{$row}{inputLable};
		my $av_proP = MyMath::round($hash_parse{$row}{1}/$round_n,6);
		my $av_proN = MyMath::round($hash_parse{$row}{-1}/$round_n,6);
		my $predictLable = 1;
		if ( $av_proP < $av_proN ){
			$predictLable = -1;
		}
		print CVOUT "$row\t$inputLable\t$predictLable\t$av_proP\t$av_proN\n";
		$hash_predict{$row}{real} = $hash_parse{$row}{inputLable};
		$hash_predict{$row}{proP} = $av_proP;
	}
	close CVOUT;
	
	
	# ================= draw ROC curve ===================
	
	my $cutoff_step = 0.001; 
	&draw_roc_curve(\%hash_predict,$outfile_roc,$cutoff_step);
	
}




# =============================	
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub training {
	
	my $libsvm_path = shift or die;
	my $infile_data = shift or die;
	my $svm_model 	= shift or die;
	my $scaled_model= shift or die;
	my $dest_dir	= shift or die;
	
	
	# ****************************************************** #
	
	my $cv = 5;
	my $kernel = 2;
	
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/cv/i) {$cv=shift @_}
		if ($argument=~/kernel/i) {$kernel=shift @_}
	}
	
	print "\nLibsvm::training\n";
	print "cv for grid search:$cv\n";
	print "kernel:$kernel\n\n";
	
	# ****************************************************** #

	$libsvm_path =~ s/\/$//g;
	$libsvm_path .= "/";
	
	
	unless(-e $infile_data){
		print STDERR "Error:$infile_data not exists\n";
		die;
	}
	
	
	# ****************************************************** #
	
	my $sample_size = 0;
	my %hash_name = ();
	my $feature_file = "$dest_dir/training.feature.txt";
	
	open TRAININ,"<$infile_data" or die;
	open FEATURE,">$feature_file" or die;
	
	
	while(defined (my $line=<TRAININ>)){
		chomp($line);
		
		$line =~ s/^\s*//g;
		$line =~ s/\s*$//g;
	
		if ($line =~ /^\s*$/) {
			next;
		}
		
		if ($line =~ /^Name\b/i){
			next;
		}
			
		my @tmp = split /\t/,$line;
		my $name = shift @tmp;
		my $type = pop @tmp;
		
		if ( $type =~ /positive/i ){
			$type = 1;
			$line = join("\t",@tmp);
			$line = "$type\t$line";
		} elsif ( $type =~ /negative/i ){
			$type = -1;
			$line = join("\t",@tmp);
			$line = "$type\t$line";
		} else {
			push(@tmp,$type);
			$line = join("\t",@tmp);
		}
		
		print FEATURE "$line\n";
		unless(exists $hash_name{$name}){
			$hash_name{$name} = 1;
		} else {
			print STDERR "Error:$name repeat in $infile_data\n";
			die;
		}
		$sample_size++;
		
	}
	
	close TRAININ;
	close FEATURE;
	
	
	# ****************************************************** #
	
	my $libsvm_file = "$dest_dir/libsvm.txt";
	&toLibsvm($feature_file,$libsvm_file);
	
	
	# ****************************************************** #
	
	my $scaled_file = "$dest_dir/scaled.txt";
	&scale_training_data($libsvm_path,$libsvm_file,$scaled_file,$scaled_model);


	# ****************************************************** #
	
	
	$cv = $sample_size if $cv =~ /loo/i;
	my $parameter_file = "$dest_dir/parameter.txt";
	my ($c,$g) = &search_parameter($libsvm_path,$scaled_file,$parameter_file,$cv,$kernel);
		
		
	# ****************************************************** #
	
	&train($libsvm_path,$scaled_file,$c,$g,$kernel,$svm_model);
	
	
	# ****************************************************** #
	
}
		


# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub predict {
	
	my $libsvm_path = shift or die;
	my $hash_model	= shift or die;
	my $infile_data = shift or die;
	my $outfile_out = shift or die;
	my $outfile_roc = shift or die;
	my $dest_dir 	= shift or die;
	
	
	# ****************************************************** #
	
	$libsvm_path  =~ s/\/$//g;
	$libsvm_path .= "/";
	
	
	unless(-e $infile_data){
		print STDERR "Error:$infile_data not exists\n";
		die;
	}
	
	
	my ($svm_model,$scaled_model);
	for my $f ( keys %$hash_model ){
		$svm_model = $f;
		$scaled_model = $hash_model->{$f};
		last;
	}
	

	$svm_model = MyFile::getAbsPath($svm_model);
	unless(-e $svm_model){
		print STDERR "Error:SVMs model not exists\n";
		die;
	}
	
	
	$scaled_model = MyFile::getAbsPath($scaled_model);
	unless(-e $scaled_model){
		print STDERR "Error:scaled model not exists\n";
		die;
	}
	

	# ****************************************************** #
	
	my $sample_size = 0;
	my %hash_name = ();
	my $feature_file = "$dest_dir/predict.feature.txt";
	
	open PREDICTIN,"<$infile_data" or die;
	open FEATURE,">$feature_file" or die;
	
	my $title = 0;
	while(defined (my $line=<PREDICTIN>)){
		chomp($line);
		
		$line =~ s/^\s*//g;
		$line =~ s/\s*$//g;
	
		if ($line =~ /^\s*$/) {
			next;
		}
		
		if ($line =~ /^Name\b/i){
			next;
		}
		
		my @tmp = split /\t/,$line;
		my $name = shift @tmp;
		my $type = pop @tmp;
		
		if ( $type =~ /positive/i ){
			$type = 1;
			$line = join("\t",@tmp);
			$line = "$type\t$line";
		} elsif ( $type =~ /negative/i ){
			$type = -1;
			$line = join("\t",@tmp);
			$line = "$type\t$line";
		} else {
			push(@tmp,$type);
			$line = join("\t",@tmp);
		}
			
		print FEATURE "$line\n";
		unless(exists $hash_name{$name}){
			$hash_name{$name} = 1;
		} else {
			print STDERR "Error:$name repeat in $infile_data\n";
			die;
		}
		$sample_size++;
			
	}
	
	close PREDICTIN;
	close FEATURE;
	
	
	# ****************************************************** #
	
	my $libsvm_file = "$dest_dir/libsvm.txt";
	&toLibsvm($feature_file,$libsvm_file);
	
	
	# ****************************************************** #
	
	my $scaled_file = "$dest_dir/scaled.txt";
	&scale_test_data($libsvm_path,$scaled_model,$libsvm_file,$scaled_file);
		
	
	# ****************************************************** #
	
	my $result_file = "$dest_dir/result.txt";
	&test($libsvm_path,$scaled_file,$svm_model,$result_file);


	# ****************************************************** #

	my $cutoff_step = 0.001;
	&parser($infile_data,$scaled_file,$result_file,
	$outfile_out,$outfile_roc,$cutoff_step);

}



# =============================
# Function: change format of data into required format. 
# Input: a data file And outfile
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub toLibsvm {
	
	my $usage = "$0 < infile > < outfile >";
	my $infile = shift or die $usage;
	my $outfile = shift or die $usage;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	open OUT,">$outfile" or die;  
	while(<IN>){   
		chomp;   
		my @cont=split;   
		my $rlable = $cont[0]; 
		my $str = $rlable;   
		for(my $i=1;$i<@cont;$i++){   
		    if($cont[$i]!=0){   
		        $str .= "\t$i:$cont[$i]";   
		    }   
		}   
		print OUT "$str\n";   
	} 
	close IN;
	close OUT;
	
}




# =============================
# Function: scale the training data
# Input: Libsvm bin path
#		 training data file
#		 outfile for scaled training data
#   	 outfile for scaled model
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub scale_training_data {
	
	my $libsvm_path 		= shift or die;
	my $infile				= shift or die;
	my $outfile_scaled_data = shift or die;	
	my $outfile_scaled_model= shift or die;

	my $current_dir  = getcwd;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	chdir( $libsvm_path );
	
	unless(-e "svm-scale"){
		print STDERR "Error:svm-scale not exists in $libsvm_path\n\n\n";
		die;
	}

	my $os = $^O;
	if ( $os =~ /win/i ){
		die if system("svm-scale -s $outfile_scaled_model $infile > $outfile_scaled_data");
	} else {
		die if system("./svm-scale -s $outfile_scaled_model $infile > $outfile_scaled_data");
	}
	
	chdir($current_dir);
	
}






# =============================
# Function: scale the training data
# Input: Libsvm bin path
#		 infile for scaled model
#		 infile for test data
#   	 outfile for scaled test data
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub scale_test_data {

	my $libsvm_path 		= shift or die;
	my $infile_scaled_model	= shift or die;
	my $infile_test_data	= shift or die;
	my $outfile_test_data 	= shift or die;
	
	my $current_dir = getcwd;
	
	unless(-e $infile_scaled_model){
		print STDERR "Error:$infile_scaled_model not exists\n";
		die;
	}
	unless(-e $infile_test_data){
		print STDERR "Error:$infile_test_data not exists\n";
		die;
	}
	
	chdir( $libsvm_path );
	
	my $os = $^O;
	if ( $os =~ /win/i ){
		die if system("svm-scale -r $infile_scaled_model $infile_test_data > $outfile_test_data");
	} else {
		die if system("./svm-scale -r $infile_scaled_model $infile_test_data > $outfile_test_data");
	}
	
	chdir($current_dir);
	
}





# =============================
# Function: search parameter 
# Input: Libsvm bin path
#		 infile for scaled training data
#		 outfile for parameter
#		 [nfold]
# Return c and gamma parameter
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub search_parameter {	
	
	my $libsvm_path 	  = shift or die;
	my $infile 			  = shift or die;
	my $outfile_parameter = shift or die;
	my $cv 			  	  = shift or die;
	my $kernel			  = shift;
	
	unless(defined $kernel){
		print STDERR "Error:kernel function not defined\n";
		die;
	}
	
	
	my $current_dir = getcwd;
	
	my $os = $^O;
	unless ( $os =~ /win/i ){
		$libsvm_path .= "tools/";
	}
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	chdir( $libsvm_path );

	unless(-e "grid.py"){
		print STDERR "Error:grid.py not exists in $libsvm_path\n\n\n";
		die;
	}
	
	
	die if system("python grid.py -v $cv -t $kernel $infile > $outfile_parameter");
	
	
	chdir( $current_dir );
	
	unless(-e $outfile_parameter){
		print STDERR "Error:$outfile_parameter not exists\n";
		die;
	} else {		
		open IN,"<$outfile_parameter" or die;	
		my $last_line = "";
		while(<IN>){
			next if /^\s*$/;
			$last_line = $_;
		}
		close IN;	
		my @tmp = split /\s+/,$last_line;
		return ($tmp[0], $tmp[1]);
	} 

}






# =============================
# Function: train
# Input: Libsvm bin path
#		 infile for scaled training data
#		 c parameter
#		 gamma parameter
#		 type of kernel function
#		 outfile for model 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub train {
	
	my $libsvm_path 	= shift or die;
	my $infile			= shift or die;
	my $c 				= shift or die;
	my $g			 	= shift or die;
	my $kernel 			= shift or die;
	my $outfile_model 	= shift or die;

	my $current_dir = getcwd;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}

	chdir( $libsvm_path );
	
	unless(-e "svm-train"){
		print STDERR "Error:svm-train not exists in $libsvm_path\n";
		die;
	}

	my $os = $^O;
	if ( $os =~ /win/i ){
		die if system("svm-train -c $c -g $g -t $kernel -b 1 $infile $outfile_model");
	} else {	
		die if system("./svm-train -c $c -g $g -t $kernel -b 1 $infile $outfile_model");
	}
	
	chdir( $current_dir );
	
}




# =============================
# Function: test
# Input: Libsvm bin path
#		 infile for scaled test data
#		 infile for model 
#		 outfile for test result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub test {
	
	my $libsvm_path 		= shift or die;
	my $infile_scaled_test 	= shift or die;
	my $infile_model 		= shift or die;
	my $outfile_test	 	= shift or die;

	my $current_dir = getcwd;

	chdir( $libsvm_path );

	my $os = $^O;
	if ( $os =~ /win/i ){
		die if system("svm-predict -b 1 $infile_scaled_test $infile_model $outfile_test");	
	} else {
		die if system("./svm-predict -b 1 $infile_scaled_test $infile_model $outfile_test");	
	}
	
	chdir( $current_dir ); 
	
}




# =============================
# Function: parse prediction result
# Input: infile for training data
#		 infile for scaled infile_model data
#		 infile for prediction result
#		 outfile for parsed prediction result
#		 cut-off step for ROC curve
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parser {
	
	my $infile_train = shift or die;
	my $infile_scaled_test = shift or die;
	my $infile_predict = shift or die;
	my $outfile_parsed_predict = shift or die;
	my $outfile_roc_curve = shift or die;
	my $cutoff_step = shift;
	
	
	unless(-e $infile_train){
		print STDERR "Error:$infile_train not exists\n";
		die;
	}
	unless(-e $infile_scaled_test){
		print STDERR "Error:$infile_scaled_test not exists\n";
		die;
	}
	unless(-e $infile_predict){
		print STDERR "Error:$infile_predict not exists\n";
		die;
	}
	unless(defined $cutoff_step){
		print STDERR "Error:cut-off step not defined\n";
		die;
	}
	
	
	# get name 
	my $m = 0;
	my %hash_ID = ();
	open IN,"<$infile_train" or die;
	while(defined(my $line = <IN> )){
		chomp($line);
		
		$line =~ s/^\s*//g;
		$line =~ s/\s*$//g;
	
		if ($line =~ /^\s*$/) {
			next;
		}
		
		if ($line =~ /^Name\b/i){
			next;
		}
		
		my @tmp = split /\t/,$line;
		my $name = shift( @tmp );
		$hash_ID{ $m } = $name;
		$m++;
	}
	close IN;
	
	
	# get true label
	$m = 0;
	my %hash_true_label = ();
	open IN,"<$infile_scaled_test" or die;
	while(defined(my $line=<IN>)){
		chomp($line);
		my @tmp = split /\s+/,$line;
		$hash_true_label{$m} = $tmp[0];
		$m++;
	}
	close IN;
	
	
	# get prediction result
	$m = 0;
	my $flag = 0;
	my $col_positive;
	my %hash_predict = ();
	open IN,"<$infile_predict" or die;
	open OUT,">$outfile_parsed_predict" or die;
	while(defined(my $line=<IN>)){
		chomp($line);
		next if $line =~/^\s*$/;
		my @tmp = split /\s+/,$line;
		$line = join("\t",@tmp);
		unless ( $flag ){	
			if ( $tmp[1] != 1 && $tmp[1] != -1 ){
				print STDERR "Error:label error ($infile_predict)\n";
				die;
			}
			if ( $tmp[2] != 1 && $tmp[2] != -1 ){
				print STDERR "Error:label error ($infile_predict)\n";
				die;
			}	
			if ( $tmp[1] == 1 ){
				$col_positive = 1;
			} elsif ( $tmp[2] == 1 ){
				$col_positive = 2;
			}		
			print OUT "Name\tinputLable\t$line\n";		
			$flag = 1;
		} else {		
			if((exists $hash_ID{$m}) and (exists $hash_true_label{$m})){
				print OUT "$hash_ID{$m}\t$hash_true_label{$m}\t$line\n";
				$hash_predict{$hash_ID{$m}}{real} = $hash_true_label{$m};
				$hash_predict{$hash_ID{$m}}{proP} = $tmp[$col_positive];
			} else {
				print STDERR "Error:no corresponding name ($m)\n";
				die;
			}				
			$m++;
		}
	}	
	close IN;
	close OUT;
	
	
	# draw ROC curve
	&draw_roc_curve(\%hash_predict,$outfile_roc_curve,$cutoff_step);
	
}
	
	

# =============================
# Function: draw ROC curve
# Input: reference of hash of parsed prediction result
#		 outfile for ROC curve data
#		 cut-off step
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub draw_roc_curve {
	
	my $hash_predict 	= shift or die;
	my $outfile_roc_data= shift or die;
	my $cutoff_step 	= shift or die;

	unless(defined $cutoff_step){
		print STDERR "Error:cut-off step not defined\n";
		die;
	}
	
	open  OUT,">$outfile_roc_data" or die;
	print OUT "Cutoff\tTOT_P\tTOT_N\tTP\tTN\tFP\tFN\tPr\tAc\tGm\t1-Sp\tSn\tSp\tMcc\n";
	
	my $cutoff = 0;
	while ( $cutoff < 1 ) {	
		
		my %hash_count = ();
		&TP_TN_FP_FN($hash_predict,\%hash_count,$cutoff);
		
		my %hash_perf = ();
		&sn_sp_ac_mcc(\%hash_count,\%hash_perf);
	
		my $TP = $hash_count{TP};
		my $TN = $hash_count{TN};
		my $FP = $hash_count{FP};
		my $FN = $hash_count{FN};
		my $TOT_P = $hash_count{total_pos};
		my $TOT_N = $hash_count{total_neg};
		
		my $sn = $hash_perf{SN};
		my $sp = $hash_perf{SP};
		my $ac = $hash_perf{AC};
		my $pr = $hash_perf{PR};
		my $Gm = $hash_perf{GM};
		my $mcc= $hash_perf{MCC};
			
		my $t = "";
		if( MyMath::is_numeric($sp) ){
			$t = MyMath::round(1-$sp,6);
		} else {
			$t = "NaN";
		}
		
		$cutoff = MyMath::round($cutoff,4);
		print OUT "$cutoff\t$TOT_P\t$TOT_N\t$TP\t$TN\t$FP\t$FN\t$pr\t$ac\t$Gm\t$t\t$sn\t$sp\t$mcc\n";
		
		$cutoff += $cutoff_step;

	}
	
	close OUT;
	
}




# =============================
# Function: count TP,TN, FP and FN
# Input:  reference of hash of prediction result
#		  reference of hash of output
#		  cut-off value
# Return: reference of hash 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub TP_TN_FP_FN {
	
	my $hash_predict= shift or die;
	my $hash_count 	= shift or die;
	my $cutoff		= shift;
	
	my $total_pos = 0;	
	my $total_neg = 0; 	

	foreach my $id ( sort keys %$hash_predict ){		
		if( $hash_predict->{$id}{real} == 1 ){ 		
			$total_pos++;		
			if ( $hash_predict->{$id}{proP} >= $cutoff ){ 
				unless ( exists $hash_count->{TP} ){
					$hash_count->{TP} = 1;
				} else {
					$hash_count->{TP}++;
				}		
			} else { 
				unless ( exists $hash_count->{FN} ){
					$hash_count->{FN} = 1;
				} else {
					$hash_count->{FN}++;
				}
			}				
		} elsif ( $hash_predict->{$id}{real} == -1 ){
			$total_neg++;
			if ( $hash_predict->{$id}{proP} >= $cutoff ){ 
				unless ( exists $hash_count->{FP} ){
					$hash_count->{FP} = 1;
				} else {
					$hash_count->{FP}++;
				}	
			} else { 
				unless ( exists $hash_count->{TN} ){
					$hash_count->{TN} = 1;
				} else {
					$hash_count->{TN}++;
				}
			}
		} else {
			print STDERR "Error: label must be 1 or -1 ($hash_predict->{$id}{real})\n";
			die;
		}				
	}
	
	$hash_count->{total_pos} = $total_pos;
	$hash_count->{total_neg} = $total_neg;
	

	unless ( exists $hash_count->{TP} ){
		$hash_count->{TP} = 0;
	}
	unless ( exists $hash_count->{TN} ){
		$hash_count->{TN} = 0;
	}
	unless ( exists $hash_count->{FP} ){
		$hash_count->{FP} = 0;
	}
	unless ( exists $hash_count->{FN} ){
		$hash_count->{FN} = 0;
	}
	
}





# =============================
# Function: calculate Specificity (Sp), Sensitivity (Sn), 
# Accuracy (Ac), and the Mathew correlation coefficient (MCC) 
# Input:  reference of hash of prediction result
#		  reference of hash of calculated result
# Return: reference of hash 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub sn_sp_ac_mcc {
	
	my $hash_count = shift or die;
	my $hash_perf  = shift or die;

	my $TP = $hash_count->{TP};
	my $TN = $hash_count->{TN};
	my $FP = $hash_count->{FP};
	my $FN = $hash_count->{FN};
	
	my $sn = "NaN";
	my $sp = "NaN";
	my $ac = "NaN";
	my $pr = "NaN";
	my $Gm = "NaN";
	my $mcc = "NaN";
	
	if ( $TP + $FN > 0 ) {
		$sn = $TP / ( $TP + $FN );
		$sn = MyMath::round($sn,6);
	}

	if ( $TN + $FP > 0  ){
		$sp = $TN / ( $TN + $FP );
		$sp = MyMath::round($sp,6);
	} 

	if ( $TP + $FP + $TN + $FN > 0 ) {
		$ac = ($TP + $TN) / ( $TP + $FP + $TN + $FN );
		$ac = MyMath::round($ac,6);
	}

 	if ( $TP + $FP > 0 ){
 		$pr = $TP / ( $TP + $FP );
 		$pr = MyMath::round($pr,6);
 	}
 	
	if( ($TP+$FN) * ($TN+$FP) * ($TP+$FP) * ($TN+$FN) > 0 ){
		$mcc = ( ($TP*$TN) - ($FN*$FP) ) / sqrt( ($TP+$FN) * ($TN+$FP) * ($TP+$FP) * ($TN+$FN) );
		$mcc = MyMath::round($mcc, 6);
	}
	
	if ( MyMath::is_numeric($sn) && MyMath::is_numeric($sp) && $sn * $sp >= 0 ){
		$Gm = sqrt( $sn * $sp );
		$Gm = MyMath::round($Gm,6);
	}
			
	
	$hash_perf->{SN}  = $sn;
	$hash_perf->{SP}  = $sp;
	$hash_perf->{AC}  = $ac;
	$hash_perf->{PR}  = $pr;
	$hash_perf->{GM}  = $Gm;
	$hash_perf->{MCC} = $mcc;
	
	
	my $t = "";
	if( MyMath::is_numeric($sp) ){
		$t = MyMath::round(1-$sp,6);
	} else {
		$t = "NaN";
	}
	
	
	return "TP:$TP\tTN:$TN\tFP:$FP\tFN:$FN\tPR:$pr\tAC:$ac\tGM:$Gm\t1-SP:$t\tSN:$sn\tSP:$sp\tMCC:$mcc";
	
}



# =============================
# Function: calculate Specificity (Sp), Sensitivity (Sn), 
# Accuracy (Ac), and the Mathew correlation coefficient (MCC) 
# Input:  reference of hash of prediction result
#		  reference of hash of calculated result
# Return: reference of hash 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub sn_sp_ac_mcc_1 {
	
	
	my ($total_pos,$total_neg,$TP,$TN,$hash_perf) = @_;
	
	
	my $FP = $total_neg - $TN;
	my $FN = $total_pos - $TP;
	
	my $sn = "NaN";
	my $sp = "NaN";
	my $ac = "NaN";
	my $pr = "NaN";
	my $Gm = "NaN";
	my $mcc = "NaN";
	
	if ( $TP + $FN > 0 ) {
		$sn = $TP / ( $TP + $FN );
		$sn = MyMath::round($sn,6);
	}

	if ( $TN + $FP > 0  ){
		$sp = $TN / ( $TN + $FP );
		$sp = MyMath::round($sp,6);
	} 

	if ( $TP + $FP + $TN + $FN > 0 ) {
		$ac = ($TP + $TN) / ( $TP + $FP + $TN + $FN );
		$ac = MyMath::round($ac,6);
	}

 	if ( $TP + $FP > 0 ){
 		$pr = $TP / ( $TP + $FP );
 		$pr = MyMath::round($pr,6);
 	}

	if( ($TP+$FN) * ($TN+$FP) * ($TP+$FP) * ($TN+$FN) > 0 ){
		$mcc = ( ($TP*$TN) - ($FN*$FP) ) / sqrt( ($TP+$FN) * ($TN+$FP) * ($TP+$FP) * ($TN+$FN) );
		$mcc = MyMath::round($mcc, 6);
	}
	
	if ( MyMath::is_numeric($sn) && MyMath::is_numeric($sp) && $sn * $sp >= 0 ){
		$Gm = sqrt( $sn * $sp );
		$Gm = MyMath::round($Gm,6);
	}
	
	if ( defined $hash_perf ) {		
		$hash_perf->{SN}  = $sn;
		$hash_perf->{SP}  = $sp;
		$hash_perf->{AC}  = $ac;
		$hash_perf->{PR}  = $pr;
		$hash_perf->{GM}  = $Gm;
		$hash_perf->{MCC} = $mcc;		
	}
	
	my $t = "";
	if( MyMath::is_numeric($sp) ){
		$t = MyMath::round(1-$sp,6);
	} else {
		$t = "NaN";
	}
	
	return "TP:$TP\tTN:$TN\tFP:$FP\tFN:$FN\tPR:$pr\tAC:$ac\tGM:$Gm\t1-SP:$t\tSN:$sn\tSP:$sp\tMCC:$mcc";
	
}


################################################################
1;