package Mireap;
#!/usr/bin/perl
use strict;
use warnings;
use Cwd;
use File::Basename;


# My Library
use MyFasta;





# =============================
# Function: use Mireap to predict novel microRNAs
# Input: infile for Bowtie default output
# 		 outfile for result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub doMireap {
	
	my $ref_genome  	= shift or die;
	my $read_fasta 		= shift or die;
	my $Mireap_mapping 	= shift or die;
	my $mireap_path		= shift or die;
	my $dest_dir		= shift or die;
	
	
	my $sample;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/sample/i) {$sample=shift @_}
	}
	
	
	unless(-e $ref_genome){
		print STDERR "Error:$ref_genome not exists\n";
		die;
	}
	unless(-e $read_fasta){
		print STDERR "Error:$read_fasta not exists\n";
		die;
	}
	unless(-e $Mireap_mapping){
		print STDERR "Error:$Mireap_mapping not exists\n";
		die;
	}
	
	my $current_dir = getcwd;
	chdir($mireap_path);
	
	unless(-e "bin/mireap.pl"){
		print STDERR "Error:$mireap_path\n";
		die;
	}
	
	if ( defined $sample ){
		
		die if system("perl bin/mireap.pl -i $read_fasta -m $Mireap_mapping ".
		"-r $ref_genome -t $sample -o $dest_dir");
		
	} else {
		
		die if system("perl bin/mireap.pl -i $read_fasta -m $Mireap_mapping ".
		"-r $ref_genome -o $dest_dir");
		
	}
		
	chdir($current_dir);
	
}
	
	



# =============================
# Function: parse the result of Mireap
# Input: infile for Bowtie default output
# 		 outfile for result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parser {
	
	my $infile_gff 			= shift or die;
	my $infile_aln 			= shift or die;
	my $outfile_annotation 	= shift or die;
	my $hash_sequence		= shift or die;


	unless(-e $infile_gff){
		print STDERR "Error:$infile_gff not exists\n";
		die;
	}
	unless(-e $infile_aln){
		print STDERR "Error:$infile_aln not exists\n";
		die;
	}
	
	my %hash_mature = ();
	&gff_parser($infile_gff,\%hash_mature);
	
	
	my %hash_precursor = ();
	&aln_parser($infile_aln,\%hash_precursor);
	
	my %hash_mature_seq = ();
	my %hash_precursor_seq = ();
	
	
	
	open  MIREAP,">$outfile_annotation" or die;
	print MIREAP "precursor\tmature\tmature position\t";
	print MIREAP "total read count\tmature read count\t";
	print MIREAP "mature sequence\tprecursor sequence\t";
	print MIREAP "mature coordiante\tprecursor coordinate\n";
	
	for my $mature ( sort keys %hash_mature ){
		
		my $mature_seq 	= $hash_mature{$mature}{seq};
		my $mature_coor = $hash_mature{$mature}{coor};
		my $mature_count= $hash_mature{$mature}{count};
		my $precursor 	= $hash_mature{$mature}{parent};
		
		unless(exists $hash_precursor{$precursor}){
			print STDERR "Error:$precursor not a precursor\n";
			die;
		}
		
		my $precursor_seq 	= $hash_precursor{$precursor}{seq};
		my $precursor_coor	= $hash_precursor{$precursor}{coor};
		my $precursor_count = $hash_precursor{$precursor}{count};
		
		my ($chrom,$pb,$pe,$mb,$me,$strand);
		if ( $mature_coor =~ /(\S+):(\d+):(\d+):(\S+)/ ){			
			$chrom 	= $1;
			$pb 	= $2;
			$pe 	= $3;
			$strand = $4;
		} else {
			print STDERR "Error:format of coordinate ($mature_coor)\n";
			die;
		}
		
		if ( $precursor_coor =~ /\S+:(\d+):(\d+):\S+/ ){		
			$mb = $1;
			$me = $2;
		} else {
			print STDERR "Error:format of coordinate ($precursor_coor)\n";
			die;
		}
			
		($mb,$me) = ($me,$mb) if $strand eq "-";
		($pb,$pe) = ($pe,$pb) if $strand eq "-";
		
		my $bd = abs($mb - $pb) + 1;
		my $ed = $bd + length($mature_seq) - 1;
		
		
		my $str = "$precursor\t$mature\t$bd:$ed\t$precursor_count\t$mature_count\t";
		$str .= "$mature_seq\t$precursor_seq\t$mature_coor\t$precursor_coor";
		print MIREAP "$str\n";
		
	
		my $mature_id = $mature;	
		my $precursor_id = $precursor;
			
		unless(exists $hash_sequence->{mature}{$mature_id}{seq}){
			$mature_seq = uc ( $mature_seq );
			$mature_seq =~ s/U/T/ig;
			$hash_sequence->{mature}{$mature_id}{seq} 		= $mature_seq;
			$hash_sequence->{mature}{$mature_id}{count} 	= $mature_count;
			$hash_sequence->{mature}{$mature_id}{site} 		= "$bd:$ed";
			$hash_sequence->{mature}{$mature_id}{coor}		= $mature_coor;
			$hash_sequence->{mature}{$mature_id}{tool}		= "Mireap";
			$hash_sequence->{mature}{$mature_id}{parent}	= $precursor_id;
		} else {
			print STDERR "Error:$mature_id repeat\n";
			die;
		}
		
		
		unless(exists $hash_sequence->{precursor}{$precursor_id}{seq}){
			$precursor_seq = uc ( $precursor_seq );
			$precursor_seq =~ s/U/T/ig;
			$hash_sequence->{precursor}{$precursor_id}{seq} 	= $precursor_seq;
			$hash_sequence->{precursor}{$precursor_id}{coor}	= $precursor_coor;
			$hash_sequence->{precursor}{$precursor_id}{count} 	= $precursor_count;	
			$hash_sequence->{precursor}{$precursor_id}{tool} 	= "Mireap";		
		} 
				
	}
	close MIREAP;
	
}
			
	
	
	
	
	
	
	
# =============================
# Function: parse the aln file from Mireap
# Input: aln file
# 		 reference of hash for parsed result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub aln_parser {
	
	my $infile_aln 		= shift or die;
	my $hash_precursor 	= shift or die;
	
	unless(-e $infile_aln){
		print STDERR "Error:$infile_aln not exists\n";
		die;
	}
	
	open IN,"<$infile_aln" or die;
	while(defined(my $line=<IN>)){
		chomp($line);
		next if $line=~/^\s*$/;
		if ($line=~/^\s*mireap/i){
			$line=<IN>;
			if($line=~/(\S+)\s+(\S+)/){
				my $precursor_id	= $1;
				my $precursor_coor 	= $2;
				$line=<IN>;
				if($line=~/(\S+)\s+\S+\s+(\S+)/){
					my $precursor_seq 	= $1;
					my $precursor_count	= $2;
					unless(exists $hash_precursor->{$precursor_id}){
						$hash_precursor->{$precursor_id}{seq} 	= $precursor_seq;
						$hash_precursor->{$precursor_id}{coor} 	= $precursor_coor;
						$hash_precursor->{$precursor_id}{count}	= $precursor_count;
					} else {
						print STDERR "Error:$precursor_id repeat\n";
						die;
					}		
				}
			}
		}
	}
	
	close IN;
	
}
	
	
	
	
	
# =============================
# Function: parse the gff file from Mireap
# Input: gff file
# 		 reference of hash for parsed result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub gff_parser {
	
	my $infile_gff 	= shift or die;
	my $hash_mature = shift or die;
	
	unless(-e $infile_gff){
		print STDERR "Error:$infile_gff not exists\n";
		die;
	}
	
	open IN,"<$infile_gff" or die;
	while(defined(my $line=<IN>)){
		chomp($line);
		next if $line=~/^\s*$|^#/;
		next if $line=~/^\s*$/;
		next if $line=~/precursor/;
		
		if ($line=~/^(\S+)\s+\S+\s+\S+\s+(\S+)\s+(\S+)\s+\.\s+(\S+)/){	
				
			my $chrom = $1;
			my $beg   = $2;
			my $end   = $3;
			my $strand= $4;		
			my $mature_coor = "$chrom:$beg:$end:$strand";
			
			if($line=~/\bID=(\S+);\bParent=(\S+);Count=(\S+);Seq=(\S+)/){
				
				my $mature_id 	 = $1;
				my $precursor_id = $2;
				my $mature_count = $3;
				my $mature_seq 	 = $4;
				
				unless(exists $hash_mature->{$mature_id}){
					$hash_mature->{$mature_id}{seq}    = $mature_seq;
					$hash_mature->{$mature_id}{coor}   = $mature_coor;
					$hash_mature->{$mature_id}{count}  = $mature_count;
					$hash_mature->{$mature_id}{parent} = $precursor_id;
				} else {
					print STDERR "Error:$mature_id repeat\n";
					die;
				}				
			} else {
				print STDERR "Error:format error ($line)\n";
				die;
			}
		} else {		
			print STDERR "Error:format error ($line)\n";
			die;	
		}
	}
	close IN;
}
			
################################################################
1;			
	