package MyFile;
#!/usr/bin/perl
use strict;
use warnings;
use Cwd;
use File::Copy;
use File::Path;
use File::Basename;






# =============================
# Function: raverse folder and descendants of a folder 
# Input: a directory.
# Return: filename array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub traverseDir {
	
    my $path = shift or die;
     
    $path =~ s/\/$//;
	$path = $path."/";
	
	my @dirs = ();
    push(@dirs,$path);
    
    my ($dir,$file,@DirFile);
    while ($dir = pop(@dirs)) {
        local *DH;
        
        next unless opendir(DH, $dir);
        
        foreach (readdir(DH)) {
        	
            next if $_ eq "." or $_ eq "..";
            
            $file = $dir.$_;
            if (!-l $file && -d _) {
                $file .= "/";
                push(@dirs, $file);
            }
            push @DirFile,$file;
        }
        closedir(DH);
    }
    
    return @DirFile;
    
}



# =============================
# Function: copy folder and descendants of a folder 
# Input: a directory.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub copyDir {
	
    my ($source_dir, $dest_dir) = @_;
   
	$source_dir =~ s/\/$//;
	$source_dir = $source_dir."/";

	$dest_dir =~ s/\/$//;
	$dest_dir = $dest_dir ."/";
   
    unless(-d $dest_dir){
    	mkdir($dest_dir,0777);
    }
    
    opendir my($DH), $source_dir or die "Could not turn on dir $source_dir: $!";
    
    for my $entry (readdir $DH) {
    	
        next if $entry eq "." or $entry eq "..";
        
        my $source = "$source_dir/$entry";
        my $destination = "$dest_dir/$entry";
        
        if (-d $source) {
            mkdir $destination or die "mkdir $destination failed: $!" unless -e $destination;
            copyDir($source, $destination);
        } else {
            copy($source, $destination) or die "copy failed: $!";
        }
    }
    closedir $DH;
    
    return 1;
}




# =============================
# Function: delete folder and descendants of a folder 
# Input: a directory.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub rm{
	
	my $usage = "< path of directory >";
	my $dir = shift or die $usage;
	
	rmtree($dir);
	
	sleep(1);
	
	return 1;
}





# =============================
# Function: make a directory.
# Input: a path of directory And recreate flag 
# 		 1 (not recreate if the directory already exists) 
# or
#        0 (recreate if the directory already exists) (default).
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub makeDir {
	
	my $usage = "< path of directory > [1 (not recreate) or 0 (recreat)] ";
	my $dir  = shift or die $usage;
	my $recreate = shift;
	
	unless( defined $recreate ){
		$recreate = 0;
	}
	
	unless( $recreate ){
		unless( -d $dir ){
			my $old_umask = umask;
			umask 0;
			mkdir($dir, 0777);
			umask $old_umask;
		}
	} else {		
		unless( -d $dir ){
			my $old_umask = umask;
			umask 0;
			mkdir($dir, 0777);
			umask $old_umask;
		} else {
			&rm($dir);
			sleep(1);
			my $old_umask = umask;
			umask 0;
			mkdir($dir, 0777);
			umask $old_umask;
		}
	}
}



# =============================
# Function: get the absolute path.
# Input: a directory or a file.
# Return: the absolute path.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getAbsPath {
	
	my $usage = "< directory or file >";
	my $file_or_dir = shift or die $usage;
	
	my $dir = dirname($file_or_dir);
	my $basename = basename($file_or_dir);
	
	my $current_dir = getcwd;
	my $child_dir = "$current_dir/$dir";
	
	if($dir eq "."){
		return "$current_dir/$basename";
	} elsif( $dir eq ".." ){
		chdir("../");
		my $parent_dir = getcwd;
		chdir($current_dir);		
		return "$parent_dir/$basename";
	} elsif ( -d $child_dir ){
		return "$child_dir/$basename";
	} else {
		return $file_or_dir;
	}
	
}




# =============================
# Function: change html to txt
# Input: infile for html
#		 outfile for parsed result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================


sub html2txt {
	
	my $infile_html = shift or die;
	my $outfile_txt = shift or die;
	
	unless(-e $infile_html){
		print STDERR "Error:$infile_html not exists\n";
		die;
	}
	
	my $str = "";
	open IN,"<$infile_html" or die;
	while(<IN>){
		next if /^\s*$/;
		$str .= $_;
	}	
	close IN;
	
	$str =~ s/\s*&#177\s*/_/g;
	$str =~ s/<(.*?)>//gi;
	
	open  OUT,">$outfile_txt" or die;
	print OUT "$str\n";
	close OUT;
	
}



# =============================
# Function: get the file list in current directory
# Input: path of directory
#		 reference of array for file list
#		 reference of array for regular expression
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getFileListInCurrentDir {
	
	my $dir 		= shift or die;
	my $ref_array 	= shift or die;
	my $ref_regexp 	= shift;
	
	opendir(DIR,$dir);
	while( my $file = readdir( DIR ) ){
		next if $file eq "." or $file eq ".."; 
		if( defined $ref_regexp ){
			for my $regexp ( @$ref_regexp ){
				if($file =~/$regexp/){
					push(@$ref_array,$file);
				}
			}
		} else {
			push(@$ref_array,$file);
		}
	}
	
	closedir(DIR);
	
}


# =============================
# Function: backup .bashrc and .bash_profile in Linux
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub bashrc_backup {
	
    my $file   = shift or die;
    my $marker = shift or die;
   
    my $install_time = time;
    
    if( -f "$ENV{'HOME'}/$file" ){
    	
        `cp ~/$file ~/${file}_$install_time`;
        `mv ~/$file ~/$file.bak`;
        
        open IN,"$ENV{'HOME'}/$file.bak";
        open OUT,">$ENV{'HOME'}/$file" or die "Cannot create file $ENV{'HOME'}/$file\n";     
        while( <IN> ){    	
           my $str = "";
            if( /$marker/ ){
            	
                my @tmp = split/:/;        
                foreach( @tmp ){
                    unless( /$marker/ ){
                        $str.="$_";
                    }
                }
                
                if( $file !~ /.cshrc/ ){                            
                    if($str !~ /PATH=\$PATH$/ and $str !~ /PERL5LIB=\$PERL5LIB\s*$/){
                        print OUT "$str\n";
                    }                
                }else{             	
                    if($str !~ /PATH \$PATH$/ and $str !~ /PERL5LIB \$PERL5LIB\s*$/){
                        print OUT "$str\n";
                    }                
                }
                
            } else {
            	
                print OUT;
                
            }          
        }       
        close IN;
        close OUT;     
    } else {
    	
        print STDERR "file $ENV{'HOME'}/$file.bak not found\n";
        
    }
}





# =============================
# Function: check perl module
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub CheckModule {
	
    my $module = shift;
    
    eval "use $module";
    
    return 1 unless $@;
    
    print STDERR "Error!\n $@\n\n\n";
    
    return 0;
}






# =============================
# Function: installer
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub installer {
	
	my $install_path = shift or die;
	my $download_url = shift or die;
	my $install_cmds = shift or die;
	
	
	my $dest_dir;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/cd/i) {$dest_dir=shift @_}
	}
	
	
	
	my @tmp = split/\//,$download_url;
	my $source_name = pop @tmp;
	
	my ($source_dir,$uncompress_cmd);	
	if( $source_name =~ /(\S+)\.tar\.gz$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "tar zxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.gz$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "gunzip $source_name";
	} elsif ( $source_name =~ /(\S+)\.tar\.bz2$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "tar jxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.bz2$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "bunzip2 $source_name";
	} elsif ( $source_name =~ /(\S+)\.tar\.bz$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "tar jxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.bz$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "bunzip2 $source_name";
	} elsif ( $source_name =~ /(\S+)\.tar\.Z$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "tar Zxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.Z$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "uncompress $source_name";
	} elsif ( $source_name =~ /(\S+)\.tar\.tgz$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "tar zxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.tgz/ ){
		$source_dir  = $1;
		$uncompress_cmd = "tar zxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.zip$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "unzip $source_name";
	} elsif ( $source_name =~ /(\S+)\.rar$/ ){
		$source_dir  = $1;
		$uncompress_cmd = "rar a $source_name";
	} else {
		print STDERR "Error:unidentified types of compressed files\n";
		die;
	}

	if( defined $dest_dir ){
		$source_dir = "$install_path/$dest_dir";
	} else {
		$source_dir = "$install_path/$source_dir";
	}
	
	print STDERR "\n$source_dir\t$source_name\t$uncompress_cmd\n\n";

		
	my $current_dir = getcwd;
	chdir( $install_path );
	
	unless( -d $source_dir ){		
		unless( -f $source_name ){			
			print STDERR "Downloading $source_name now\n\n";		
			my $err = system("wget $download_url");		
			if( $err ){
				die "Download of $source_name not successful\n\n";
			}		
			unless(-f $source_name ){
				die "$source_name download failed\n";
			}
		}

		`$uncompress_cmd`;
					
		if ( -d $source_dir ){
			chdir($source_dir);	
		} else {
			print STDERR "Error:$source_dir not exists\n";
			die;
		}
				
		my $dir = getcwd;
		print "dir:$dir\n";
		
		if( defined $install_cmds ){				
			foreach my $cmd ( @$install_cmds ){
				system("$cmd");
			}				
		}	
	}
	
	chdir($current_dir);
	
	return $source_dir;
		
}




# =============================
# Function: unpack 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub do_unpack {
	
	my $source_name = shift or die;
	
	my @tmp = split/\//,$source_name;
	$source_name = pop @tmp;
	my $dest_dir = join("/",@tmp);
	
	my $current_dir = getcwd;
	if ( length $dest_dir ){
		chdir($dest_dir);
	}
	
	my ($uncompress_name,$uncompress_cmd);	
	if( $source_name =~ /(\S+)\.tar\.gz$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "tar zxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.gz$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "gunzip $source_name";
	} elsif ( $source_name =~ /(\S+)\.tar\.bz2$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "tar jxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.bz2$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "bunzip2 $source_name";
	} elsif ( $source_name =~ /(\S+)\.tar\.bz$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "tar jxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.bz$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "bunzip2 $source_name";
	} elsif ( $source_name =~ /(\S+)\.tar\.Z$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "tar Zxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.Z$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "uncompress $source_name";
	} elsif ( $source_name =~ /(\S+)\.tar\.tgz$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "tar zxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.tgz/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "tar zxvf $source_name";
	} elsif ( $source_name =~ /(\S+)\.zip$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "unzip $source_name";
	} elsif ( $source_name =~ /(\S+)\.rar$/ ){
		$uncompress_name  = $1;
		$uncompress_cmd = "rar a $source_name";
	} else {
		print STDERR "Error:unidentified types of compressed files\n";
		die;
	}
	
	unlink($uncompress_name) if -e $uncompress_name;
	die if system($uncompress_cmd);	
	
	unless(-e $uncompress_name){
		print STDERR "Error:$uncompress_name not exists\n";
		die;
	} 
	
	if ( length $dest_dir ){
		$uncompress_name = "$dest_dir/$uncompress_name";
		chdir($current_dir);
	}
	
	return $uncompress_name;
	
}



# =============================
# Function: check whether installed
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_install {
    
    my $cmd 	= shift or die;
    my $marker 	= shift or die;
    
	my $ret = system("$cmd> tmp 2>tmp2");
	
    open IN,"<tmp";
    my $found = 1;
    while( <IN> ){
		if( /$marker/ ){
			$found = 0;			
		}
	}
	close IN;
	
	if( $found ){
		open IN,"<tmp2";
		while( <IN> ){
			if( /$marker/ ){
				$found = 0;			
			}
		}
		close IN;
	}
    
    
    unless( $found ){
		unlink("tmp" ) if -e "tmp";
		unlink("tmp2") if -e "tmp2";
    	return 1;
    } else {
    	return 0;
    }
}




# =============================
# Function: install perl modules
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub perl_module_installer {
	
	my $install_dir = shift or die;
	my $modules 	= shift or die;
	
	my $current_dir = getcwd;
	chdir($install_dir);
	
	foreach my $module ( @$modules ){
			
		print STDERR "Checking required module '$module'...\n\n\n";
	    
	    if ( &CheckModule($module) ) {
	        print STDERR "Ok!\n";
	        next;
	    }

	    print STDERR "Required module '$module' is not found on this system!\n\n\n";
	    
	    print STDERR "Install '$module' now ...\n\n\n";
	    
		system("cpan $module");  
		
		if ( &CheckModule($module) ) {
			print STDERR "Ok!\n";
		} else {
			print  STDERR "\n\n\nInstall $module error\n\n\n";
			die;
		}
		
	}
	
	chdir($current_dir);
		
}



# =============================
# Function: install perl modules
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub wget {
	
	my $dest_dir = shift or die;
	my $download_url = shift or die;
	
	my $current_dir = getcwd;
	chdir($dest_dir);
	
	unless(-d $dest_dir){
		&makeDir($dest_dir);
	}
	
	my $err = system("wget --timestamping $download_url");		
	if( $err ){
		die "Download of $download_url not successful\n\n";
	}

	chdir($current_dir);
	
	my @tmp = split/\//,$download_url;
	my $source_name = pop @tmp;
	
	return "$dest_dir/$source_name";
	
}



# =============================
# Function: install perl modules
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub check_md5 {
	
	my $infile = shift or die;
	my $remote_md5_file = shift or die;
	
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	unless(-e $remote_md5_file){
		print STDERR "Error:$remote_md5_file not exists\n";
		die;
	}
	
	my $current_dir = getcwd;
	my $dir = dirname($infile);
	my $basename = basename($infile);
	chdir( $dir );
	
	my $local_md5_file = "$infile.local.md5";
	my $err = system("md5sum $basename > $local_md5_file");
	if( $err ){
		die "check md5 of $infile not successful\n\n";
	}
	
	my $local_md5 =`cat $local_md5_file 2>&1`;
	
	$local_md5 =~ s/^\s*//g;
	$local_md5 =~ s/\s*$//g; 
		
	my $in = `grep "$local_md5" $remote_md5_file`;
	
	chdir($current_dir);
	
	if( $in ){
		print "\n\n$infile md5 correspond\n\n";
		return 1;
	} else {
		print "\n\n$infile md5 no correspond\n\n";
		return 0;
	}
		
}




# =============================
# Function: start timing.
# Input:  Optional file handle.
# Return: the number of total read and unique read.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub start {
	
	my $out = shift; 
	
    my($second, $minute, $hour, $dayOfMonth, $month, 
    $yearOffset, $dayOfWeek, $dayOfYear, $daylightSavings) = localtime();
    $second = "0$second" if($second =~ /^\d$/);
    my $sTime = "$hour:$minute:$second";
    my $stime = time;
    my $start_time = localtime;
    
    print "\n\nstarted: $start_time\n\n";
    if( defined $out ){
    	print $out "started: $start_time\n";
    }
    
    return $stime;
    
}



# =============================
# Function: end timing.
# Input:  stime and optional file handle.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub end {
	
	my $stime = shift or die;
	my $out = shift;
	
    my $etime = time - $stime;
    my ($second, $minute, $hour, $dayOfMonth, $month, 
    $yearOffset, $dayOfWeek, $dayOfYear, $daylightSavings) = localtime();
    $second = "0$second" if($second =~ /^\d$/);
    my $eTime = "$hour:$minute:$second";
    
    my $end_time = localtime;
    
    print "\n\nended: $end_time\n";
    print "total:", int($etime / 3600),"h:",
    int(($etime % 3600) / 60),"m:",int($etime % 60),"s\n\n";
    
    if( defined $out ){
    	print $out "\n\nended: $end_time\n";
    	print $out "total:", int($etime / 3600),"h:",
    	int(($etime % 3600) / 60),"m:",int($etime % 60),"s\n\n";
    }
}





# =============================
# Function: get time.
# Return: time in a variety of formats.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getTime {
   
    my $time = shift || time();
    
    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime($time);
    
    $mon ++;
    $sec  = ($sec <10) ? "0$sec" : $sec;
    $min  = ($min <10) ? "0$min" : $min;
    $hour = ($hour<10) ? "0$hour": $hour;
    $mday = ($mday<10) ? "0$mday": $mday;
    $mon  = ($mon <9 ) ? "0".($mon+1):$mon;
    $year += 1900;
     
    my $weekday = ('Sun','Mon','Tue','Wed','Thu','Fri','Sat')[$wday];
   
    return { 
     'second' => $sec,
     'minute' => $min,
     'hour'   => $hour,
     'day'    => $mday,
     'month'  => $mon,
     'year'   => $year,
     'weekNo' => $wday,
     'wday'   => $weekday,
     'yday'   => $yday,
     'date'   => "$year-$mon-$mday"
	};
}





# =============================	
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub check_R_package {
	
	my $package_name = shift or die;
	my $R_bin = shift or die;
	
	my $current_dir = getcwd;
	
	unless ( MyFile::is_install("R --version","version") ){
		print STDERR "Error:R not install\n";
		die;
	}
	
	my @R_cmds = (
		'.packages(all.available = TRUE)',
		'q("yes")'
	);
	
	my $R_script = "$current_dir/check.R";
	open RSCRIPT, ">$R_script" or die;
	for my $cmd ( @R_cmds ){
	    print RSCRIPT "$cmd\n";
	}
	close RSCRIPT;
		
		
	die if system("$R_bin CMD BATCH $R_script ");
	
	my $Rout = "$R_script"."out";
	unless( -e $Rout ){
		print STDERR "Error:$Rout not exists\n";
		die;
	}
	
	
	my @lines = ();
	open INSTALL,"<$Rout" or die;
	while( <INSTALL> ){
		push(@lines,$_);
	}
	close INSTALL;
	
	my $available = 0;
	for my $line ( @lines ){
		if ( $line =~ /"$package_name"/ ){
			$available = 1;
			last;
		}
	}
	
	unlink($Rout);
	unlink($R_script);
	
	return $available;
	
}
	
	
	
	
	
	
# =============================	
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub install_R_package {
	
	my $package_name = shift or die;
	
	my $R_bin = "R";
	my $mirror ="http://ftp.ctex.org/mirrors/CRAN/";
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/R_bin/i) {$R_bin=shift @_}
		if ($argument=~/mirror/i) {$mirror=shift @_}
	}

	my $current_dir = getcwd;
	
	unless ( MyFile::is_install("R --version","version") ){
		print STDERR "Error:R not install\n";
		die;
	}
	
	
	# check
	my $available = &check_R_package($package_name,$R_bin);
	
	
	# install
	unless( $available ){
		
		my @R_cmds = ( 
			'local({r <- getOption("repos") '."\n".' r["CRAN"] <- "' . $mirror . '" '."\n".' options(repos=r)})',
			'install.packages("' . $package_name . '" )',
			'q("yes")'
		);

	
		my $R_script = "$current_dir/install.R";
		open RSCRIPT, ">$R_script" or die;
		for my $cmd ( @R_cmds ){
			print RSCRIPT "$cmd\n";
		}
		close RSCRIPT;
		
		my $ret = system("$R_bin CMD BATCH $R_script ");
		
		if ( $ret ){
			print STDERR "Error:Unable to install R package:$package_name\n";
			print STDERR "Manual Install it by typing in R command line:\n";
			print STDERR "install.packages(\"$package_name\")\n";
			die;
		}
		
		$available = &check_R_package($package_name,$R_bin);
		
		if ( $available ){
			unlink($R_script);
			unlink("$current_dir/install.Rout");
		}
		
	}
	
	return $available;
		
}	

################################################################
1;