package MyMath;
#!/usr/bin/perl
use strict;
use warnings;





# =============================
# Input: a reference of numeric array.
# Return: median of this numeric array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub median {
	
	my $usage = "median < reference of numeric array >";
    my $rpole = shift or die $usage;
    
    my @pole = @$rpole;
    
    my $ii = 0;
    foreach my $i ( @pole ){
	    unless ( &is_numeric($i) ){
			print STDERR "Error:$i not a numeric\n";
			die;
		}
    	$ii++;
    }
    
    if( $ii == 0 ){
    	print STDERR "Error:Empty array";
    	die;
    }
    
   	@pole = sort( {$a<=>$b} @pole);
   	
   	my $ret;
    if( (@pole % 2) == 1 ) {
        $ret = $pole[((@pole+1) / 2)-1];
    } else {
        $ret = ($pole[(@pole / 2)-1] + $pole[@pole / 2]) / 2;
    }
    
    return $ret;
    
}



# ===============================
# Input: a reference of array.
# Return: reference of shuffled array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub shuffle {
	
	my $usage = " shuffle < reference of array >";
	my $array = shift or die $usage;
	
	for (my $j = @$array-1; $j > 0; --$j) {
  		my $i = int(rand($j));
   		my $t = $array->[$j]; $array->[$j] = $array->[$i]; $array->[$i] = $t;
	}
}



# =============================
# Input: a reference of numeric array.
# Return: the maximum value of this numeric list.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub max {
	
    my $array = shift;
    my $v = shift;
    
    if ( &is_array_ref( $array ) ){
	    my $max = $array->[0];
	    foreach (@$array) {
	        $max = $max > $_ ? $max : $_;
	    }  
	    return $max;
	} elsif ( defined $v ) {
		my $t = $array > $v ? $array : $v;
		return $t;
	}
    
}



# =============================
# Input: a reference of numeric array.
# Return: the minimum value of this numeric list.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub min {
	
    my $array = shift;
    my $v = shift;
    
    if ( &is_array_ref( $array ) ){
	    my $min = $array->[0];
	    foreach (@$array) {
	        $min = $min < $_ ? $min : $_;
	    }
	    return $min;
	} elsif ( defined $v ) {
		my $t = $array < $v ? $array : $v;
		return $t;
	}
    
}






# =============================
# Input:  two references of array. 
# Return: intersection of both array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub intersection {
	
	my $usage = "< reference of array A > < reference of array B >";
	my $arrayA = shift or die $usage;
	my $arrayB = shift or die $usage;
	
	my %union = ();
	my %isect = ();
	
	foreach my $e(@$arrayA,@$arrayB){
		$union{$e}++ && $isect{$e}++;
	}	
	
	my @ret = keys %isect;	
	while ( @_ ) {
		my $t = shift @_;
		@ret = &intersection(\@ret,$t);
	}
	
	return @ret;
	
} 



# =============================
# Input:  a parameter of any type.
# Return: 1 (numeric) or 0 (non-numeric). 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_numeric {
	
	use Scalar::Util qw(looks_like_number);
	
    my $v = shift;

    if( looks_like_number( $v ) ){  	
        return 1;
    } else {
        return 0;
    }
    
}



# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_array_ref {
    if(	$_[0] and Scalar::Util::reftype($_[0]) and
		Scalar::Util::reftype($_[0]) eq "ARRAY") {
        return 1;
    } else {
        return 0;
    }
}



# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_hash_ref {
    if( $_[0] and Scalar::Util::reftype($_[0]) and
		Scalar::Util::reftype($_[0]) eq "HASH") {
        return 1;
    } else {
        return 0;
    }
}



# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_scalar_ref {
    if(	$_[0] and Scalar::Util::reftype($_[0]) and
		Scalar::Util::reftype($_[0]) eq "SCALAR") {
        return 1;
    } else {
        return 0;
    }
}


# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_code_ref {
    if(	$_[0] and Scalar::Util::reftype($_[0]) and
		Scalar::Util::reftype($_[0]) eq "CODE") {
        return 1;
    } else {
        return 0;
    }
}


# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_ref_ref {
    if(	$_[0] and Scalar::Util::reftype($_[0]) and
		Scalar::Util::reftype($_[0]) eq "REF") {
        return 1;
    } else {
        return 0;
    }
}



# =============================
# Input:  a numerical value.
# Return: order of magnitude of this numerical value.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub order_of_magnitude {
	
	my $usage = "< numerical value >";
	my $value = shift;
	
	die $usage unless defined $value;
	
	unless ( &is_numeric($value) ){
		print STDERR "Error:$value not a numeric\n";
		die;
	}
	
	if ( $value < 1 ){
		return 0;
	} else {
		my $n = 0;
		while( $value >= 1 ){
			$value = int( $value / 10 );
			$n++;
		}
		return $n;
	}
}



# =============================
# Input:  a numerical value (greater than 0).
# Return: log10 value of this numerical value.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub log10 { 
	
	my $usage = "< numerical value (>0) >";
	my $n = shift;
	
	die $usage unless defined $n;
	
	unless(&is_numeric($n)){
		print STDERR "Error:$n not a numeric\n";
		die;
	}
	
	if ( $n <= 0 ){
		print STDERR "Error:$n <= 0\n";
		die;
	} else {
		return log($n)/log(10);    
	}
}




# =============================
# Input:  a numerical value (greater than 0).
# Return: log2 value of this numerical value.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub log2 { 
	
	my $usage = "< numerical value (>0) >";
	my $n = shift;
	
	die $usage unless defined $n;
	
	unless(&is_numeric($n)){
		print STDERR "Error:$n not a numeric\n";
		die;
	}
	
	if ( $n <= 0 ){
		print STDERR "Error:$n <= 0\n";
		die;
	} else {
		return log($n)/log(2);    
	}
}




# =============================
# Input: a numerical value and the numbers after the decimal point.
# Return: rounded value of this numerical value.
# Date: 2013-2-20
# =============================

sub round {
	
	my $usage = "<  numerical value > < the numbers after the decimal point >";
    my $val = shift;
    my $col = shift;
    
    unless(defined $val){
    	die $usage;
    }
    
    unless(defined $col){
    	die $usage;
    }
    
    unless( &is_numeric($val) ){
    	print STDERR "Error:$val not a numeric";
		die;
	}
	
	unless( &is_numeric($col) ){
    	print STDERR "Error:$col not a numeric";
		die;
	}
    
    my $r = 10 ** $col;
    my $a = ($val > 0) ? 0.5 : -0.5;
    
    return int($val * $r + $a) / $r;
    
}





# =============================
# Input: a numeric array.
# Return: rank of this numeric array.
# Reference: Spearman's rank correlation coefficient.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub rank {
	
	my @data = @_;
	
	my $ii = 0;
	foreach my $i ( @data ){
		unless( &is_numeric($i) ){
			print STDERR "Error:$i not a numeric";
			die;
		}
		$ii++;
	}
	
	if( $ii == 0 ) {
		print STDERR "Error:Empty array\n";
		die;
	}
	
	
	my $m = 0;
	my @rank = ();
	
	my @orderData = sort( {$a <=> $b }  @data);
	
	for (my $i = 0; $i < scalar @data; $i++ ){
		
		my $n = 0;
		my @tmp = ();
		
		for( my $j = 0; $j < scalar @orderData; $j++ ){
			if( $data[$i] == $orderData[$j] ){
				$tmp[$n++] = $j + 1;
			}
		}
		
		if( $n == 1 ){
			$rank[$m++] = $tmp[0];
		}else{
			my $s = 0;
			my $k = 0;
			for( ; $k < scalar @tmp; $k++ ){
				$s += $tmp[$k];
			}
			$rank[$m++] = $s / $k ;
		}
	}
	
	return @rank;
	
}


# =============================
# Input: get sign of value.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub sign {
    if($_[0] > 0) {
        return 1;
    } elsif($_[0] < 0) {
        return -1;
    } else {
        return 0;
    }
}



# =============================
# Input: a numeric array.
# Return: mean of this numeric array.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub mean {
	
	my @data = @_;
	
	my $n = 0;
	foreach my $i ( @data ){
		unless( &is_numeric($i) ){
			print STDERR "Error:$i not a numeric";
			die;
		}
		$n++;
	}
	
	if( $n == 0 ) {
		print STDERR "Error:Empty array\n";
		die;
	}
	
	my $total = 0;	
	for( my $i = 0; $i  < scalar @data; $i++ ){
		$total += $data[$i];
	}
	
	return  $total / $n;
}


# =============================
# Input: two references of numeric arrays.
# Return: Pearson's correlation coefficient of these numeric arrays.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub pearson {
	
	my ( $dataA, $dataB ) = @_;
	
	# check dim.
	if( $#$dataA != $#$dataB ){
		print STDERR "Error:Unequal dimension\n";
		die;
	}
	
	# calculate means.
	my $meanA = &mean( @$dataA );
	my $meanB = &mean( @$dataB );
	
	# calculate numerator.
	my $sum = 0;
	for( my $i = 0; $i < scalar @$dataA; $i++ ){
		$sum += ( $dataA->[$i] - $meanA ) * ( $dataB->[$i] - $meanB );
	}
	my $up = $sum;
	
	# calculate denominator.
	$sum = 0;
	for( my $i = 0; $i < scalar @$dataA; $i++ ){
		$sum += ( $dataA->[$i] - $meanA ) * ( $dataA->[$i] - $meanA );
	}
	my $sqrtA = sqrt( $sum );	
	
	$sum = 0;
	for( my $i = 0; $i < scalar @$dataB; $i++ ){
		$sum += ( $dataB->[$i] - $meanB ) * ( $dataB->[$i] - $meanB );
	}
	my $sqrtB = sqrt( $sum );
	
	# check whether denominator is 0.
	if( $sqrtA <= 1.e-100 || $sqrtB <= 1.e-100 ){
		return 0;
	} else {
		return $up / ( $sqrtA * $sqrtB );
	}
	
}




# =============================
# Input: two numeric arrays.
# Return: Spearman's rank correlation coefficient of these numeric arrays.
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub spearman {
	
	my ( $dataAA, $dataBB ) = @_;	
	
	my @dataA = &rank( @$dataAA );
	my @dataB = &rank( @$dataBB );	
	
	my $spearman = &pearson( \@dataA, \@dataB );
	
	return $spearman;
}





# =============================
# Function: read a file into matrix
# Input:  a file 
#		  reference of hash 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub toMatrix {
	
	my $infile 	 = shift or die;
	my $hash_row = shift or die;
	
	
	my $name;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/name/i) {$name=shift @_}
	}
	

	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	
	my $flag = 1;
	my @columns = ();
	my %hash_col_tmp = ();
	
	while( defined( my $line = <IN> )){
		chomp($line);
		next if $line =~/^\s*$/;
		
		my @tmp = split/\t/,$line;
		my $row = shift @tmp;
		
		if ( $flag ) {
			if ( defined $name ){
				unshift(@tmp,$row);
			} else {	
				push(@columns,$row);
			}
			for my $i ( 0 .. $#tmp ){		
				unless( exists $hash_col_tmp{ $i } ){
					$hash_col_tmp{ $i } = $tmp[$i];
				}	
				push(@columns,$tmp[$i]);		
			}
			$flag = 0;
			next;
		} 
		
		for my $i ( 0 .. $#tmp ){
			my $col = $hash_col_tmp{ $i };
			unless( exists $hash_row->{ $row }{ $col } ){	
				$hash_row->{ $row }{ $col } = $tmp[$i];
			} else {
				print STDERR "Error:($row,$col) repeat\n";
				die;
			}
		}
	}
	close IN;
	
	return @columns;
	
}




# =============================
# Function: read a file into matrix
# Input:  a file 
#		  reference of hash 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub toMatrix_no_title {
	
	my $infile 	   = shift or die;
	my $hash_row   = shift or die;
	my $orderd_row = shift or die;
	my $orderd_col = shift or die;

	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	
	my $flag = 1;
	my %hash_col_tmp = ();
	
	while( defined( my $line = <IN> )){
		chomp($line);
		next if $line =~/^\s*$/;
		next if $line =~/^#/;
		
		my @tmp = split/\t/,$line;
		my $row = shift @tmp;
		push(@$orderd_row,$row);
		
		if ( $flag ) {
			for my $i ( 0 .. $#tmp ){		
				unless( exists $hash_col_tmp{ $i } ){
					$hash_col_tmp{ $i } = $i + 1;
				}
				my $t = $i + 1;
				push(@$orderd_col,$t);		
			}
			$flag = 0;
		} 
		
		for my $i ( 0 .. $#tmp ){
			my $col = $hash_col_tmp{ $i };
			unless( exists $hash_row->{ $row }{ $col } ){	
				$hash_row->{ $row }{ $col } = $tmp[$i];
			} else {
				print STDERR "Error:($row,$col) repeat\n";
				die;
			}
		}
	}
	close IN;
	
}





# =============================
# Function: get permutation with specific number characters
# Input:  reference of array for input character set 
#		  reference of array for output permutation
#		  length of permutation
#		  prefix of each permutation
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub permutation {
	
	my ($in_ref_array,$out_ref_array,$n,$prefix) = @_;
	
	unless( defined $prefix ){
		$prefix = "";
	}
	
	if( $n == 0 ){
		for my $r ( @$in_ref_array ){
			push (@$out_ref_array,$prefix.$r);
		}
      	return;
   }
   
   for my $r ( @$in_ref_array ){
		&permutation($in_ref_array,$out_ref_array,$n-1,$prefix.$r);
	}
}





# =============================
# Function: each element in each column
# is normalized by sum of this column
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub normalize_by_sum {
	
	my $infile  = shift or die;
	my $outfile = shift or die;
	

	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
		
	my %hash_row = ();
	my @cols = &toMatrix($infile,\%hash_row);

	my @rows = keys %hash_row;
	
	my %hash_sum = ();
	my $title = shift @cols;
	
	for my $col ( @cols ){
		my $sum = 0;
		for my $row ( @rows ){
			if( exists $hash_row{$row}{$col} ){
				if( &is_numeric($hash_row{$row}{$col}) ){	
					$sum += $hash_row{$row}{$col};
				}
			}
		}
		$hash_sum{$col} = $sum;
		$title .= "\t$col";
	}

	
	
	open  NORM,">$outfile" or die;
	print NORM "$title\n";
	
	for my $row ( @rows ){
		my $str = $row;
		for my $col ( @cols ){
			my $t = 0;
			if( exists $hash_row{$row}{$col} && exists $hash_sum{$col} && $hash_sum{$col} > 0 ){
				$t = $hash_row{$row}{$col}/$hash_sum{$col};
			}
			$t = &round($t,6);
			$str .= "\t$t";
		}
		print NORM "$str\n";
	}
	close NORM;
	
}
		
		
	


# =============================
# Function: each element in each column
# is normalized by absolute max in this column
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub normalize_by_max {
	
	my $infile  = shift or die;
	my $outfile = shift or die;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
		
	my %hash_row = ();
	my @cols = &toMatrix($infile,\%hash_row);

	my @rows = keys %hash_row;
	
	my %hash_max = ();
	my $title = shift @cols;
	
	for my $col ( @cols ){
		my $max = 0;
		for my $row ( @rows ){
			if( exists $hash_row{$row}{$col} ){
				if( &is_numeric($hash_row{$row}{$col}) ){	
					my $v = abs($hash_row{$row}{$col});
					if(  $v > $max ){
						$max = $v;
					}
				}
			}
		}
		$hash_max{$col} = $max;
		$title .= "\t$col";
	}

	
	
	open  NORM,">$outfile" or die;
	print NORM "$title\n";
	
	for my $row ( @rows ){
		my $str = $row;
		for my $col ( @cols ){
			my $t = 0;
			if( exists $hash_row{$row}{$col} && exists $hash_max{$col} && $hash_max{$col} > 0 ){
				$t = $hash_row{$row}{$col}/$hash_max{$col};
			}
			$t = &round($t,6);
			$str .= "\t$t";
		}
		print NORM "$str\n";
	}
	close NORM;
	
}



# =============================
# Function: each element in each column
# is normalized by absolute max in this column
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub normalize_by_max_no_title {
	
	my $infile  = shift or die;
	my $outfile = shift or die;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
		
	my (%hash_row,@rows,@cols) = ();
	&toMatrix_no_title($infile,\%hash_row,\@rows,\@cols);

	
	my %hash_max = ();
	for my $col ( @cols ){
		my $max = 0;
		for my $row ( @rows ){
			if( exists $hash_row{$row}{$col} ){
				if( &is_numeric($hash_row{$row}{$col}) ){	
					my $v = abs($hash_row{$row}{$col});
					if(  $v > $max ){
						$max = $v;
					}
				}
			}
		}
		$hash_max{$col} = $max;
	}

	
	open  NORM,">$outfile" or die;
	for my $row ( @rows ){
		my $str = $row;
		for my $col ( @cols ){
			my $t = 0;
			if( exists $hash_row{$row}{$col} && exists $hash_max{$col} && $hash_max{$col} > 0 ){
				$t = $hash_row{$row}{$col}/$hash_max{$col};
			}
			$t = &round($t,6);
			$str .= "\t$t";
		}
		print NORM "$str\n";
	}
	close NORM;
	
}




# =============================
# Function: delete overlap coordinates 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub del_overlap {
	
	my $begs = shift or die;
	my $ends = shift or die;
	my $new_begs = shift or die;
	my $new_ends = shift or die;
	
	my @ranges;
	while ( @$begs && @$ends ) {
	    my $b = shift ( @$begs );
	    my $e = shift ( @$ends );
	    push ( @ranges, [ $b, $e ] );
	}
	
	my @merged_ranges;
	push ( @merged_ranges, shift (@ranges) );
	
	for my $range ( @ranges ) {
	    my $overlap = 0;
	    for my $m_range ( @merged_ranges ) {
	        if ( $range->[0] <= $m_range->[1] && $m_range->[0] <= $range->[1] ){
	            $overlap = 1;            
	            my $t1 = &min($range->[0],$m_range->[0]);
	            my $t2 = &max($range->[1],$m_range->[1]);	        
	            $m_range = [ $t1 , $t2 ];
	        }
	    }
	    unless ( $overlap ) {
	        push (@merged_ranges, $range);
	    }
	}
	
	for my $range (@merged_ranges) {
		push(@$new_begs,$range->[0]);
		push(@$new_ends,$range->[1]);
	}
	
}



# =============================
# Function: calcluate factorial
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub factorial {
	
	my $n = shift;
	
	unless( &is_numeric($n) ){
		print STDERR "Error:$n not a numeric\n";
		die;
	}
	
	if ( $n <= 0 ){
		print STDERR "Error:$n <= 0\n";
		die;
	}
	
	my $nj = 1; 
	until( $n == 0 ) { 
		$nj *= $n; 
		$n--; 
	} 
	
	return $nj;
	
}


####################################################################################
1;