package Redundancy;
#!/usr/bin/perl
use strict;
use warnings;


# My Library
use MyFasta;
use MyMath;


# =============================
# Function: remove redundant predicted novel microRNAs
# Input: mature fasta from miRBase
# 		 hairpin fasta from miRBase
#		 gff3 annotation file from miRBase
#		 mapped output from Bowtie
#		 outfile for profile
#		 reference of hash for parsed result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub remove {
	
	my $hash_count = shift or die;
	my $hash_precursor_score = shift or die;
	my $hash_precursor_mature = shift or die;
	my $infile_mature	= shift or die;
	my $infile_hairpin 	= shift or die;	
	my $infile_mapping  = shift or die;
	my $hairpin_struct 	= shift or die;
	my $outfile_profile = shift or die;
	my $hash_parse 		= shift or die;
	my $hash_mature_tag = shift or die;
	
	
	
	my $BEG_DIF  = -2;
	my $END_DIF  = 5;
	
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/beg_dif/i) {$BEG_DIF=shift @_}
		if ($argument=~/end_dif/i) {$END_DIF=shift @_}
	}
	
	
	unless(-e $infile_mature){
		print STDERR "Error:$infile_mature not exists\n";
		die;
	}
	unless(-e $infile_hairpin){
		print STDERR "Error:$infile_hairpin not exists\n";
		die;
	}
	unless(-e $infile_mapping){
		print STDERR "Error:$infile_mapping not exists\n";
		die;
	}
	unless(-e $hairpin_struct){
		print STDERR "Error:$hairpin_struct not exists\n";
		die;
	}
	

	my %hash_mature = ();
	MyFasta::readfasta($infile_mature,\%hash_mature);
	
	
	my %hash_struct = ();
	&parse_struct($hairpin_struct,\%hash_struct);
	
	
	my %hash_precursor = ();
	MyFasta::readfasta($infile_hairpin,\%hash_precursor);
	
	
	my %hash_map = ();
	my %hash_tag = ();
	my %hash_maptime = ();
	Bowtie::parser5($infile_mapping,\%hash_map,\%hash_tag,\%hash_maptime);
	

	# ************************************************* #

	
	my %hash_precursor_count_tmp = ();
	
	for my $precursor ( sort keys %$hash_precursor_mature ){
	for my $mature ( sort keys %{ $hash_precursor_mature->{$precursor} } ){
		
		my ($mbeg,$mend) = split/:/,$hash_precursor_mature->{$precursor}{$mature};
		
		for my $tag ( keys %{ $hash_map{$precursor} } ){
		for my $beg ( keys %{ $hash_map{$precursor}{$tag} } ){
		for my $end ( keys %{ $hash_map{$precursor}{$tag}{$beg} } ){
				
			my $bd = $beg - $mbeg;
			my $ed = $end - $mend;
			
			my $count = MyMath::round($hash_count->{$tag}/$hash_maptime{$tag},6);
			
			if( $bd >= $BEG_DIF && $ed <= $END_DIF ){	
				unless(exists $hash_precursor_count_tmp{$precursor}){
					$hash_precursor_count_tmp{$precursor} = $count;
				} else {
					$hash_precursor_count_tmp{$precursor}+= $count;
				}
			}
				
		}}}
	}}
		
	

	
	# ************************************************* #
	
	
	my %hash_pass = ();
	my %hash_profile = ();
	my %hash_mature_tag_same = ();
	my %hash_precursor_count = ();
	
	for my $precursor ( sort { $hash_precursor_score->{$b} <=> $hash_precursor_score->{$a} or 
		$hash_precursor_count_tmp{$b} <=> $hash_precursor_count_tmp{$a} or $a cmp $b } 
		keys %hash_precursor_count_tmp ){
	
		unless(exists $hash_precursor_mature->{$precursor}){
			print STDERR "warn:$precursor not exists\n";
			die;
		}
			
		for my $mature ( keys %{ $hash_precursor_mature->{$precursor} } ){
						
			my ($mbeg,$mend) = split/:/,$hash_precursor_mature->{$precursor}{$mature};
		
			for my $tag ( keys %{ $hash_map{$precursor} } ){
			for my $beg ( keys %{ $hash_map{$precursor}{$tag} } ){
			for my $end ( keys %{ $hash_map{$precursor}{$tag}{$beg} } ){
				
				my $bd = $beg - $mbeg;
				my $ed = $end - $mend;
				
				my $mis = $hash_map{$precursor}{$tag}{$beg}{$end};
				my $count = MyMath::round($hash_count->{$tag}/$hash_maptime{$tag},6);
				
				
				if( $bd >= $BEG_DIF && $ed <= $END_DIF ){
					
					unless( exists $hash_pass{$tag} ){
			
						if( $bd ==0 && $ed == 0 && MyMath::is_numeric($mis) && $mis == 0 ){
							unless(exists $hash_mature_tag_same{$mature}{$tag}){
								$hash_mature_tag_same{$mature}{$tag} = $count;
							} else {
								$hash_mature_tag_same{$mature}{$tag}+= $count;
							}
						}
														
						unless(exists $hash_mature_tag->{$mature}{$tag}){
							$hash_mature_tag->{$mature}{$tag} = $count;
						} else {
							$hash_mature_tag->{$mature}{$tag}+= $count;
						}
						
						unless(exists $hash_profile{$precursor}{$mature}{$tag}){
							$hash_profile{$precursor}{$mature}{$tag} = "$bd#$ed#$mis";
						} else {
							print STDERR "Error:($precursor,$mature,$tag) repeat\n";
							die;
						}
						
						unless(exists $hash_precursor_count{$precursor}){
							$hash_precursor_count{$precursor} = $count;
						} else {
							$hash_precursor_count{$precursor}+= $count;
						}	
						
						$hash_pass{$tag} = 1;
						
					}	
											
				}					
				
			}}}
		}			
	}
		

	# ************************************************* #
	
	for my $mature (sort keys %$hash_mature_tag ){
		for my $tag ( sort {$hash_mature_tag->{$mature}{$b} <=> $hash_mature_tag->{$mature}{$a} } 
			keys %{ $hash_mature_tag->{$mature} } ){
			$hash_parse->{$mature}{most_count} = $hash_mature_tag->{$mature}{$tag};
			$hash_parse->{$mature}{most_seq} = $hash_tag{$tag};
			last;
		}
	}
	
	for my $mature (sort keys %$hash_mature_tag ){
		my $tot_count = 0;
		for my $tag ( keys %{ $hash_mature_tag->{$mature} } ){
			$tot_count += $hash_mature_tag->{$mature}{$tag};
		}
		$hash_parse->{$mature}{total} = $tot_count;
	}
	
	
	for my $mature (sort keys %hash_mature_tag_same ){
		my $tot_count = 0;
		for my $tag ( keys %{ $hash_mature_tag_same{$mature} } ){
			$tot_count += $hash_mature_tag_same{$mature}{$tag};
		}
		$hash_parse->{$mature}{same} = $tot_count;
	}
	
	
	# ************************************************* #


	open PROFILE,">$outfile_profile" or die;	
	
	for my $precursor ( sort { $hash_precursor_score->{$b} <=> $hash_precursor_score->{$a} or 
		$hash_precursor_count_tmp{$b} <=> $hash_precursor_count_tmp{$a} } 
		keys %hash_precursor_count_tmp ){
	
		if ( exists $hash_profile{$precursor} ){
		
			my $precursor_lng = length($hash_precursor{$precursor});
			my $mfe = MyMath::round($hash_struct{$precursor}{mfe},2);
			my $struct = $hash_struct{$precursor}{struct};
			my $count = $hash_precursor_count{$precursor};		
			my $precursor_seq = uc( $hash_precursor{$precursor} );
			$precursor_seq =~ tr/T/U/;
			
			print PROFILE ">$precursor Reads:$count Length:$precursor_lng(nt) ";
			print PROFILE "MFE:$mfe(kcal/mol) probability:$hash_precursor_score->{$precursor}\n";
			print PROFILE "$precursor_seq\n$struct\tName\tReads\tMismatch\n";
			
			for my $mature ( sort keys %{ $hash_profile{$precursor} } ){
				
				my $count = $hash_parse->{$mature}{total};
				my ($beg,$end) = split/:/,$hash_precursor_mature->{$precursor}{$mature};
				my $mature_seq = uc( $hash_mature{$mature} );
				$mature_seq =~ tr/T/U/;
						
				my $str = "*" x (length $hash_precursor{$precursor});
				substr($str,$beg-1,length($hash_mature{$mature})) = $mature_seq;					
				print PROFILE "$str\t$mature\t$count\n";	
								
				for my $tag ( sort {$hash_mature_tag->{$mature}{$b} <=> $hash_mature_tag->{$mature}{$a}} 
					keys %{ $hash_mature_tag->{$mature} }){				
					if(exists $hash_profile{$precursor}{$mature}{$tag}){
						my ($bd,$ed,$mis) = split/#/,$hash_profile{$precursor}{$mature}{$tag};
						my $count = $hash_mature_tag->{$mature}{$tag};
						my $tag_seq = uc( $hash_tag{$tag} );
						$tag_seq =~ tr/T/U/;
						
						my $tag_beg	= $beg + $bd;
						my $str = "-" x (length $hash_precursor{$precursor});
						substr($str,$tag_beg-1,length($hash_tag{$tag})) = $tag_seq;	
						print PROFILE "$str\t$tag\t$count\t$mis\n";
					} 	
				}		
			}
			print PROFILE "//\n";
		}
	}
	close PROFILE;

	# ************************************************* #		
	
}



# =============================
# Function: parse the output structure of RNAfold
# Input: a RNAfold structure file 
#		 (output) Reference of hash for structure
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parse_struct {
	
	my $infile_struct = shift or die;
	my $hash_struct	= shift or die;
	
	unless(-e $infile_struct){
		print STDERR "Error:$infile_struct not exists\n";
		die;
	}
	
	open IN,"<$infile_struct" or die;
	
	my $id = "";
	my %hash_tmp = ();
	
	while( defined(my $line = <IN> ) ){
		chomp( $line );
		next if $line =~ /^\s*$/;	
			
		if( $line =~/>/ ){
			$id = substr($line,1);
		} elsif ( $line =~ /[ACGUTN]+/i ) {
			
			$line = uc( $line );
			$hash_tmp{$id}{seq} = $line;
				
		} elsif ( $line =~ /\.|\(|\)/  ){				
			
			my @tmp = split /\s+\(/,$line;	
			$hash_tmp{$id}{struct} = $tmp[0];
			
			my $mfe = $tmp[1];
			$mfe =~ s/\(|\)//g;			
			$hash_tmp{$id}{mfe}= $mfe;
			
		}
	}
	
	close IN;
	
	
	for my $id ( keys %hash_tmp ){
		
		my $mfe = $hash_tmp{$id}{mfe};
		my $seq = $hash_tmp{$id}{seq};
		my $struct = $hash_tmp{$id}{struct};
		
		unless( $struct =~ /\(/ ) {
			next;
		}
		
		unless( $struct =~ /\)/ ) {
			next;
		}
		
		if (length($struct) != length($seq)){
			next;
		}
		
		unless( MyMath::is_numeric($mfe) ) {
			next;
		}
		
		$hash_struct->{$id}{mfe} = $mfe;
		$hash_struct->{$id}{seq} = $seq;
		$hash_struct->{$id}{struct} = $struct;
		
	}

}
		
#################################################################
1;