package Repeat;
#!/usr/bin/perl
use strict;
use warnings;
use Cwd;
use File::Basename;
use MyFile;




# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub rescue {
	
	my $sampleNames 		= shift or die;
	my $rescuedTyps 		= shift or die;
	my $hash_mapping_files 	= shift or die;
	my $hash_repeat_reads 	= shift or die;
	my $dest_dir 			= shift or die;
	
	my @samples = ();
	my %hash_stat = ();
	for my $file ( sort keys %$hash_mapping_files ){
		
		unless(-e $file){
			print STDERR "Error:$file not exists\n";
			die;
		}
		
		my $n = 0;
		my $sample;
		for my $r ( @$sampleNames ){
			if($file=~/\b$r\b/){
				$sample = $r;
				$n++;
			}
		}
		
		if ( $n == 1) {
			push(@samples,$sample);
		} else {
			print STDERR "Error:\n";
			die;
		}
		
		# ==============================================
		
		my %hash_id  = ();
		my %hash_tag = ();
		my %hash_tmp = ();
		
		my $total_read = 0;
		my $total_tag = 0;
		
		open RESCUEIN,"<$file" or die;
		while(defined (my $line = <RESCUEIN>)){
			chomp($line);
			my @tmp = split /\t/,$line;
			my $tag_id = $tmp[0];
			my $repeat_id = $tmp[2];
			
			unless(exists $hash_id{$tag_id}){
				$hash_id{$tag_id} = 1;
				my $freq = &getTagFreq($tag_id);
				$total_read += $freq;
				$total_tag++;
			}
		
			# SSU-rRNA_Hsa:rRNA#chr4:61288251:61289959:-
			if ( $repeat_id =~ /(\S+):(\S+)#(\S+):(\S+):(\S+):(\S+)/ ){	
			
				my $repeatName = $1;
				my $className  = $2;
			
				my $flag = 0;
				for my $type ( @$rescuedTyps ){
					if ( $className =~ /^$type/i ){
						$hash_tmp{$type}{$line} = 1;
						unless(exists $hash_tag{$type}{$tag_id}){
							$hash_tag{$type}{$tag_id} = 1;
						}
						$flag++;
					} 						
				}
				
				if ( $flag > 1 ){
					print STDERR "Error:repeat annotation\n";
					die;
				} elsif ( $flag == 0 ){
					my $type = "repeat";
					$hash_tmp{$type}{$line} = 1;
					unless(exists $hash_tag{$type}{$tag_id}){
						$hash_tag{$type}{$tag_id} = 1;
					}
					unless(exists $hash_repeat_reads->{$sample}{$tag_id}){
						$hash_repeat_reads->{$sample}{$tag_id} = 1;
					}
				}
					
			} else {	
				
				print STDERR "Error:$repeat_id format error\n";
				die;
				
			}
		}
		close RESCUEIN;
		
		
		# ==============================================
		
		
		my %hash_seq = ();
		my $mapped_file = $hash_mapping_files->{$file};
		&readfasta($mapped_file,\%hash_seq);
		
			
		# ==============================================
		
		
		for my $type ( sort keys %hash_tmp ){
			
			my $tmp_dir = "$dest_dir/$type";
			MyFile::makeDir($tmp_dir);
			my $basename = basename($file);
			
			
			my $outfile = "$tmp_dir/$basename";
			open RESCUEOUT,">$outfile" or die;
			for my $line ( sort keys %{$hash_tmp{$type}} ){
				print RESCUEOUT "$line\n";
			}
			close RESCUEOUT;
			
			
			my $mapped_read = 0;
			my $mapped_tag = 0;
			
			$basename = basename($mapped_file);
			$outfile = "$tmp_dir/$basename";
			
			open RESCUEOUT,">$outfile" or die;	
			for my $tag_id ( sort keys %{$hash_tag{$type}} ){
				my $freq = &getTagFreq($tag_id);
				$mapped_read += $freq;
				$mapped_tag++;
				my $seq = $hash_seq{$tag_id};
				print RESCUEOUT ">$tag_id\n$seq\n";	
			}
			close RESCUEOUT;
			
			
			my $unmapped_read = $total_read - $mapped_read;
			my $unmapped_tag = $total_tag - $mapped_tag;
			
			my $mapped_read_ratio = &round($mapped_read/$total_read,6);
			my $mapped_tag_ratio = &round($mapped_tag/$total_tag,6);
			
			
			# ============================================
				
			$hash_stat{$type}{total_read}{$sample} = $total_read;
			$hash_stat{$type}{total_tag}{$sample} = $total_tag;
			
			$hash_stat{$type}{mapped_read}{$sample} = $mapped_read;
			$hash_stat{$type}{mapped_tag}{$sample} = $mapped_tag;
			
			$hash_stat{$type}{mapped_read_ratio}{$sample} = $mapped_read_ratio;
			$hash_stat{$type}{mapped_tag_ratio}{$sample} = $mapped_tag_ratio;
			
			$hash_stat{$type}{unmapped_read}{$sample} = $unmapped_read;
			$hash_stat{$type}{unmapped_tag}{$sample} = $unmapped_tag;
			
		}
		
	}
	
	
	for my $type ( sort keys %hash_stat ){
		
		my $title = "Item";
		for my $sample ( @samples ){
			$title .= "\t$sample";
		}
		
		my $outfile = "$dest_dir/$type/$type.stat.txt";	
		open  RESCUEOUT,">$outfile" or die;
		print RESCUEOUT "$title\n";
		
		for my $item ( sort keys %{$hash_stat{$type}} ){	
			my $str = $item;	
			for my $sample ( @samples ){
				$str .= "\t$hash_stat{$type}{$item}{$sample}";
			}
			print RESCUEOUT "$str\n";
		}
			
		close RESCUEOUT;
		
	}
	
}
		


# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub category {
	
	my $sampleNames				= shift or die;
	my $types 					= shift or die;
	my $hash_mapping_files 		= shift or die;
	my $outfile_category 		= shift or die;
	my $outfile_category_percent= shift or die;
	
	
	my @samples = ();
	my %hash_category = ();
	for my $file ( sort keys %$hash_mapping_files ){
		
		unless(-e $file){
			print STDERR "Error:$file not exists\n";
			die;
		}
				
				
		my $n = 0;
		my $sample;
		for my $r ( @$sampleNames ){
			if($file=~/\b$r\b/){
				$sample = $r;
				$n++;
			}
		}
		
		if ( $n == 1) {
			push(@samples,$sample);
		} else {
			print STDERR "Error:\n";
			die;
		}
			
				
		# ============================================
		
		my %hash_filter_id = ();
		if (-e $hash_mapping_files->{$file} ){
		
			&readfasta($hash_mapping_files->{$file},\%hash_filter_id);
			
		}
		
		# ============================================
		
		my %hash_map = ();
		&mapping_parser($file,\%hash_map);
				
				
		for my $repeat_id ( keys %hash_map ){

			# SSU-rRNA_Hsa:rRNA#chr4:61288251:61289959:-
			if ( $repeat_id =~ /(\S+):(\S+)#(\S+):(\S+):(\S+):(\S+)/ ){	
			
				my $repeatName 	= $1;
				my $className 	= $2;
				
				my $repeatClass = "other";
				for my $type ( @$types ){
					if ( $className =~ /^$type/i ){
						$repeatClass = $type; 
					} 						
				}
				
				for my $tag_id ( keys %{$hash_map{$repeat_id}} ){
					
					if ( exists $hash_filter_id{$tag_id} ){
						next;
					}
					
					my $count = $hash_map{$repeat_id}{$tag_id};			
					unless( exists $hash_category{$repeatClass}{$sample} ){
						$hash_category{$repeatClass}{$sample} = $count;
					} else {
						$hash_category{$repeatClass}{$sample}+= $count;
					}
				}
				
			} else {
				
				print STDERR "Error:$repeat_id format error\n";
				die;
				
			}
		}
	}	
	
	
	
	my %hash_sample = ();
	for my $sample ( @samples ){
		my $sum = 0;
		for my $className ( sort keys %hash_category ){
			if ( exists $hash_category{$className}{$sample} ){
				$sum += $hash_category{$className}{$sample};
			} 
		}
		$hash_sample{$sample} = $sum;
	}
	
	my $title = "Category";
	for my $sample ( @samples ){
		$title .= "\t$sample";
	}
	
	open  CLASSOUTA,">$outfile_category" or die;
	open  CLASSOUTB,">$outfile_category_percent" or die;
	
	print CLASSOUTA "$title\n";
	print CLASSOUTB "$title\n";

	for my $className ( sort keys %hash_category ){
		my $str_A = $className;
		my $str_B = $className;
		for my $sample ( @samples ){
			my $t = 0;
			if ( exists $hash_category{$className}{$sample} ){
				$t = $hash_category{$className}{$sample};
			} 
			$str_A .= "\t$t";
			
			if ( $hash_sample{$sample} > 0 ){
				$t = &round($t/$hash_sample{$sample},6);
			} else {
				$t = 0;
			}
			$str_B .= "\t$t";
		}
		print CLASSOUTA "$str_A\n";
		print CLASSOUTB "$str_B\n";
	}
	
	close CLASSOUTA;
	close CLASSOUTB;
	
}
			
			
	
# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub mapping_parser {	
	
	my $file = shift or die;
	my $hash_map = shift or die;
	
	unless(-e $file){
		print STDERR "Error:$file not exists\n";
		die;
	}
	
	open IN,"<$file" or die;
	while( defined( my $line = <IN> ) ){
		chomp( $line  );
		next if $line =~ /^\s*$/;	
			
		my @tmp 	= split /\t/,$line;
		my $id 		= $tmp[0];
		my $strand 	= $tmp[1];
		my $chrom	= $tmp[2];
		my $beg		= $tmp[3];
		my $seq	 	= $tmp[4];
		my $mapTime	= $tmp[6];	
		my $mis	  	= $tmp[7];
		my $lng     = length( $seq );	
		
		unless ( defined $mis ) {
			$mis = 0;
		}	
		
		
		$beg++;
		my $end = $beg + length($seq) - 1;	
		
	
		my $count = &getTagFreq($id);
		$mapTime++;
		$count = &round($count/$mapTime,6);
		
		
		unless ( exists $hash_map->{$chrom}{$id} ){
			$hash_map->{$chrom}{$id} = $count;
		} else {
			$hash_map->{$chrom}{$id}+= $count;
		}
		
	}
	
	close IN;	
	
}

	



	
# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================
			
sub round {
	
	my $usage = "<  numerical value > < the numbers after the decimal point >";
    my $val = shift;
    my $col = shift;
    
    unless(defined $val){
    	die $usage;
    }
    
    unless(defined $col){
    	die $usage;
    }
    
    unless( &is_numeric($val) ){
    	print STDERR "Error:$val not a numeric";
		die;
	}
	
	unless( &is_numeric($col) ){
    	print STDERR "Error:$col not a numeric";
		die;
	}
    
    my $r = 10 ** $col;
    my $a = ($val > 0) ? 0.5 : -0.5;
    
    return int($val * $r + $a) / $r;
    
}
		
		

# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_numeric {
	
	use Scalar::Util qw(looks_like_number);
	
    my $v = shift;

    if( looks_like_number( $v ) ){  	
        return 1;
    } else {
        return 0;
    }
    
}




# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getTagFreq {
	
	my $usage = "< tag >";
	my $tag = shift or die $usage;
	
	my $count = 1;
	if($tag =~ /_x(\S+)/){
		$count = $1;
	} elsif ($tag =~ /\S+\s+(\S+)/){
		$count = $1;
	} 
	
	unless(&is_numeric($count)){
		print STDERR "Error:$count not a numeric\n";
		print STDERR "Error:format wrong";
		die;
	}
	
	return $count;
	
}



# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub readfasta {
	
	my $usage = "< fasta file > < hash reference >";
	my $infile = shift or die $usage;
	my $hash_seq = shift or die $usage;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists";
		die;
	}
	
	open IN, $infile || die;
	
	my $c=0;
	my $seqId;
	while (defined (my $line=<IN>)) {
		chomp($line);
		next if $line=~/^\s*$/;
		if ($line =~/^>/) {
			$line =~s/\s*$//g;	
			$line =~s/^\s*//g;	
			$seqId = substr($line,1);
			$c++;
		} else {
			$line =~s/\s*//g;
			$hash_seq->{$seqId}.=$line;
		}
	}
	close IN;
	
	return $c;
	
}


################################################
1;
	