package RepeatMasker;
#!/usr/bin/perl
use strict;
use warnings;
use Scalar::Util qw(looks_like_number);
use threads;
use threads::shared;


# My libray
use MyFasta;







# =============================
# Input:  a parameter of any type.
# Return: 1 (numeric) or 0 (non-numeric). 
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub is_numeric {
	
    my $v = shift;

    if( looks_like_number( $v ) ){  	
        return 1;
    } else {
        return 0;
    }
    
}




# =============================
# Function: parse the .out file of RepeatMasker
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parser {
	
	my $infile = shift or die;
	my $hash_parse = shift or die;
	
	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	open IN,"<$infile" or die;
	while(defined(my $line =<IN>)){
		chomp($line);
		next if $line =~/^\s*$/;
		$line =~ s/^\s*//g;
		$line =~ s/\s*$//g;
		my @tmp = split/\s+/,$line;
		unless( &is_numeric($tmp[0]) ){
			next;
		}
		
		my $queryName 	= $tmp[4];
		my $beg 		= $tmp[5];
		my $end 		= $tmp[6];
		my $matchStrand = $tmp[8];
		my $repeatName	= $tmp[9];
		my $className	= $tmp[10];
		
		$matchStrand =~ s/C/-/i;
		
		my $id = "$repeatName:$className#$queryName:$beg:$end:$matchStrand";
		unless( exists $hash_parse->{$queryName}{$id} ){
			$hash_parse->{$queryName}{$id} = "$beg:$end:$matchStrand";
		} else {
			print STDERR "Error:($queryName,$id) repeat\n";
			die;
		}
	}
	close IN;
}
		

		
		
		
		
# =============================
# Function: get repeat from genome
# according to RepeatMasker annotation
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getRepeat {
	
	my $genome_fasta = shift or die;
	my $repeat_fasta = shift or die;
	my $hash_parse	 = shift or die;
	
	unless(-e $genome_fasta){
		print STDERR "Error:$genome_fasta not exists\n";
		die;
	}
	

	my $c = 0;
    my($seqId, $seq) = ();
    
    open FASTA,"<$genome_fasta" or die;
    open REPEAT,">$repeat_fasta" or die;
    
    while (<FASTA>) {
        chomp;
        if (/^>(\S+)/){
		    $seq = "";
		    $seqId = $1;
		    while (<FASTA>){
				chomp;
				if (/^>(\S+)/){							
					if( exists $hash_parse->{$seqId} ){
						for my $id ( keys %{$hash_parse->{$seqId}} ){
							my ($beg,$end,$strand) = split /:/,$hash_parse->{$seqId}{$id};
							my $repeat_seq = MyFasta::getSeqByCoordinate($seq,$beg,$end,$strand);
							my $j = 0;
							my $str_out = "";
							while($j < length($repeat_seq) ){		
								my $subseq = substr($repeat_seq,$j,50);
								$str_out .= "$subseq\n";
								$j += 50;
							}							
							$str_out =~ tr/[acgtuUn\.]/[ACGTTTNN]/;	
							print REPEAT ">$id\n$str_out";
						}
					}
					
					$seqId = $1;	
					$seq  = "";
					$c++;
					next;
				}
				s/\s*//g;
				$seq .=  $_;
		    }
		}
    }
    
    close FASTA;
     
    if( exists $hash_parse->{$seqId} ){
		for my $id ( keys %{$hash_parse->{$seqId}} ){
			my ($beg,$end,$strand) = split /:/,$hash_parse->{$seqId}{$id};
			my $repeat_seq = MyFasta::getSeqByCoordinate($seq,$beg,$end,$strand);
			my $j = 0;
			my $str_out = "";
			while($j < length($repeat_seq) ){		
				my $subseq = substr($repeat_seq,$j,50);
				$str_out .= "$subseq\n";
				$j += 50;
			}		
			$str_out =~ tr/[acgtuUn\.]/[ACGTTTNN]/;	
			print REPEAT ">$id\n$str_out";
		}
	}
    
    close REPEAT;
    										
	$c++;
	
    return $c;
    
}




# =============================
# Function: get repeat from genome
# according to RepeatMasker annotation
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub extract_seq {
	
	my $hash_repeat = shift or die;
	my $hash_parse  = shift or die;
	my $hash_genome = shift or die;
	my $chrom = shift or die;
	
	my $seq;
	if ( exists $hash_genome->{$chrom} ){
		$seq = $hash_genome->{$chrom};
	} else {
		print STDERR "Error:$chrom not exists\n";
		die;
	}
	
	for my $id ( keys %{$hash_parse->{$chrom}} ){
		my ($beg,$end,$strand) = split /:/,$hash_parse->{$chrom}{$id};
		my $repeat_seq = MyFasta::getSeqByCoordinate($seq,$beg,$end,$strand);
		my $j = 0;
		my $str_out = "";
		while($j < length($repeat_seq) ){		
			my $subseq = substr($repeat_seq,$j,50);
			$str_out .= "$subseq\n";
			$j += 50;
		}							
		$str_out =~ tr/[acgtuUn\.]/[ACGTTTNN]/;
		unless(exists $hash_repeat->{$id}){
			$hash_repeat->{$id} = $str_out;	
		} else {
			print STDERR "Error:$id repeat\n";
			die;
		}	
	}
}



# =============================
# Function: get repeat from genome
# according to RepeatMasker annotation
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub getRepeatMultiThreads {
	
	my $RepeatMasker = shift or die;
	my $genome_fasta = shift or die;
	my $repeat_fasta = shift or die;
	my $threads_num  = shift or die;
	
		
	unless(-e $RepeatMasker){
		print STDERR "Error:$RepeatMasker not exists\n";
		die;
	}
	
	unless(-e $genome_fasta){
		print STDERR "Error:$genome_fasta not exists\n";
		die;
	}
	
	my %hash_parse = ();
	&parser($RepeatMasker,\%hash_parse);
	
	
	my %hash_genome = ();
	MyFasta::readfasta($genome_fasta,\%hash_genome);
	
	
	my %hash_repeat = ();
	my @chroms = sort keys %hash_genome;
	
	
	while ( @chroms ) {
		my @tmp = ();
		for my $c ( $threads_num ){
			push(@tmp,shift @chroms);
		}
		my @threads = ();
		for my $chrom ( @tmp ) {
			my $t = threads->new(\&extract_seq,\%hash_repeat,\%hash_parse,\%hash_genome,$chrom);
			push(@threads,$t);
		}
		foreach my $thread (@threads) {
			my $num = $thread->join;
		}
	}
	
	open REPEAT,">$repeat_fasta" or die;
	for my $id ( sort keys %hash_repeat ){
		print REPEAT ">$id\n$hash_repeat{$id}";
	}
	close REPEAT;
	
}

############################################################
1;
	