#!/usr/bin/perl
use strict;
use warnings;
use Cwd;

# My Library
use MyFile;
use MyFasta;
use refGene;
use RepeatMasker;

my $usage = "$0 < miRBase species name > [ local dir ]";
my $species = shift or die $usage;
my $download_dir = shift; 


# ++++++++++++++ UCSC database +++++++++++++++++++++


my $genome 				= "danRer7.fa.gz";
my $RepeatMasker 		= "danRer7.fa.out.gz";
my $md5sum 				= "md5sum.txt";
my $refMrna 			= "refMrna.fa.gz";
my $refMrna_md5 		= "refMrna.fa.gz.md5";
my $refGene			  	= "refGene.txt.gz";
my $UCSC_download_url 	= "ftp://hgdownload.cse.ucsc.edu/goldenPath/danRer7/bigZips/";
 

# ++++++++++++++ Rfam database +++++++++++++++++++++


my $Rfam = "Rfam.fasta.gz";
my $Rfam_download_url = "ftp://ftp.sanger.ac.uk/pub/databases/Rfam/11.0/";
 
 
# ++++++++++++++ miRBase database +++++++++++++++++++++ 
 
 
my $mature	= "mature.fa.gz";
my $hairpin	= "hairpin.fa.gz";
my $gff3	= "genomes/$species.gff3";
my $miRBase_download_url = "ftp://mirbase.org/pub/mirbase/CURRENT/";


# +++++++++++++++++++++++++++++++++++++++++++++++++++++ 


my ($md5,$genome_local,$md5sum_local,$RepeatMasker_local);
my ($refMrna_local,$refGene_local,$gff3_local,$refMrna_md5_local);
my ($Rfam_local,$mature_local,$hairpin_local);


unless( defined $download_dir ){
	$download_dir = getcwd;
	$download_dir .="/Data";
}


$download_dir = MyFile::getAbsPath($download_dir);
MyFile::makeDir($download_dir);


my $genome_fasta = "$download_dir/$species.genome.fa";
my $repeat_fasta = "$download_dir/$species.repeat.fa";
my $mRNA_fasta 	 = "$download_dir/$species.mRNA.fa";
my $cds_fasta  	 = "$download_dir/$species.cds.fa";
my $UTR3_fasta 	 = "$download_dir/$species.3UTR.fa";


# +++++++++++++++++++++++++++++++++++++++++++++++++++++


my $start_time = MyFile::start();

print STDERR "\n\n\nDownload required data ...\n\n\n";


# +++++++++++++++++++++++++++++++++++++++++++++++++++++


$genome_local = MyFile::wget($download_dir,"$UCSC_download_url$genome");
$md5sum_local = MyFile::wget($download_dir,"$UCSC_download_url$md5sum");
$md5 = MyFile::check_md5($genome_local,$md5sum_local);
die unless $md5;


$RepeatMasker_local = MyFile::wget($download_dir,"$UCSC_download_url$RepeatMasker");
$md5 = MyFile::check_md5($RepeatMasker_local,$md5sum_local);
die unless $md5;

	
$refMrna_local = MyFile::wget($download_dir,"$UCSC_download_url$refMrna");
$refMrna_md5_local = MyFile::wget($download_dir,"$UCSC_download_url$refMrna_md5");
$md5 = MyFile::check_md5($refMrna_local,$refMrna_md5_local);
die unless $md5;



my @tmp = split/\//,$UCSC_download_url;
pop( @tmp );
$UCSC_download_url  = join("/",@tmp);
$UCSC_download_url .= "/database/";
$refGene_local = MyFile::wget($download_dir,"$UCSC_download_url$refGene");


# +++++++++++++++++++++++++++++++++++++++++++++++++++++


$Rfam_local = MyFile::wget($download_dir,"$Rfam_download_url$Rfam");


# +++++++++++++++++++++++++++++++++++++++++++++++++++++


$mature_local = MyFile::wget($download_dir,"$miRBase_download_url$mature");
$hairpin_local = MyFile::wget($download_dir,"$miRBase_download_url$hairpin");
$gff3_local = MyFile::wget($download_dir,"$miRBase_download_url$gff3");


# +++++++++++++++++++++++++++++++++++++++++++++++++++++


$genome_local = MyFile::do_unpack($genome_local);
rename($genome_local,$genome_fasta);


$refMrna_local = MyFile::do_unpack($refMrna_local);
rename($refMrna_local,$mRNA_fasta);


$RepeatMasker_local = MyFile::do_unpack($RepeatMasker_local);


$refGene_local = MyFile::do_unpack($refGene_local);


$Rfam_local = MyFile::do_unpack($Rfam_local);


$mature_local = MyFile::do_unpack($mature_local);


$hairpin_local = MyFile::do_unpack($hairpin_local);



# +++++++++++++++++++++++++++++++++++++++++++++++++++++



print STDERR "\n\nget sequences of pre-miRNA and miRNA of $species\n\n\n";
my $this_species_mature = "$download_dir/$species.mature.fa";
my $this_species_precursor = "$download_dir/$species.hairpin.fa";
MyFasta::extractSeqInMiRBase($species,$mature_local,$hairpin_local,
$this_species_mature,$this_species_precursor);



print STDERR "\n\nget repeat from genome according to .out file of RepeatMasker\n\n";
my %hash_parse = ();
RepeatMasker::parser($RepeatMasker_local,\%hash_parse);
RepeatMasker::getRepeat($genome_fasta,$repeat_fasta,\%hash_parse);



my %hash_genome = ();
MyFasta::readfasta($genome_fasta,\%hash_genome);



refGene::get_exon_CDS_UTR($refGene_local,\%hash_genome,$cds_fasta,"CDS");



refGene::get_exon_CDS_UTR($refGene_local,\%hash_genome,$UTR3_fasta,"3UTR");



# +++++++++++++++++++++++++++++++++++++++++++++++++++++


MyFile::end($start_time);


print "\n\n\n################## END ################\n\n\n";



