#!/usr/bin/perl
use strict;
use warnings;
use Cwd;
use File::Copy;


# My Library
use MyFile;




my $continue_install = 0;
$continue_install = shift;




# ******************************** #
# global variables
# ******************************** #


my $install_path  = "Packages";
my $LIBSVM_url 	  = "http://www.csie.ntu.edu.tw/~cjlin/libsvm/oldfiles/libsvm-3.16.zip";
my $GNUPLOT_url   = "http://pkgs.fedoraproject.org/repo/pkgs/gnuplot/gnuplot-4.6.1.tar.gz/4c9a06461f402482c30cf94e267eb877/gnuplot-4.6.1.tar.gz";
my $miRDeep2_url  = "--no-check-certificate https://www.mdc-berlin.de/36381670/en/research/research_teams/systems_biology_of_gene_regulatory_elements/projects/miRDeep/older_miRDeep2_versions/mirdeep2_0_0_1.zip";
my $Mireap_url	  = "http://jaist.dl.sourceforge.net/project/mireap/mireap/mireap_0.2/mireap_0.2.tar.gz";
my $local_lib_url = "http://search.cpan.org/CPAN/authors/id/A/AP/APEIRON/local-lib-1.004009.tar.gz";
my $ViennaRNA_url = "http://www.tbi.univie.ac.at/~ivo/RNA/ViennaRNA-1.8.5.tar.gz";
my $PDF_API2_url  = "http://search.cpan.org/CPAN/authors/id/A/AR/AREIBENS/PDF-API2-0.73.tar.gz";






print STDERR "\n

###############################################################
#
# This is the MVP installer. 
# The installer will attempt to automatically set up a number 
# of dependent softwares, but this may fail due to permission 
# settings. In this case you will have to install them
# manually according to their installation instructions
# Then you have to specify their installation path when
# running MVP.pl
# 
# The dependent softwares as following:
# LIBSVM
# Mireap
# miRDeep2
# Bowtie
# RNAfold (ViennaRNA)
# 
###############################################################

";

sleep(10);



my $dir =`pwd 2>&1`;
chomp $dir;
$install_path = "$dir/$install_path";
my @install_cmds = ();



# ++++++++++++++++++++++++++++++ #
unless ( $continue_install ) {
# ++++++++++++++++++++++++++++++ #	
	
	
	# ******************************** #
	# make directorys
	# ******************************** #


	unless(-d $install_path){
		MyFile::makeDir($install_path);
	}

	




	print STDERR "\n

	# ******************************** #
	# check install environment
	# ******************************** #

	";


	# check shell
	my $shell = $ENV{"SHELL"};
	unless($shell=~/bash/){
		print STDERR "Error:please work under bash shell\n\n\n";
		die;
	}

	# check gcc
	my $gcc =`gcc --version 2>&1`;
	if($gcc !~ /(GCC)/i){
	    die "\nError:\n\tno gcc compiler installed. Please install a gcc compiler\n";
	}else{
	    if($gcc =~ /^gcc\s*\S*\s*(\d+\S+)\s*/){
	        print STDERR "gcc version: $1  already installed, nothing to do ...\n";
	    }
	} 

	# check g++
	my $gpp =`g++ --version 2>&1`;
	if($gpp !~ /(g\+\+)/i){
	    die "\nError:\n\tno g++ compiler installed. Please install a g++ compiler\n";
	}else{
	    if($gpp =~ /^g\+\+\s*\S*\s*(\d+\S+)\s*/){
	        print STDERR "g++ version: $1  already installed, nothing to do ...\n";
	    }
	} 



	MyFile::bashrc_backup(".bashrc","mirdeep2");
	MyFile::bashrc_backup(".bash_profile","mirdeep2");
	MyFile::bashrc_backup(".bashrc","mireap");
	MyFile::bashrc_backup(".bash_profile","mireap");



	print STDERR "\n

	# ******************************** #
	# install LIBSVM
	# ******************************** #

	";


	push(@install_cmds,"make");

	my $libsvm_dir = MyFile::installer($install_path,$LIBSVM_url,\@install_cmds);

	@install_cmds = ();

	
	print STDERR "\n
		
	# ******************************** #
	# install miRDeap2
	# ******************************** #

	";



	push(@install_cmds,"perl install.pl");
	MyFile::installer($install_path,$miRDeep2_url,\@install_cmds,'cd'=>"mirdeep2");
	@install_cmds = ();





	print STDERR "\n

	# ******************************** #
	# install Mireap
	# ******************************** #

	";




	@install_cmds = ();
	my $Mireap_dir = MyFile::installer($install_path,$Mireap_url,\@install_cmds);
	@install_cmds = ();

	my $mireap_set_env_path = "$Mireap_dir/lib";
	my $in = `grep "$mireap_set_env_path" ~/.bashrc`;
	if(not $in){
	    `echo 'export PERL5LIB=$mireap_set_env_path' >> ~/.bashrc`;
	}

	$in = `grep "$mireap_set_env_path" ~/.bash_profile`;
	if(not $in){
	    `echo 'export PERL5LIB=$mireap_set_env_path' >> ~/.bash_profile`;
	}

	unless(-d "$Mireap_dir/bin"){
		print STDERR "Error:bin not exists\ninstall Mireap unsuccessfully!\n\n\n";
		die;
	}

	unless(-d "$Mireap_dir/lib"){
		print STDERR "Error:lib not exists\ninstall Mireap unsuccessfully!\n\n\n";
		die;
	}



	if ( not ( MyFile::CheckModule("RNA") ) || not ( MyFile::CheckModule("API2") ) || 
		 not (-e "$install_path/mirdeep2/lib/PDF/API2.pm") ){
	
	
		print STDERR "\n

		###################################################################
		# 
		# Install local::lib now
		# During the installation,you may be asked to offer some infos.
		# If you are not clear, please press enter key
		# Maybe take a very long time, so be patient ....
		# 
		###################################################################

		";	
		
		sleep(10);
		
		
		my $in = `grep "export PERL5LIB=\$PERL5LIB:$install_path/lib/perl5" ~/.bashrc`;
		if(not $in){
			`echo 'export PERL5LIB=\$PERL5LIB:$install_path/lib/perl5' >> ~/.bashrc`;
		}
		$in = `grep "export PERL5LIB=\$PERL5LIB:$install_path/lib/perl5" ~/.bash_profile`;
		if(not $in){
		    `echo 'export PERL5LIB=\$PERL5LIB:$install_path/lib/perl5' >> ~/.bash_profile`;
		}
		
		
		push(@install_cmds,"perl Makefile.PL --bootstrap=$install_path");
		push(@install_cmds,"make test && make install");
		MyFile::installer($install_path,$local_lib_url,\@install_cmds);
		@install_cmds = ();
		
		
		$in = `grep 'eval \$(perl -I$install_path/lib/perl5 -Mlocal::lib=$install_path)' ~/.bashrc`;
		if(not $in){
		    `echo 'eval \$(perl -I$install_path/lib/perl5 -Mlocal::lib=$install_path)' >>~/.bashrc`;
		}
		
		unless(-d "$install_path/lib/perl5"){	
			print STDERR "\n\n\n$install_path/lib/perl5 not exists\n\n\n";
			die;	
		}
		
	}
	
	
	die if system("chmod -R 777 $install_path");


	print STDERR "\n

	##############################################################################
	#
	# Please verify that installation has been completed 
	# successfully by typing the following command:
	# source ~/.bashrc
	# sudo perl install.pl 1
	#
	##############################################################################

	";

	sleep(5);
	
	




# ++++++++++++++++++++++++++++++ #
} else {
# ++++++++++++++++++++++++++++++ #



	
	
	if ( not ( MyFile::CheckModule("RNA") ) ){
			
		print STDERR "\n
		
		######################################################
		#
		# Install ViennaRNA now
		#
		######################################################

		";
		
		sleep(10);
		
		push(@install_cmds,"./configure --prefix=$install_path");
		push(@install_cmds,"make");
		push(@install_cmds,"make install");
		
		MyFile::installer($install_path,$ViennaRNA_url,\@install_cmds);
		@install_cmds = ();
		
		my $module = "RNA";
		if ( MyFile::CheckModule($module) ) {
	        print STDERR "Ok!\n";
	    } else {
	    	print  STDERR "\n\n\nInstall $module error\n\n\n";
	    	die;
	    }

	}



	if ( not ( MyFile::CheckModule("API2") ) ) {
			
		print STDERR "\n
		
		######################################################
		#
		# Install required PDF::API2 now
		#
		######################################################

		";
		
		sleep(10);
		
		
		my @modules = qw( Compress::Raw::Zlib Compress::Raw::Bzip2 Compress::Zlib );
		MyFile::perl_module_installer($install_path,\@modules);
		
		
		if(-e "$install_path/mirdeep2/essentials/PDF-API2-0.73.tar.gz"){
			chdir("$install_path/mirdeep2/essentials");
		} else {
			chdir($install_path);
		}
		
		
		push(@install_cmds,"perl Makefile.PL");
		push(@install_cmds,"make");
		push(@install_cmds,"make install");
		
		MyFile::installer($install_path,$PDF_API2_url,\@install_cmds);
		@install_cmds = ();


		my $module = "PDF::API2";
		if ( MyFile::CheckModule($module) ) {
	        print STDERR "Ok!\n";
	    } else {
	    	print  STDERR "\n\n\nInstall $module error\n\n\n";
	    	die;
	    }
	    
	    
	    unless(-e "$install_path/mirdeep2/lib/API2.pm"){
	    	if ( -e "$install_path/lib/perl5/PDF" ){
				MyFile::copyDir("$install_path/lib/perl5/PDF","$install_path/mirdeep2/lib");
			}
		}
		
	}

	
	
	

	print STDERR "\n


	###############################################################
	#
	# Installation has been completed successfully
	# Plase start a new shell 
	# or type the following command:
	# source ~/.bashrc
	# 
	###############################################################

	";

	sleep(10);
	
	
}






















