package piRNA;
#!/usr/bin/perl
use strict;
use warnings;
use File::Basename;



# My Library
use Bowtie;




# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================	

sub parser {

	my $hash_mapping 	= shift or die;
	my $hash_piRNA 		= shift or die;
	my $outfile_prefix  = shift or die;

	
	my $dist = 1000;
	my $height = 10;
	my $MIN_LEN = 18;
	my $MAX_LEN = 35;
	my $multimapping = 1;
	my $unique_mapping = 0;
	my $parse_coor = 1;
	
	my @samples;
	
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/MIN_LEN/i) {$MIN_LEN=shift @_}
		if ($argument=~/MAX_LEN/i) {$MAX_LEN=shift @_}
		if ($argument=~/unique/i) {$unique_mapping=shift @_}
		if ($argument=~/multimapping/i) {$multimapping=shift @_}
		if ($argument=~/dist/i) {$dist=shift @_}
		if ($argument=~/height/i) {$height=shift @_}
		if ($argument=~/parse_coor/i) {$parse_coor=shift @_}
	}
	

	
	################################
	for my $file ( sort keys %$hash_mapping ){
	################################
			
		unless(-e $file){
			print STDERR "Error:$file not exists\n";
			die;
		}
		
		
		my $sample = $hash_mapping->{$file};
		push(@samples,$sample);
		
		
		# ============= get mapping =================
		
		my %hash_map = ();
		unless ( $unique_mapping ){
			Bowtie::parser9($file,\%hash_map,'MIN_LEN'=>$MIN_LEN,
			'MAX_LEN'=>$MAX_LEN,'multimapping'=>$multimapping);
		} else {
			Bowtie::parser10($file,\%hash_map,'MIN_LEN'=>$MIN_LEN,
			'MAX_LEN'=>$MAX_LEN,'multimapping'=>$multimapping);
		}
		
		
		# ================ get coordiante ===============
		
		my %hash_parser = ();
		if ( $parse_coor ){
			&parser_mapping(\%hash_map,\%hash_parser);
		} else {
			%hash_parser = %hash_map;
		}
		%hash_map = ();
	
		
		# ================ get cluster =================
		
		my %hash_cluster = ();
		&cluster(\%hash_parser,\%hash_cluster,'dist'=>$dist,'height'=>$height);
	
		
		for my $cluster_id ( keys %hash_cluster ){
			
			if (exists $hash_piRNA->{$sample}{$cluster_id}{num}){
				print STDERR "Error:($sample,$cluster_id) repeat\n";
				die;
			}
		
			$hash_piRNA->{$sample}{$cluster_id}{num} 		= $hash_cluster{$cluster_id}{num};
			$hash_piRNA->{$sample}{$cluster_id}{count} 		= $hash_cluster{$cluster_id}{count};
			$hash_piRNA->{$sample}{$cluster_id}{most_count} = $hash_cluster{$cluster_id}{most_count};
			$hash_piRNA->{$sample}{$cluster_id}{most_beg} 	= $hash_cluster{$cluster_id}{most_beg};
			$hash_piRNA->{$sample}{$cluster_id}{most_end} 	= $hash_cluster{$cluster_id}{most_end};
			
			
			for my $r ( sort {$a<=>$b} keys %{$hash_cluster{$cluster_id}{r_pos_beg}} ){
				my $freq = $hash_cluster{$cluster_id}{r_pos_beg}{$r};			
				$hash_piRNA->{$sample}{$cluster_id}{r_pos_beg}{$r} = $freq;
			}
			
			for my $r ( sort {$a<=>$b} keys %{$hash_cluster{$cluster_id}{r_pos_end}} ){
				my $freq = $hash_cluster{$cluster_id}{r_pos_end}{$r};
				$hash_piRNA->{$sample}{$cluster_id}{r_pos_end}{$r} = $freq;
			}
			
		}
	
	################################	
	}	# End parse
	################################
	

	
	################################
	for my $sample ( @samples ){
	################################
		
		my $cluster_sum = 0;
		my $cluster_plus_sum = 0;
		my $cluster_minus_sum = 0;
		
		my $most_count_sum = 0;
		my $total_count_sum = 0;
			
		open  CLUSTER,">$outfile_prefix$sample.cluster.txt\n";
		print CLUSTER "cluster_id\ttotal_count\tunique_piRNA_num\taverage_count\t";
		print CLUSTER "most_count\tmost_beg\tmost_end\tmost_count/total_count\n";
		
		
		################################
		for my $cluster_id ( keys %{$hash_piRNA->{$sample}} ){
		################################
		
			my $unique_num  = $hash_piRNA->{$sample}{$cluster_id}{num};
			my $total_count = $hash_piRNA->{$sample}{$cluster_id}{count};
			my $most_count  = $hash_piRNA->{$sample}{$cluster_id}{most_count};
			my $most_beg 	= $hash_piRNA->{$sample}{$cluster_id}{most_beg};
			my $most_end 	= $hash_piRNA->{$sample}{$cluster_id}{most_end};
			
			
			my $r = MyMath::round($most_count/$total_count,6);
			my $av = MyMath::round($total_count/$unique_num,6);
			
			print CLUSTER "$cluster_id\t$total_count\t$unique_num\t$av\t$most_count\t$most_beg\t$most_end\t$r\n";
			
			# ===========================
			
			$total_count_sum += $total_count;
			$most_count_sum += $most_count;
			
			my @tmp 	= split/:/,$cluster_id;
			my $chrom 	= shift @tmp;
			my $beg 	= shift @tmp;
			my $end		= shift @tmp;
			my $strand 	= shift @tmp;
			
			$cluster_sum++;
			
			if ( $strand eq "+" ){
				$cluster_plus_sum++;
			} else {
				$cluster_minus_sum++;
			}
			
		################################
		}	# End cluster
		################################
		
			
		my $r = MyMath::round($most_count_sum/$total_count_sum,6);
		my $av = MyMath::round($total_count_sum/$cluster_sum,6);
		
		print CLUSTER "//\n\n";
		print CLUSTER "cluster_num\tcluster_plus_sum\tcluster_minus_sum\ttotal_count/cluster_num\t";
		print CLUSTER "most_count_sum\ttotal_count_sum\tmost_count_sum/total_count_sum\n";
		print CLUSTER "$cluster_sum\t$cluster_plus_sum\t$cluster_minus_sum\t$av\t";
		print CLUSTER "$most_count_sum\t$total_count_sum\t$r\n";
		close CLUSTER;
	
	################################		
	}	# End samples
	################################
	
}



	
# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parser_mapping {
	
	my $hash_map 	= shift or die;
	my $hash_parser = shift or die;
	
	for my $piR_id ( keys %$hash_map ){
		
		my @tmp 	= split /:/,$piR_id;
		my $strand 	= pop( @tmp );
		my $end 	= pop( @tmp );
		my $beg 	= pop( @tmp );
		my $chrom 	= pop( @tmp );
		my $lng 	= $end - $beg + 1;
		
		if ( $strand =~ /Plus/i ){
			$strand = "+";
		} else {
			$strand = "-";
		}
		
		for my $strand_m ( keys %{$hash_map->{$piR_id}} ){
		for my $relative_beg ( keys %{$hash_map->{$piR_id}{$strand_m}} ){
		for my $relative_end ( keys %{$hash_map->{$piR_id}{$strand_m}{$relative_beg}} ){
			
			my ($abs_pos_beg,$abs_pos_end) = 
			MyFasta::getCoordinateRelativeLocation($strand,
			$beg,$end,$relative_beg,$relative_end);
			
			my $count = $hash_map->{$piR_id}{$strand_m}{$relative_beg}{$relative_end};
			
			unless(exists $hash_parser->{$chrom}{$strand}{$abs_pos_beg}{$abs_pos_end}){
				$hash_parser->{$chrom}{$strand}{$abs_pos_beg}{$abs_pos_end} = $count;
			} else {
				$hash_parser->{$chrom}{$strand}{$abs_pos_beg}{$abs_pos_end}+= $count;
			}
			
		}}}
		
	}
	
}
	
	
	

# =============================
# block: if mapped reads aligned to the same strand
# with in 30 nucleotides of each other,they were blocked.
# Reference: mirdeep

# cluster: We defined piRNA clusters as genomic fragments
# encoding at least 10 unique piRNAs with gaps no greater than 1kb
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub cluster {
	
	my $hash_map = shift or die;
	my $hash_piRNA_cluster = shift or die;
	
	
	my $dist = 1000;
	my $height = 10;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/dist/i) {$dist=shift @_}
		if ($argument=~/height/i) {$height=shift @_}
	}
	
	
		
	my $cluster_id;
	my %hash_end = ();
	my %hash_cluster = ();
	
	for my $chrom ( sort keys %$hash_map ){
	for my $strand ( sort keys %{$hash_map->{$chrom}} ){
		
		for my $beg ( sort {$a <=> $b} keys %{$hash_map->{$chrom}{$strand}} ){	
		for my $end ( sort {$a <=> $b} keys %{$hash_map->{$chrom}{$strand}{$beg}} ){
			
			my $count = $hash_map->{$chrom}{$strand}{$beg}{$end};
			unless( defined $count ){
				print STDERR "Error:($chrom:$strand:$beg:$end)\n";
				die;
			}
			
			unless( exists $hash_end{$chrom}{$strand} ){
				
				$hash_end{$chrom}{$strand} = $end;
				
				$cluster_id = "$chrom:$strand:$beg";
				$hash_cluster{$cluster_id}{num} = 1;
				$hash_cluster{$cluster_id}{count} = $count;
				$hash_cluster{$cluster_id}{most_count} = $count;
				$hash_cluster{$cluster_id}{most_beg} = $beg;
				$hash_cluster{$cluster_id}{most_end} = $end;
				$hash_cluster{$cluster_id}{beg} = $beg;
							
			} elsif ( $beg <= $hash_end{$chrom}{$strand} + $dist ){
				
				$hash_cluster{$cluster_id}{num}++;
				$hash_cluster{$cluster_id}{count} += $count;
				
				if ( $count > $hash_cluster{$cluster_id}{most_count} ){ 
					$hash_cluster{$cluster_id}{most_count} = $count;
					$hash_cluster{$cluster_id}{most_beg} = $beg;
					$hash_cluster{$cluster_id}{most_end} = $end;
				}
					
			} elsif ( $hash_end{$chrom}{$strand} + $dist < $beg ){
				
				if ( $hash_cluster{$cluster_id}{num} >= $height ){
					$hash_cluster{$cluster_id}{end} = $hash_end{$chrom}{$strand};
				}
				
				# ========= New cluster ============
				
				$cluster_id = "$chrom:$strand:$beg";				
				$hash_cluster{$cluster_id}{num} = 1;
				$hash_cluster{$cluster_id}{count} = $count;
				$hash_cluster{$cluster_id}{most_count} = $count;
				$hash_cluster{$cluster_id}{most_beg} = $beg;
				$hash_cluster{$cluster_id}{most_end} = $end;			
				$hash_cluster{$cluster_id}{beg} = $beg;
							
			}
				
			if ( $end > $hash_end{$chrom}{$strand} ){
				$hash_end{$chrom}{$strand} = $end;
			}
			
		}}
		
		if ( $hash_cluster{$cluster_id}{num} >= $height ){
			$hash_cluster{$cluster_id}{end} = $hash_end{$chrom}{$strand};
		}
			
	}}
	
	
	#########################################################
				
				
	for my $cluster_id ( sort keys %hash_cluster ){
		
		if( exists $hash_cluster{$cluster_id}{end} ){
			
			my @tmp 	= split /:/,$cluster_id;
			my $chrom 	= shift @tmp;
			my $strand 	= shift @tmp;
			my $beg 	= shift @tmp;
			my $end		= $hash_cluster{$cluster_id}{end};
			my $new_id 	= "$chrom:$beg:$end:$strand";
						
			unless(exists $hash_piRNA_cluster->{$new_id}){
				
				$hash_piRNA_cluster->{$new_id}{num} = $hash_cluster{$cluster_id}{num};
				$hash_piRNA_cluster->{$new_id}{count} = $hash_cluster{$cluster_id}{count};
				$hash_piRNA_cluster->{$new_id}{most_count} = $hash_cluster{$cluster_id}{most_count};
				$hash_piRNA_cluster->{$new_id}{most_beg} = $hash_cluster{$cluster_id}{most_beg};
				$hash_piRNA_cluster->{$new_id}{most_end} = $hash_cluster{$cluster_id}{most_end};
				
				my $most_beg = $hash_cluster{$cluster_id}{most_beg};
				my $most_end = $hash_cluster{$cluster_id}{most_end};
				
				if ($most_beg >= $most_end ){
					print STDERR "Error:$new_id\t$most_beg\t$most_end\n";
					die;
				}
				if ($most_beg >= $end ){
					print STDERR "Error:$new_id\t$most_beg\t$most_end\n";
					die;
				}
				if ($most_beg < $beg ){
					print STDERR "Error:$new_id\t$most_beg\t$most_end\n";
					die;
				}
				if ($most_end <= $beg ){
					print STDERR "Error:$new_id\t$most_beg\t$most_end\n";
					die;
				}
				if ($most_end > $end ){
					print STDERR "Error:$new_id\t$most_beg\t$most_end\n";
					die;
				}
				
			} else {
				
				print STDERR "Error:$new_id repeat\n";
				die;
				
			}
		}
	}
	
}
	
################################################################
1;

	


