package refGene;
#!/usr/bin/perl
use strict;
use warnings;


# My Library
use MyFasta;



# =============================
# Function:get exon, CDS, 5' UTR and 3' UTR
# sequences from genome 
# according to refGene.txt
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub get_exon_CDS_UTR {
	
	my $refGene 	= shift or die;
	my $hash_genome = shift or die;
	my $outfile		= shift or die;
	my $exon_CDS_UTR= shift or die;

	
	unless(-e $refGene){
		print STDERR "Error:$refGene not exists\n";
		die;
	}
	
	
	if ( $exon_CDS_UTR =~ /^CDS$/i ){
		print "\nExtract CDS sequences ...\n\n";
	} elsif ( $exon_CDS_UTR =~ /^3UTR$/i ){
		print "\nExtract 3' UTR sequences ...\n\n";
	} elsif ( $exon_CDS_UTR =~ /^5UTR$/i ){
		print "\nExtract 5' UTR sequences ...\n\n";
	} else {
		print "\nExtract exon sequences ...\n\n";
	}
	
	
	my %hash_info = ();

	open IN,"<$refGene" or die;
	while( <IN> ){
		chomp;
		if(/\S+\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/){
			
			my $name		= $1;
			my $chrom		= $2;
			my $strand		= $3;
			my $txStart		= $4;
			my $txEnd		= $5;
			my $cdsStart	= $6;
			my $cdsEnd		= $7;
			my $exonCount	= $8;
			my $exonStarts 	= $9;
			my $exonEnds	= $10;
					
			my @begs = split/,/,$exonStarts;
			my @ends = split/,/,$exonEnds;
			
			my $n = (scalar @begs) - 1;
			for my $i ( 0 .. $n ){ 
				my $beg = $begs[$i];
				my $end = $ends[$i];
				my $tmp = [ $beg, $end, $cdsStart, $cdsEnd, $exonCount ];
				push @{$hash_info{$chrom}{$strand}{$name}},$tmp;
			}

		}
	}
	close IN;
	
	
	open SEQ,">$outfile" or die;
	
	for my $chrom ( sort keys %hash_info ){
	for my $strand ( sort keys %{$hash_info{$chrom}} ){
	for my $name ( sort keys %{$hash_info{$chrom}{$strand}} ){	
		
		
		my (@tmp_begs,@tmp_ends,@out_begs,@out_ends) = ();
		my $coordinates = $hash_info{$chrom}{$strand}{$name};
		for my $r ( @$coordinates ){
			my $beg 		= $r->[0];
			my $end 		= $r->[1];
			my $cdsStart	= $r->[2];
			my $cdsEnd 		= $r->[3];
			my $exonCount 	= $r->[4];
			
			push(@tmp_begs,$beg);
			push(@tmp_ends,$end);
			
			if ( (scalar @tmp_begs) == $exonCount ){
				
				if ( $exon_CDS_UTR =~ /^CDS$/i ){	
					&CDS($cdsStart,$cdsEnd,\@tmp_begs,\@tmp_ends,\@out_begs,\@out_ends);
				} elsif ( $exon_CDS_UTR =~ /^3UTR$/i ){
					&UTR_3($cdsStart,$cdsEnd,$strand,\@tmp_begs,\@tmp_ends,\@out_begs,\@out_ends);
				} elsif ( $exon_CDS_UTR =~ /^5UTR$/i ){
					&UTR_5($cdsStart,$cdsEnd,$strand,\@tmp_begs,\@tmp_ends,\@out_begs,\@out_ends);
				} else {
					for $beg ( @tmp_begs ){
						push(@out_begs,$beg);
					}
					for $end ( @tmp_ends ){
						push(@out_ends,$end);
					}
				}
				
				my @begs = @out_begs;
				my @ends = @out_ends;
				
				my $k = (scalar @begs) - 1;
				my $m = (scalar @ends) - 1;
				my $n = $k < $m ? $k : $m;
				
				for my $i ( 0 .. $n ){ 	
					
					my $beg = $begs[$i] + 1;
					my $end = $ends[$i];		

					next if $beg >= $end + 1;	
					#print "$name\t$chrom\t$strand\t$beg\t$end\n";
						
					if ( exists $hash_genome->{$chrom} ){
						my $seq = $hash_genome->{$chrom};
						my $extract_seq = MyFasta::getSeqByCoordinate($seq,$beg,$end,$strand);
			
						my $j = 0;
						my $str = "";
						while( $j < length($extract_seq) ){		
							my $subseq = substr($extract_seq,$j,50);
							$str .= "$subseq\n";
							$j += 50;
						}
						
						$j = $i;
						$j = $n - $i if $strand eq "-";
						my $id  = "$name:$j:$chrom:$beg:$end:$strand";
						print SEQ ">$id\n$str";
					} 
				}
				
				@tmp_begs = ();
				@tmp_ends = ();	
				@out_begs = ();
				@out_ends = ();
				
			}
		}
		
	}}}
	
	close SEQ;

}		





# =============================
# Function:get CDS sequences from 
# genome according to refGene.txt
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub CDS {
	
	my $cdsStart = shift;
	my $cdsEnd	 = shift;
	my $in_begs  = shift or die;
	my $in_ends  = shift or die;
	my $out_begs = shift or die;
	my $out_ends = shift or die;
	
	
	
	my $n = ( scalar @$in_begs ) - 1;
	for my $i ( 0 .. $n ){	
		my $beg = $in_begs->[$i];
		if ( $beg > $cdsStart && $beg < $cdsEnd ){
			push(@$out_begs,$beg);
		} 		
	}
	unshift (@$out_begs,$cdsStart);
	

	my $m = ( scalar @$in_ends ) - 1;
	for my $i ( 0 .. $m ){		
		my $end = $in_ends->[$i];
		if ( $end > $cdsStart && $end < $cdsEnd ){
			push(@$out_ends,$end);
		} 		
	}
	push (@$out_ends,$cdsEnd);
	

	
}




# =============================
# Function:get 3' UTR sequences from 
# genome according to refGene.txt
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub UTR_3 {
	
	
	my $cdsStart = shift;
	my $cdsEnd	 = shift;
	my $strand	 = shift or die;
	my $in_begs  = shift or die;
	my $in_ends  = shift or die;
	my $out_begs = shift or die;
	my $out_ends = shift or die;
	
			
	my $n = ( scalar @$in_begs ) - 1;
	if ( $strand eq "+" ){
		for my $i ( 0 .. $n ){		
			my $beg = $in_begs->[$i];
			if ( $beg > $cdsEnd ){
				push(@$out_begs,$beg);
			} 		
		}
		unshift (@$out_begs,$cdsEnd);
	} else {
		for my $i ( 0 .. $n ){		
			my $beg = $in_begs->[$i];
			if ( $beg < $cdsStart ){
				push(@$out_begs,$beg);
			} 		
		}
	}
	
			
	my $m = ( scalar @$in_ends ) - 1;
	if ( $strand eq "+" ){	
		for my $i ( 0 .. $m ){		
			my $end = $in_ends->[$i];
			if ( $end >= $cdsEnd ){
				push(@$out_ends,$end);
			} 		
		}
	} else {
		for my $i ( 0 .. $m ){		
			my $end = $in_ends->[$i];
			if ( $end < $cdsStart ){
				push(@$out_ends,$end);
			} 		
		}
		push(@$out_ends,$cdsStart);
	}
	
}




# =============================
# Function:get 3' UTR sequences from 
# genome according to refGene.txt
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub UTR_5 {
	
	my $cdsStart = shift;
	my $cdsEnd	 = shift;
	my $strand	 = shift or die;
	my $in_begs  = shift or die;
	my $in_ends  = shift or die;
	my $out_begs = shift or die;
	my $out_ends = shift or die;
	
	
	my $n = ( scalar @$in_begs ) - 1;
	if ( $strand eq "+" ){
		for my $i ( 0 .. $n ){		
			my $beg = $in_begs->[$i];
			if ( $beg < $cdsStart ){
				push(@$out_begs,$beg);
			} 		
		}
	} else {
		for my $i ( 0 .. $n ){		
			my $beg = $in_begs->[$i];
			if ( $beg > $cdsEnd ){
				push(@$out_begs,$beg);
			} 		
		}
		unshift(@$out_begs,$cdsEnd);
	}
	
			
	my $m = ( scalar @$in_ends ) - 1;
	if ( $strand eq "+" ){	
		for my $i ( 0 .. $m ){		
			my $end = $in_ends->[$i];
			if ( $end < $cdsStart ){
				push(@$out_ends,$end);
			} 		
		}
		push(@$out_ends,$cdsStart);
	} else {
		for my $i ( 0 .. $m ){		
			my $end = $in_ends->[$i];
			if ( $end >= $cdsEnd ){
				push(@$out_ends,$end);
			} 		
		}
	}
	
}



############################################################
1;