#!/usr/bin/perl
use strict;
use warnings;
use Cwd;
use File::Copy;
use File::Basename;
use Getopt::Long  qw(:config bundling);

# My Library
use MyFasta;
use MyFile;
use Feature;
use MyMath;
use Libsvm;




# ***************************************** #
my $program		= $0;
my $help 		= 0;
my $n_bin 		= 10;
# *************** Paths/Files ************* #
my $dir;
my $LIBSVM;
my $posFasta;
my $parsedOut;
my $roc_curve;
my $svm_model;
my $scaled_model;
# *********** Default Feature ************* #
my $mfe_index_1_flag 					= 1;
my $mfe_index_5_flag					= 1;
my $pb_ratio_flag 						= 1;
my $unPb_ratio_flag 					= 1;
my $bp_propensity_flag 					= 1;
my $bp2UnPb_ratio_flag 					= 1;
my $normalized_mfe_flag 				= 1;
my $normalized_n_bulges_flag 			= 1;
my $mfe_index_3_flag 					= 1;
my $n_bulges_flag 						= 1;
my $mfe_flag							= 1;
my $mfe_index_2_flag 					= 1;
my $avg_bp_stem_flag 					= 1;
my $max_consecutive_bp_falg				= 1;
my $avg_unbp_bulge_flag 				= 1;
my $mfe_index_4_flag 					= 1;
my $GCc_flag 							= 1;
my $dinucleotide_flag 					= 1;
my $triplet_flag						= 1;
# **************** Feature **************** #
my $binary_flag							= 0;
my $rbp2UnpB_dis_flag 					= 0;
my $max_bulge_size_flag					= 0;
my $asymmetry_size_flag					= 0;
my $n_stems_flag 						= 0;
my $n_loops_flag 						= 0;
my $normalized_n_stems_flag 			= 0;
my $normalized_n_loops_flag 			= 0;
my $normalized_bp_count_flag 			= 0;
my $normalized_bp_count_per_stem_flag 	= 0;
my $reverse_align_binary_flag 			= 0;
# ***************************************** #


	
GetOptions( 	

	"help|h!"							=> \$help,

	"dir|D=s"							=> \$dir,
	"posF|P=s"							=> \$posFasta,
	"LIBSVM|L=s"						=> \$LIBSVM,

	"SVMmodel|M=s"						=> \$svm_model,
	"scaleModel|H=s"					=> \$scaled_model,
	
);


# ********** Required paramters *********** #

if ( $help ) {
	&usage(); exit;
}


unless( defined $posFasta ){
	print STDERR "Error:positive samples not spcify\n";
	print STDERR "You can specify it by typing in command:\n";
	print STDERR "-P xxx\n";
	&usage(); exit;
} else {
	$posFasta = MyFile::getAbsPath($posFasta);
}

# ********* Default Paths/Files ********** #

$program =~ s/\.pl$//i;
my $current_dir = getcwd;
unless( defined $dir ){
	$dir = "$current_dir/$program"."-".time;
} else {
	$dir = MyFile::getAbsPath($dir);
} 


unless( defined $LIBSVM ){
	$LIBSVM = "$current_dir/Packages/libsvm-3.16/";
} else {
	$LIBSVM = MyFile::getAbsPath($LIBSVM);
}


unless( defined $parsedOut ){
	$parsedOut = "$dir/out.txt";
} else {
	$parsedOut = MyFile::getAbsPath($parsedOut);
}


unless( defined $roc_curve ){
	$roc_curve = "$dir/roc.txt";
} else {
	$roc_curve = MyFile::getAbsPath($roc_curve);
}


unless( defined $svm_model ){
	$svm_model = "$dir/svm_model.txt";
} else {
	$svm_model = MyFile::getAbsPath($svm_model);
}


unless( defined $scaled_model ){
	$scaled_model = "$dir/scaled_model.txt";
} else {
	$scaled_model = MyFile::getAbsPath($scaled_model);
}


# ********** Check required files ********* #

unless(-x "$LIBSVM/svm-train"){
	print STDERR "Error:svm-train not exists in $LIBSVM\n";
	print STDERR "You can specify it by typing in command:\n";
	print STDERR "-L LIBSVM path\n";
	&usage(); exit;
}



########################################
# start 
########################################


MyFile::makeDir($dir);


my $parameter_file = "$dir/$program-log.txt";
open ZMIRP,">$parameter_file" or die;

my $start_time = MyFile::start(\*ZMIRP);

print "dir:$dir\n";

print ZMIRP "dir:$dir\n";


my $SVM_feature = "$dir/SVM-feature.txt";
open FEATURE,">$SVM_feature" or die;



#############################################################
# Positive
#############################################################


my %hash_positive_seq = ();
MyFasta::readfasta($posFasta,\%hash_positive_seq);


my $valided_positive_fasta = "$dir/valided-positive.fa";
open OUT,">$valided_positive_fasta";
for my $id ( sort keys %hash_positive_seq ){
	my $seq = $hash_positive_seq{$id};
	$seq = uc( $seq );
	$seq =~ tr/T/U/;
	$seq =~ s/^\s*//g;
	$seq =~ s/\s*$//g; 
	unless( not ($seq =~/^([A|C|G|U]+)$/) ){
		print OUT ">$id\n$seq\n";
	}
}
close OUT;



my $valided_positive_struct = "$dir/valided-positive-struct.txt";
die if system("perl fold_linux.pl $valided_positive_fasta $valided_positive_struct");



my %hash_feature = ();
&extract_feature($n_bin,1,$valided_positive_struct,\%hash_feature,\*FEATURE);



close FEATURE;



#############################################################
# Predict
#############################################################


my %hash_model = ();
$hash_model{$svm_model} = $scaled_model;
Libsvm::predict($LIBSVM,\%hash_model,$SVM_feature,$parsedOut,$roc_curve,$dir);




#############################################################
# End
#############################################################


MyFile::end($start_time,\*ZMIRP);

print "################### END ###################\n";
print ZMIRP "################### END ###################\n";
close ZMIRP;








######################################################################
######################################################################
######################################################################

sub extract_feature {
	
	my $n_bin = shift or die;
	my $lable = shift or die;
	my $struct = shift or die;
	my $hash_feature = shift or die;
	local *FEATURE = shift or die;
	
	
	my %hash_struct   = ();
	Feature::parse_struct($struct,\%hash_struct);
	
	my $num_featrure  = 0;	
	my %hash_check_id = ();
	for my $id ( sort keys %hash_struct ){
		
		my $mfe = $hash_struct{$id}{mfe};
		my $seq = $hash_struct{$id}{seq};
		my $struct = $hash_struct{$id}{struct};
		
		$seq = uc( $seq );
		$seq =~ tr/T/U/;
		$seq =~ s/^\s*//g;
		$seq =~ s/\s*$//g;
		if( not ($seq =~/^([A|C|G|U]+)$/) ){
			next;
		}
		
		my $new_id = $id;	
		$new_id =~ s/^\s*//g;
		$new_id =~ s/\s*$//g;
		if($new_id =~ /(\S+)/){
			$new_id = $1;
		}
		
		unless(exists $hash_check_id{$new_id}){
			$hash_check_id{$new_id} = 1;
		} else {
			print STDERR "Error:$new_id repeat\n";
			die;
		}
		
		my $n = 0;
		my $str = "$new_id\t$lable";		
		
		
		# ************* sequence feature ************** #
		
		# GC content
		if( $GCc_flag ){
			my $GCc = Feature::GC_content($seq);
			$GCc = MyMath::round($GCc,6);
			$str .= "\t$GCc";
			$hash_feature->{$n} = "GC_content";
			$n++;
		}


		
		# dinucleotide frequencies
		if( $dinucleotide_flag ){
			my @dinucleotide = ();
			my @dinucleotide_str = ();
			Feature::dinucleotide_frequency($seq,\@dinucleotide,\@dinucleotide_str);	
			my $ii = 0;
			for my $di ( @dinucleotide ){
				$di = MyMath::round($di,6);
				$str .= "\t$di";
				$hash_feature->{$n} = $dinucleotide_str[$ii];
				$n++;
				$ii++;
			}
		}

		
		
		# ************* structure feature ************** #


		
		# the distribution of unpaired base to paired base in secondary structure
		if ( $rbp2UnpB_dis_flag ){
			my @rbp2UnpB_distribution = ();
			Feature::unpaired2paired_distribution($struct,$n_bin,\@rbp2UnpB_distribution);
			my $ii = 0;
			for my $r ( @rbp2UnpB_distribution ){
				$r = MyMath::round($r,6);
				$str .= "\t$r";
				$hash_feature->{$n} = "$ii-rbp2UnpB_dis";
				$n++;
				$ii++;
			}
		}
		
				
		# the size of biggest bulge in structure
		if( $max_bulge_size_flag ){
			my $max_bulge_size = Feature::biggest_bulge_size($struct);
			$max_bulge_size = MyMath::round($max_bulge_size,6);
			$str .= "\t$max_bulge_size";
			$hash_feature->{$n} = "max_bulge_size";
			$n++;
		}
			
		
		# asymmetry of structure
		if ( $asymmetry_size_flag ){
			my $asymmetry = Feature::asymmetry($struct);
			$asymmetry = MyMath::round($asymmetry,6);
			$str .= "\t$asymmetry";
			$hash_feature->{$n} = "asymmetry_size";
			$n++;
		}
		
		
		# the maximum of consecutive paired base
		if( $max_consecutive_bp_falg ){
			my $max_consecutive_bp = Feature::biggest_consecutive_paired_base($struct);
			$max_consecutive_bp = MyMath::round($max_consecutive_bp,6);
			$str .= "\t$max_consecutive_bp";
			$hash_feature->{$n} = "max_consecutive_bp";
			$n++;
		}
		
		
		# ratio of paired base (rpb)
		if( $pb_ratio_flag ){
			my ($rpb,$rUnpB,$rbp2UnpB) = Feature::paired_unpaired_base($struct);
			$rpb = MyMath::round($rpb,6);
			$str .= "\t$rpb";
			$hash_feature->{$n} = "rpb";
			$n++;
		}
		
		
		# ratio of unpaired base (rUnpB)
		if ( $unPb_ratio_flag ){
			my ($rpb,$rUnpB,$rbp2UnpB) = Feature::paired_unpaired_base($struct);
			$rUnpB = MyMath::round($rUnpB,6);
			$str .= "\t$rUnpB";
			$hash_feature->{$n} = "rUnpB";
			$n++;
		}
		
		
		# And ratio of paired base to unpaired base (rbp2UnpB)
		if ( $bp2UnPb_ratio_flag ){
			my ($rpb,$rUnpB,$rbp2UnpB) = Feature::paired_unpaired_base($struct);
			$rbp2UnpB = MyMath::round($rbp2UnpB,6);
			$str .= "\t$rbp2UnpB";
			$hash_feature->{$n} = "rbp2UnpB";
			$n++;
		}
		
		
		# the number of stems
		if ( $n_stems_flag ){
			my $n_stems = Feature::count_n_stems($struct);
			$n_stems = MyMath::round($n_stems,6);
			$str .= "\t$n_stems";
			$hash_feature->{$n} = "n_stems";
			$n++;
		}
		
		
		# the number of loop
		if ( $n_loops_flag ){
			my $n_loops = Feature::count_n_loops($struct);
			$n_loops = MyMath::round($n_loops,6);
			$str .= "\t$n_loops";
			$hash_feature->{$n} = "n_loops";
			$n++;
		}
		
		
		# normalized stems
		if ( $normalized_n_stems_flag ){
			my $normalized_n_stems = Feature::normalized_n_stems($struct);
			$normalized_n_stems = MyMath::round($normalized_n_stems,6);
			$str .= "\t$normalized_n_stems";
			$hash_feature->{$n} = "normalized_n_stems";
			$n++;
		}		
		
		
		# normalized loops
		if ( $normalized_n_loops_flag ){
			my $normalized_n_loops = Feature::normalized_n_loops($struct);
			$normalized_n_loops = MyMath::round($normalized_n_loops,6);
			$str .= "\t$normalized_n_loops";
			$hash_feature->{$n} = "normalized_n_loops";
			$n++;
		}
		
		
		# Average base pairs per stem
		if ( $avg_bp_stem_flag ){
			my $avg_bp_stem = Feature::avg_bp_stem($struct);
			$avg_bp_stem = MyMath::round($avg_bp_stem,6);
			$str .= "\t$avg_bp_stem";
			$hash_feature->{$n} = "avg_bp_stem";
			$n++;
		}
		

		# Normalized base-paring propensity (dP)
		if( $bp_propensity_flag ){
			my $dP = Feature::normalized_base_paring_propensity($struct);
			$dP = MyMath::round($dP,6);
			$str .= "\t$dP";
			$hash_feature->{$n} = "dP";
			$n++;
		}
		
		
		# avg_unbp_bulge = tot_unpaired_bases / n_bulges
		if ( $avg_unbp_bulge_flag ){
			my $avg_unbp_bulge = Feature::avg_unbp_bulge($struct);
			$avg_unbp_bulge = MyMath::round($avg_unbp_bulge,6);
			$str .= "\t$avg_unbp_bulge";
			$hash_feature->{$n} = "avg_unbp_bulge";
			$n++;
		}
		
		
		# normalized_n_bulges = n_bulges / L
		if ( $normalized_n_bulges_flag ){
			my $normalized_n_bulges = Feature::normalized_n_bulges($struct);
			$normalized_n_bulges = MyMath::round($normalized_n_bulges,6);
			$str .= "\t$normalized_n_bulges";
			$hash_feature->{$n} = "normalized_n_bulges";
			$n++;
		}
		
		
		# the number of bulge
		if ( $n_bulges_flag ){
			my $n_bulges = Feature::count_n_bulges($struct);
			$n_bulges = MyMath::round($n_bulges,6);
			$str .= "\t$n_bulges";
			$hash_feature->{$n} = "n_bulges";
			$n++;
		}


		# ************** structure and sequence feature ************ #
		

		# Normalized base pair counts:|A-U|/L, |G-C|/L and |G-U|/L
		if ( $normalized_bp_count_flag ){
			my @nbpc = ();my @nbpc_stems = ();
			my @nbpc_str = (); my @nbpc_stems_str = ();
			Feature::normalized_base_pair_count($seq,$struct,
			\@nbpc,\@nbpc_stems,\@nbpc_str,\@nbpc_stems_str);
			my $ii = 0;
			for my $r ( @nbpc ){
				$r = MyMath::round($r,6);
				$str .= "\t$r";
				$hash_feature->{$n} = $nbpc_str[$ii]."/L";
				$n++;
				$ii++;
			}
		}
		
		
		# %(A-U)/n_stems, %(G-C)/n_stems and %(G-U)/n_stems (where %(X-Y) = |X-Y|/tot_bases)
		if( $normalized_bp_count_per_stem_flag ){
			my @nbpc = ();my @nbpc_stems = ();
			my @nbpc_str = (); my @nbpc_stems_str = ();
			Feature::normalized_base_pair_count($seq,$struct,
			\@nbpc,\@nbpc_stems,\@nbpc_str,\@nbpc_stems_str);
			my $ii = 0;
			for my $r ( @nbpc_stems ){
				$r = MyMath::round($r,6);
				$str .= "\t$r";
				$hash_feature->{$n} = "%(".$nbpc_stems_str[$ii].")/n_stems";
				$n++;
				$ii++;
			}
		}
		
		
		
		# triplet elements
		if ( $triplet_flag ){
			my @triplets = ();
			my @triplets_str = ();
			Feature::triplet($seq,$struct,\@triplets,\@triplets_str);
			my $ii = 0;
			for my $r ( @triplets ){
				$r = MyMath::round($r,6);
				$str .= "\t$r";
				$hash_feature->{$n} = $triplets_str[$ii];
				$n++;
				$ii++;
			}
		}


		# reverse tructure strings and character strings, then 
		# aligne the strings with their reversed counterparts.
		if ( $reverse_align_binary_flag ){
			my %hash_rabp = ();
			Feature::reverse_align_bp($seq,$struct,$n_bin,\%hash_rabp);		
			for my $i ( sort {$a<=>$b} keys %hash_rabp ){
				for my $j ( sort  keys %{ $hash_rabp{ $i } } ){
					my $r = $hash_rabp{ $i }{ $j };	
					$r = MyMath::round($r,6);		
					$str .= "\t$r";
					$hash_feature->{$n} = "$i$j";
					$n++;
				}
			}	
		}


		# binary elements
		if ( $binary_flag ){
			my @binary_array = ();
			my @binary_array_str = ();
			Feature::binary($seq,$struct,\@binary_array,\@binary_array_str);
			my $ii = 0;
			for my $r ( @binary_array ){
				$r = MyMath::round($r,6);
				$str .= "\t$r";
				$hash_feature->{$n} = $binary_array_str[$ii];
				$n++;
				$ii++;
			}
		}

		
		# ************** energy feature ************ #
		
		
		# minimum folding energy
		if ( $mfe_flag ){
			$mfe = MyMath::round($mfe,6);
			$hash_feature->{$n} = "mfe";
			$str .= "\t$mfe";
			$n++;
		}
			
		
		# dG = MFE / L
		if ( $normalized_mfe_flag ){
			my $normalized_mfe = Feature::normalized_mfe($struct,$mfe);
			 $normalized_mfe = MyMath::round($normalized_mfe,6);
			 $str .= "\t$normalized_mfe";
			 $hash_feature->{$n} = "normalized_mfe";
			 $n++;
		}
		
		
		
		# MFE Index 1 = dG / %(G+C) (where dG = MFE / L)
		if ( $mfe_index_1_flag ){
			my $mfe_index_1 = Feature::mfe_index_1($seq,$mfe);
			$mfe_index_1 = MyMath::round($mfe_index_1,6);
			$str .= "\t$mfe_index_1";
			$hash_feature->{$n} = "mfe_index_1";
			$n++;
		}
		
		
		# MFE Index 2 = dG / n_stems
		if ( $mfe_index_2_flag ){
			my $mfe_index_2 = Feature::mfe_index_2($struct,$mfe);
			$mfe_index_2 = MyMath::round($mfe_index_2,6);
			$str .= "\t$mfe_index_2";
			$hash_feature->{$n} = "mfe_index_2";
			$n++;
		}
		
		
		# MFE Index 3 = dG / n_loops
		if ( $mfe_index_3_flag ){
			my $mfe_index_3 = Feature::mfe_index_3($struct,$mfe);
			$mfe_index_3 = MyMath::round($mfe_index_3,6);
			$str .= "\t$mfe_index_3";
			$hash_feature->{$n} = "mfe_index_3";
			$n++;
		}
		
		
		# MFE Index 4 = dG / tot_bases
		if ( $mfe_index_4_flag ){
			my $mfe_index_4 = Feature::mfe_index_4($struct,$mfe);
			$mfe_index_4 = MyMath::round($mfe_index_4,6);
			$str .= "\t$mfe_index_4";
			$hash_feature->{$n} = "mfe_index_4";
			$n++;
		}
		
		
		# MFE Index 5 = dG / n_bulges
		if ( $mfe_index_5_flag ){
			my $mfe_index_5 = Feature::mfe_index_5($struct,$mfe);
			$mfe_index_5 = MyMath::round($mfe_index_5,6);
			$str .= "\t$mfe_index_5";
			$hash_feature->{$n} = "mfe_index_5";
			$n++;
		}
		

		
		
		
		# ***************************************** #
		
		print FEATURE "$str\n";
		
		unless( $num_featrure ){
			$num_featrure = $n;
		} elsif ( $num_featrure != $n ){
			print STDERR "Error:lack feature (>$id\n$seq\n$struct)\n";
			die;
		}
		
	}

}



# =============================
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub usage {
	
my $usage = << "USAGE";

Version: 1.0
Contact: Yao Yuangen <yygen89\@163.com>

Usage:

	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 

	Options:
	
	--LIBSVM	| -L [LIBSVM path] : the path to LIBSVM package. (optional,default:./Packages/libsvm-3.16/)			
	--posFasta	| -P <fasta> : query fasta file containing putative pre-miRNA sequences
	--SVMmodel	| -M <SVM model> : SVM model file containing parameters and support vectors
	--scaleModel	| -H <scaled model> : scaled model file containing parameters for scaling input data
	--dir		| -D [dir] : result directory containing predicted results (optional)
	--help		| -h : help
	
	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 

	$0 [options] -L [LIBSVM path] -P <fasta> -M <SVM model> -H <scaled model>

	+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ 

USAGE
print $usage;

}


