package miRNA;
#!/usr/bin/perl
use strict;
use warnings;
use File::Basename;

# My Library
use MyFasta;
use Bowtie;
use MyMath;



# =============================
# Function: find microRNA
# Input: mature fasta from miRBase
# 		 hairpin fasta from miRBase
#		 gff3 annotation file from miRBase
#		 mapped output from Bowtie
#		 outfile for profile
#		 reference of hash for parsed result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================


sub parser {
	
	my $bowtie_path		= shift or die;
	my $hash_parse 		= shift or die;
	my $hash_mature_tag = shift or die;
	my $infile_mature	= shift or die;
	my $infile_hairpin 	= shift or die;
	my $infile_mapping  = shift or die;
	my $hairpin_struct 	= shift or die;
	my $outfile_profile = shift or die;

	
	my $BEG_DIF  = -2;
	my $END_DIF  = 5;
	
	my $dest_dir;
	my $infile_gff3;
	my $annotation;
	
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/gff3/i) {$infile_gff3=shift @_}
		if ($argument=~/dest_dir/i) {$dest_dir=shift @_}
		if ($argument=~/annotation/i) {$annotation=shift @_}
		if ($argument=~/beg_dif/i) {$BEG_DIF=shift @_}
		if ($argument=~/end_dif/i) {$END_DIF=shift @_}
	}
	
	
	unless(-e $infile_mapping){
		print STDERR "Error:$infile_mapping not exists\n";
		die;
	}
	unless(-e $infile_mature){
		print STDERR "Error:$infile_mature not exists\n";
		die;
	}
	unless(-e $infile_hairpin){
		print STDERR "Error:$infile_hairpin not exists\n";
		die;
	}
	unless(-e $hairpin_struct){
		print STDERR "Error:$hairpin_struct not exists\n";
		die;
	}
	
	
	my %hash_mature = ();
	MyFasta::readfasta($infile_mature,\%hash_mature);
	
	
	my %hash_struct = ();
	&parse_struct($hairpin_struct,\%hash_struct);
	
	
	my %hash_precursor_mature = ();	
	unless(defined $annotation){
		if( defined $infile_gff3 ) {
			if( defined $dest_dir ){
				&mature2precursor(\%hash_precursor_mature,$bowtie_path,
				$infile_mature,$infile_hairpin,'gff3'=>$infile_gff3,'dest_dir'=>$dest_dir);
			} else {
				&mature2precursor(\%hash_precursor_mature,$bowtie_path,
				$infile_mature,$infile_hairpin,'gff3'=>$infile_gff3);
			} 
		} else {
			if( defined $dest_dir ){
				&mature2precursor(\%hash_precursor_mature,$bowtie_path,
				$infile_mature,$infile_hairpin,'dest_dir'=>$dest_dir);
			} else {
				&mature2precursor(\%hash_precursor_mature,
				$bowtie_path,$infile_mature,$infile_hairpin);
			}
		}
	} else {
		%hash_precursor_mature = %$annotation;
	}
		
		
	
	my %hash_precursor = ();
	MyFasta::readfasta($infile_hairpin,\%hash_precursor);
	

	my %hash_map = ();
	my %hash_tag = ();
	my %hash_tag_count = ();
	Bowtie::parser3($infile_mapping,\%hash_map,\%hash_tag,\%hash_tag_count);
	

	# ************************************************* #
	
	my %hash_profile = ();
	my %hash_mature_tag_same = ();
	my %hash_precursor_count = ();
	
	
	for my $precursor ( keys %hash_precursor_mature ){
	for my $mature ( keys %{ $hash_precursor_mature{$precursor} } ){
		
		my ($mbeg,$mend) = split/:/,$hash_precursor_mature{$precursor}{$mature};
		
		for my $tag ( keys %{ $hash_map{$precursor} } ){		
		for my $beg ( keys %{ $hash_map{$precursor}{$tag} } ){
		for my $end ( keys %{ $hash_map{$precursor}{$tag}{$beg} } ){
			
			my $bd = $beg - $mbeg;
			my $ed = $end - $mend;
			
			my $count = $hash_tag_count{$tag};	
			my $mis = $hash_map{$precursor}{$tag}{$beg}{$end};
			
			if( $bd >= $BEG_DIF && $ed <= $END_DIF ){
			
				if( $bd ==0 && $ed == 0 && MyMath::is_numeric($mis) && $mis == 0 ){
					unless(exists $hash_mature_tag_same{$mature}{$tag}){
						$hash_mature_tag_same{$mature}{$tag} = $count;
					} else {
						$hash_mature_tag_same{$mature}{$tag}+= $count;
					}
				}
												
				unless(exists $hash_mature_tag->{$mature}{$tag}){
					$hash_mature_tag->{$mature}{$tag} = $count;
				} else {
					$hash_mature_tag->{$mature}{$tag}+= $count;
				}
				
				unless(exists $hash_profile{$precursor}{$mature}{$tag}){
					$hash_profile{$precursor}{$mature}{$tag} = "$bd#$ed#$mis";
				} else {
					print STDERR "Error:($precursor,$mature,$tag) repeat\n";
					die;
				}
				
				unless(exists $hash_precursor_count{$precursor}){
					$hash_precursor_count{$precursor} = $count;
				} else {
					$hash_precursor_count{$precursor}+= $count;
				}								
			}	
										
		}}}
	}}
				
	

	# ************************************************* #
	
	
	for my $mature (sort keys %$hash_mature_tag ){
		for my $tag ( sort {$hash_mature_tag->{$mature}{$b} <=> $hash_mature_tag->{$mature}{$a} } 
			keys %{ $hash_mature_tag->{$mature} } ){
			$hash_parse->{$mature}{most_count} = $hash_mature_tag->{$mature}{$tag};
			$hash_parse->{$mature}{most_seq} = $hash_tag{$tag};
			last;
		}
	}
		

	for my $mature (sort keys %$hash_mature_tag ){
		my $tot_count = 0;
		for my $tag ( keys %{ $hash_mature_tag->{$mature} } ){
			$tot_count += $hash_mature_tag->{$mature}{$tag};
		}
		$hash_parse->{$mature}{total} = $tot_count;
	}
	

	for my $mature (sort keys %hash_mature_tag_same ){
		my $tot_count = 0;
		for my $tag ( keys %{ $hash_mature_tag_same{$mature} } ){
			$tot_count += $hash_mature_tag_same{$mature}{$tag};
		}
		$hash_parse->{$mature}{same} = $tot_count;
	}
	
	
	# ************************************************* #


	open PROFILE,">$outfile_profile" or die;	
	for my $precursor ( sort keys %hash_profile ){
		
		my $mfe = MyMath::round($hash_struct{$precursor}{mfe},2);
		my $struct = $hash_struct{$precursor}{struct};
		my $precursor_lng = length($hash_precursor{$precursor});
		my $precursor_seq = uc( $hash_precursor{$precursor} );
		$precursor_seq =~ tr/T/U/;
		
		print PROFILE ">$precursor Length:$precursor_lng(nt) MFE:$mfe(kcal/mol)\n";
		print PROFILE "$precursor_seq\n$struct\n";
		
		for my $mature ( sort keys %{ $hash_profile{$precursor} } ){
			
			my $mirbase_count = 0;
			if( exists $hash_parse->{$mature}{same}){
				$mirbase_count = $hash_parse->{$mature}{same};
			}
			
			my $most_count = $hash_parse->{$mature}{most_count};
			my $total_count = $hash_parse->{$mature}{total};
		
					
			my ($beg,$end) = split/:/,$hash_precursor_mature{$precursor}{$mature};	
			my $mature_seq = uc( $hash_mature{$mature} );
			$mature_seq =~ tr/T/U/;
			
		
			my $str = "*" x (length $precursor_seq);
			substr($str,$beg-1,length($hash_mature{$mature})) = $mature_seq;				
			print PROFILE "$str\t$mature\tTotal_count:$total_count\t";
			print PROFILE "Most_count:$most_count\tmiRBase_count:$mirbase_count\n";	
							
			for my $tag ( sort {$hash_mature_tag->{$mature}{$b} <=> $hash_mature_tag->{$mature}{$a}} 
				keys %{ $hash_mature_tag->{$mature} }){				
				if(exists $hash_profile{$precursor}{$mature}{$tag}){
					my ($bd,$ed,$mis) = split/#/,$hash_profile{$precursor}{$mature}{$tag};
					my $normalized_multimapping_count = $hash_mature_tag->{$mature}{$tag};
					my $nt = int($normalized_multimapping_count/$hash_tag_count{$tag});
					
					my $tag_seq = uc( $hash_tag{$tag} );
					$tag_seq =~ tr/T/U/;
		
					my $tag_beg	= $beg + $bd;
					my $str = "-" x (length $precursor_seq);
					substr($str,$tag_beg-1,length($hash_tag{$tag})) = $tag_seq;	
					print PROFILE "$str\t$tag\t$hash_tag_count{$tag}(_x$nt)\t$mis\n";
				} 	
			}		
		}
		print PROFILE "//\n";
	}
	close PROFILE;


	# ************************************************* #		
	
}
		
	







# =============================
# Function: parse the gff3 file from miRBase database
# Input: a gff3 file from miRBase database
# 		 reference of hash for parsed result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub gff3_parser {
	
	my $infile		= shift or die;
	my $hash_gff	= shift or die;

	unless(-e $infile){
		print STDERR "Error:$infile not exists\n";
		die;
	}
	
	
	my %hash_all = ();
	my %hash_mature = ();

	open IN,"<$infile" or die;
	while(defined(my $line=<IN>)){
		chomp($line);
		next if $line=~/^\s*$|^#/;
		
		$line =~ s/^\s*//g;
		$line =~ s/\s*$//g;
			
		if($line=~/^(\S+)\s+\.\s+(\S+)\s+(\d+)\s+(\d+)\s+\.\s+(\S+)\s+\.\s+(\S+)/){
			my $chrom 	= $1;
			my $type 	= $2;
			my $beg		= $3;
			my $end		= $4;
			my $strand	= $5;
			
			my $str = $6;
			$str =~ s/ID=//ig;
			$str =~ s/ID=//ig;
			$str =~ s/accession_number=//ig;
			$str =~ s/Name=//ig;
			$str =~ s/^\s*//g;
			$str =~ s/\s*$//g;
			
			my @tmp = split/;/,$str;
			my $id 	= shift @tmp;
			my $an 	= shift @tmp;
			my $name= shift @tmp;
			my $source = shift @tmp;
			
			unless(exists $hash_all{$id}{$chrom}{$strand}){
				$hash_all{$id}{$chrom}{$strand} = "$name:$beg:$end";
			} else {
				warn "$line repeat\n";
			}
			
			if(defined $source){
				$source =~ s/derives_from=//ig;
				unless(exists $hash_mature{$name}{$source}){
					$hash_mature{$name}{$source} = "$id:$chrom:$beg:$end:$strand";
				} else {
					print STDERR "$id repeat\n";
					die;
				}
			}
			
		} else {
			warn "$line format error!\n";
		}
	}
	close IN;
	
	
	for my $name ( keys %hash_mature ){
		for my $source ( keys %{ $hash_mature{$name} } ){
			
			my ($id,$chrom,$mb,$me,$strand) = split/:/,$hash_mature{$name}{$source};
			
			if( exists $hash_all{$source}{$chrom}{$strand} ){
				
				my ($pname,$pb,$pe) = split/:/,$hash_all{$source}{$chrom}{$strand};
				($mb,$me) = ($me,$mb) if $strand eq "-";
				($pb,$pe) = ($pe,$pb) if $strand eq "-";
				my $bd = abs($mb - $pb) + 1;
				my $ed = abs($me - $pb) + 1;
			
				
				unless(exists $hash_gff->{$pname}{$name}){
					$hash_gff->{$pname}{$name} = "$bd:$ed";
				} else {
					print STDERR "Error: ($pname,$name) repeat\n";
					die;
				}
				
			} else {
				
				print STDERR "Error:precurosr not exists ($name,$source)\n";
				die;
				
			}
		}
	}
	
}
		
		
		
# =============================
# Function: map mature miRNA into pre-miRNAs and 
# 			find the position of mature on his pre-miRNA
# Input: mature fasta from miRBase
# 		 hairpin fasta from miRBase
#		 gff3 annotation file from miRBase
#		 mapped output from Bowtie
#		 outfile for profile
#		 reference of hash for parsed result
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub mature2precursor {

	my $hash_precursor_mature = shift or die;
	my $bowtie_path		= shift or die;
	my $infile_mature	= shift or die;
	my $infile_hairpin 	= shift or die;

	
	my $dest_dir;
	my $infile_gff3;
	while (@_) {
		my $argument = shift @_;
		if ($argument=~/gff3/i) {$infile_gff3=shift @_}
		if ($argument=~/dest_dir/i) {$dest_dir=shift @_}
	}
	
	
	unless(-e $infile_mature){
		print STDERR "Error:$infile_mature not exists\n";
		die;
	}
	
	unless(-e $infile_hairpin){
		print STDERR "Error:$infile_hairpin not exists\n";
		die;
	}

	my @reads = ();
	push(@reads,$infile_mature);
	
	my $cmd = "bowtie -p 2 -t -f -v 0 -a --best --strata --norc";
	if( defined $dest_dir ){
		Bowtie::mapper($infile_hairpin,\@reads,$cmd,$bowtie_path,
		'analysis'=>0,'dest_dir'=>$dest_dir,'del_index'=>1);
	} else {
		Bowtie::mapper($infile_hairpin,\@reads,$cmd,
		$bowtie_path,'analysis'=>0,'del_index'=>1);
	}
	
	
	my @suffix = qw(.fa .txt);
	my $dirname = dirname($infile_mature);
	my $ref_basename  = basename($infile_hairpin,@suffix);
	my $read_basename = basename($infile_mature,@suffix);
	my $mapping_file  = "$dest_dir/$ref_basename.$read_basename.mapping.txt";
	
	my %hash_map = ();
	Bowtie::parser4($mapping_file,\%hash_map);
	
	
	my %hash_gff = ();
	if ((defined $infile_gff3) and (-e $infile_gff3)){
		&gff3_parser($infile_gff3,\%hash_gff);
	} 
		
	
	
	for my $precursor ( sort keys %hash_map ){
	for my $mature ( sort keys %{ $hash_map{$precursor} } ){
	for my $beg ( sort keys %{ $hash_map{$precursor}{$mature} } ){
	for my $end ( sort keys %{ $hash_map{$precursor}{$mature}{$beg} } ){
			
		my $tmp_mature = uc( $mature );
		
		$tmp_mature =~ s/^\s*//g;
		$tmp_mature =~ s/\s*$//g;
		$tmp_mature =~ s/-3P$//ig;
		$tmp_mature =~ s/-5P$//ig;
		
		my $tmp_precursor = uc( $precursor );
		$tmp_precursor =~ s/^\s*//g;
		$tmp_precursor =~ s/\s*$//g;
		
		
		if( $tmp_precursor =~ /^$tmp_mature\b/ ){
			unless(exists $hash_precursor_mature->{$precursor}{$mature}){
				$hash_precursor_mature->{$precursor}{$mature} = "$beg:$end";
			} else {
				if( exists $hash_gff{$precursor}{$mature} ){
					$hash_precursor_mature->{$precursor}{$mature} = $hash_gff{$precursor}{$mature};
				}
			}
		}
		
	}}}}

}
	
	
	

# =============================
# Function: parse the output structure of RNAfold
# Input: a RNAfold structure file 
#		 (output) Reference of hash for structure
# Author: Yuangen Yao
# Date: 2013-2-20
# =============================

sub parse_struct {
	
	my $infile_struct = shift or die;
	my $hash_struct	= shift or die;
	
	unless(-e $infile_struct){
		print STDERR "Error:$infile_struct not exists\n";
		die;
	}
	
	open IN,"<$infile_struct" or die;
	
	my $id = "";
	my %hash_tmp = ();
	
	while( defined(my $line = <IN> ) ){
		chomp( $line );
		next if $line =~ /^\s*$/;	
			
		if( $line =~/>/ ){
			$id = substr($line,1);
		} elsif ( $line =~ /[ACGUTN]+/i ) {
			
			$line = uc( $line );
			$hash_tmp{$id}{seq} = $line;
				
		} elsif ( $line =~ /\.|\(|\)/  ){				
			
			my @tmp = split /\s+\(/,$line;	
			$hash_tmp{$id}{struct} = $tmp[0];
			
			my $mfe = $tmp[1];
			$mfe =~ s/\(|\)//g;			
			$hash_tmp{$id}{mfe}= $mfe;
			
		}
	}
	
	close IN;
	
	
	for my $id ( keys %hash_tmp ){
		
		my $mfe = $hash_tmp{$id}{mfe};
		my $seq = $hash_tmp{$id}{seq};
		my $struct = $hash_tmp{$id}{struct};
		
		unless( $struct =~ /\(/ ) {
			next;
		}
		
		unless( $struct =~ /\)/ ) {
			next;
		}
		
		if (length($struct) != length($seq)){
			next;
		}
		
		unless( MyMath::is_numeric($mfe) ) {
			next;
		}
		
		$hash_struct->{$id}{mfe} = $mfe;
		$hash_struct->{$id}{seq} = $seq;
		$hash_struct->{$id}{struct} = $struct;
		
	}

}

################################################################
1;		
	